@@ -2,7 +2,7 @@
 /* Enlightenment - The Window Manager that dares to do what others don't     */
 /*****************************************************************************/
 /*
-  Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+  Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
   
   Permission is hereby granted, free of charge, to any person obtaining a copy
   of this software and associated documentation files (the "Software"), to
@@ -33,10 +33,6 @@
 #include <X11/cursorfont.h>
 #include <X11/extensions/shape.h>
 #include <X11/extensions/XShm.h>
-#include <Imlib.h>
-#if USE_FNLIB
-#include <Fnlib.h>
-#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -55,14 +51,17 @@
 #include <sys/resource.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
-#ifdef HAVE_FREETYPE1_FREETYPE_FREETYPE_H
-#include <freetype1/freetype/freetype.h>
-#elif defined(HAVE_FREETYPE_FREETYPE_H)
-#include <freetype/freetype.h>
-#else
-#include <freetype.h>
-#endif
 
+#include <Imlib2.h>
+
+#define EAllocColor(pxc) \
+	XAllocColor(disp, VRoot.cmap, pxc)
+
+#if HAVE___ATTRIBUTE__
+#define __UNUSED__ __attribute__((unused))
+#else
+#define __UNUSED__
+#endif
 
 #define FILEPATH_LEN_MAX 4096
 
@@ -70,7 +69,6 @@
 #define DEFAULT_LINKCOLOR_G 50
 #define DEFAULT_LINKCOLOR_B 160
 
-#define TT_VALID( handle )  ( ( handle ).z != NULL )
 #ifndef MAX
 #define MAX(a,b)  ((a)>(b)?(a):(b))
 #endif
@@ -81,163 +79,127 @@
 #define ENCOING_ISO_8859_3 2
 #define ENCOING_ISO_8859_4 3
 
-#define Esetenv(var, val, overwrite) \
-{ \
-  static char envvar[1024]; \
-  sprintf(envvar, "%500s=%500s", var, val);\
-  putenv(envvar);\
-}
+#if defined(HAVE_WCTYPE_H) && defined(HAVE_WCHAR_T) && !defined(__FreeBSD__)
+#define USE_WORD_MB
+#endif
 
-typedef struct _efont
-  {
-     TT_Engine           engine;
-     TT_Face             face;
-     TT_Instance         instance;
-     TT_Face_Properties  properties;
-     int                 num_glyph;
-     TT_Glyph           *glyphs;
-     TT_Raster_Map     **glyphs_cached;
-     TT_CharMap          char_map;
-     int                 max_descent;
-     int                 max_ascent;
-  }
-Efont;
+typedef struct _efont Efont;
+
+typedef struct _root
+{
+   Window              win;
+   Visual             *vis;
+   unsigned int        depth;
+   Colormap            cmap;
+   int                 scr;
+   unsigned int        w, h;
+}
+Root;
 
 typedef struct _textstate
-  {
-     char               *fontname;
-#if USE_FNLIB
-     FnlibStyle          style;
-     FnlibFont          *font;
-#endif
-     ImlibColor          fg_col;
-     ImlibColor          bg_col;
-     int                 effect;
-     Efont              *efont;
-     XFontStruct        *xfont;
-     XFontSet            xfontset;
-     int                 xfontset_ascent;
-     int                 height;
-  }
+{
+   char               *fontname;
+   XColor              fg_col;
+   XColor              bg_col;
+   int                 effect;
+   Efont              *efont;
+   XFontStruct        *xfont;
+   XFontSet            xfontset;
+   int                 xfontset_ascent;
+   int                 height;
+}
 TextState;
 
 typedef enum _type
-  {
-     IMG,
-     BR,
-     FONT,
-     P,
-     TEXT,
-     PAGE
-  }
+{
+   IMG,
+   BR,
+   FONT,
+   P,
+   TEXT,
+   PAGE
+}
 Type;
 
 typedef struct _img
-  {
-     char               *src;
-     char               *src2;
-     char               *src3;
-     int                 x, y;
-     char               *link;
-     int                 w, h;
-  }
+{
+   char               *src;
+   char               *src2;
+   char               *src3;
+   int                 x, y;
+   char               *link;
+   int                 w, h;
+}
 Img_;
 
 typedef struct _font
-  {
-     char               *face;
-     int                 r, g, b;
-  }
+{
+   char               *face;
+   int                 r, g, b;
+}
 Font_;
 
 typedef struct _p
-  {
-     float               align;
-  }
+{
+   float               align;
+}
 P_;
 
 typedef struct _object
-  {
-     Type                type;
-     void               *object;
-  }
+{
+   Type                type;
+   void               *object;
+}
 Object;
 
 typedef struct _page
-  {
-     char               *name;
-     int                 count;
-     Object             *obj;
-     int                 columns;
-     int                 padding;
-     int                 linkr, linkg, linkb;
-     char               *background;
-  }
+{
+   char               *name;
+   int                 count;
+   Object             *obj;
+   int                 columns;
+   int                 padding;
+   int                 linkr, linkg, linkb;
+   char               *background;
+}
 Page;
 
 typedef struct _link
-  {
-     char               *name;
-     int                 x, y, w, h;
-     struct _link       *next;
-  }
+{
+   char               *name;
+   int                 x, y, w, h;
+   struct _link       *next;
+}
 Link;
 
-void                Efont_extents(Efont * f, char *text,
+void                ESetColor(XColor * pxc, int r, int g, int b);
+void                EGetColor(XColor * pxc, int *pr, int *pg, int *pb);
+
+void                Efont_extents(Efont * f, const char *text,
 				  int *font_ascent_return,
 				  int *font_descent_return, int *width_return,
 				  int *max_ascent_return,
 				  int *max_descent_return,
 				  int *lbearing_return, int *rbearing_return);
-Efont              *Efont_load(char *file, int size);
+Efont              *Efont_load(const char *file, int size);
 void                Efont_free(Efont * f);
 void                EFont_draw_string(Display * disp, Drawable win, GC gc,
-				      int x, int y, char *text,
+				      int x, int y, const char *text,
 				      Efont * font, Visual * vis, Colormap cm);
 
-char              **TextGetLines(char *text, int *count);
 void                TextStateLoadFont(TextState * ts);
-void                TextSize(TextState * ts, char *text,
+void                TextSize(TextState * ts, const char *text,
 			     int *width, int *height, int fsize);
 void                TextDraw(TextState * ts, Window win, char *text,
 			     int x, int y, int w, int h, int fsize,
 			     int justification);
 
-char               *FileExtension(char *file);
-void                md(char *s);
-int                 exists(char *s);
-void                mkdirs(char *s);
-int                 isfile(char *s);
-int                 isdir(char *s);
-char              **ls(char *dir, int *num);
 void                freestrlist(char **l, int num);
-void                rm(char *s);
-void                mv(char *s, char *ss);
-void                cp(char *s, char *ss);
-time_t              moddate(char *s);
-int                 filesize(char *s);
-int                 fileinode(char *s);
-int                 filedev(char *s);
-void                cd(char *s);
-char               *cwd(void);
-int                 permissions(char *s);
-int                 owner(char *s);
-int                 group(char *s);
-char               *username(int uid);
-char               *homedir(int uid);
-char               *usershell(int uid);
-char               *atword(char *s, int num);
-char               *atchar(char *s, char c);
-char               *getword(char *s, int num);
 void                word(char *s, int num, char *wd);
+
+#ifdef USE_WORD_MB
 void                word_mb(char *s, int num, char *wd, int *spaceflag);
-int                 canread(char *s);
-int                 canwrite(char *s);
-int                 canexec(char *s);
-char               *fileof(char *s);
-char               *fullfileof(char *s);
-char               *pathtoexec(char *file);
-char               *pathtofile(char *file);
+#endif
 int                 findLocalizedFile(char *fname);
 
 void                AddPage(Object * obj);
@@ -252,8 +214,10 @@ void                GetLinkColors(int page_num, int *r, int *g, int *b);
 Link               *RenderPage(Window win, int page_num, int w, int h);
 
 extern Display     *disp;
-extern ImlibData   *pImlibData;
-#if USE_FNLIB
-extern FnlibData   *pFnlibData;
-#endif
+extern Root         VRoot;
 extern char        *docdir;
+
+#define Emalloc malloc
+#define Efree   free
+
+#define Esnprintf snprintf
