@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
@@ -93,10 +93,8 @@ AddObject(Object * obj)
    page[num_pages - 1].obj =
       realloc(page[num_pages - 1].obj,
 	      sizeof(Object) * (page[num_pages - 1].count));
-   page[num_pages - 1].obj[page[num_pages - 1].count - 1].type =
-      obj->type;
-   page[num_pages - 1].obj[page[num_pages - 1].count - 1].object =
-      obj->object;
+   page[num_pages - 1].obj[page[num_pages - 1].count - 1].type = obj->type;
+   page[num_pages - 1].obj[page[num_pages - 1].count - 1].object = obj->object;
 }
 
 void
@@ -515,23 +513,12 @@ RenderPage(Window win, int page_num, int w, int h)
    int                 x, y;
    int                 justification = 0;
    int                 firstp = 1;
-   ImlibImage         *im;
+   Imlib_Image        *im;
    int                 wastext = 0;
 
    ts.fontname = NULL;
-#if USE_FNLIB
-   ts.style.orientation = FONT_TO_RIGHT;
-   ts.style.mode = MODE_WRAP_WORD;
-   ts.style.justification = 0;
-   ts.style.spacing = 0;
-   ts.font = NULL;
-#endif
-   ts.fg_col.r = 0;
-   ts.fg_col.g = 0;
-   ts.fg_col.b = 0;
-   ts.bg_col.r = 0;
-   ts.bg_col.g = 0;
-   ts.bg_col.b = 0;
+   ESetColor(&ts.fg_col, 0, 0, 0);
+   ESetColor(&ts.bg_col, 0, 0, 0);
    ts.effect = 0;
    ts.efont = NULL;
    ts.xfont = NULL;
@@ -549,11 +536,13 @@ RenderPage(Window win, int page_num, int w, int h)
 
 	sprintf(tmp, "%s/%s", docdir, pg->background);
 	findLocalizedFile(tmp);
-	im = Imlib_load_image(pImlibData, tmp);
+	im = imlib_load_image(tmp);
 	if (im)
 	  {
-	     Imlib_paste_image(pImlibData, im, win, 0, 0, w, h);
-	     Imlib_destroy_image(pImlibData, im);
+	     imlib_context_set_image(im);
+	     imlib_context_set_drawable(win);
+	     imlib_render_image_on_drawable_at_size(0, 0, w, h);
+	     imlib_free_image();
 	  }
      }
    for (i = 0; i < pg->count; i++)
@@ -563,7 +552,7 @@ RenderPage(Window win, int page_num, int w, int h)
 	Font_              *fn;
 	P_                 *p;
 	int                 wc, eol, eot;
-	int                 link = 0, lx, lw;
+	int                 islink = 0, lx, lw;
 
 	switch (pg->obj[i].type)
 	  {
@@ -574,14 +563,16 @@ RenderPage(Window win, int page_num, int w, int h)
 		  char                tmp[4096];
 
 		  sprintf(tmp, "%s/%s", docdir, img->src);
-		  im = Imlib_load_image(pImlibData, tmp);
+		  im = imlib_load_image(tmp);
 		  if (im)
 		    {
-		       img->w = im->rgb_width;
-		       img->h = im->rgb_height;
-		       Imlib_paste_image(pImlibData, im, win, img->x, img->y,
-					 im->rgb_width, im->rgb_height);
-		       Imlib_destroy_image(pImlibData, im);
+		       imlib_context_set_image(im);
+		       img->w = imlib_image_get_width();
+		       img->h = imlib_image_get_height();
+		       imlib_context_set_drawable(win);
+		       imlib_render_image_on_drawable_at_size(img->x, img->y,
+							      img->w, img->h);
+		       imlib_free_image();
 		    }
 		  if (img->link)
 		    {
@@ -606,21 +597,8 @@ RenderPage(Window win, int page_num, int w, int h)
 	  case FONT:
 	     fn = pg->obj[i].object;
 	     ts.fontname = NULL;
-#if USE_FNLIB
-	     ts.style.orientation = FONT_TO_RIGHT;
-	     ts.style.mode = MODE_WRAP_WORD;
-	     ts.style.justification = 0;
-	     ts.style.spacing = 0;
-	     if (ts.font)
-		Fnlib_free_font(pFnlibData, ts.font);
-	     ts.font = NULL;
-#endif
-	     ts.fg_col.r = 0;
-	     ts.fg_col.g = 0;
-	     ts.fg_col.b = 0;
-	     ts.bg_col.r = 0;
-	     ts.bg_col.g = 0;
-	     ts.bg_col.b = 0;
+	     ESetColor(&ts.fg_col, 0, 0, 0);
+	     ESetColor(&ts.bg_col, 0, 0, 0);
 	     ts.effect = 0;
 	     if (ts.efont)
 		Efont_free(ts.efont);
@@ -634,9 +612,7 @@ RenderPage(Window win, int page_num, int w, int h)
 	     ts.xfontset_ascent = 0;
 	     ts.height = 0;
 	     ts.fontname = fn->face;
-	     ts.fg_col.r = fn->r;
-	     ts.fg_col.g = fn->g;
-	     ts.fg_col.b = fn->b;
+	     ESetColor(&ts.fg_col, fn->r, fn->g, fn->b);
 	     TextStateLoadFont(&ts);
 	     break;
 	  case P:
@@ -665,83 +641,93 @@ RenderPage(Window win, int page_num, int w, int h)
 		  int                 sx, sy, ssx, ssy;
 		  char                link_txt[1024];
 		  char                link_link[1024];
-		  int                 spaceflag, oldwc=0;
+		  int                 spaceflag, oldwc = 0;
 
 		  wd[0] = 0;
-#ifdef HAVE_WCTYPE_H
-		  if ( MB_CUR_MAX > 1 )                 /* If multibyte locale,... */
-		    word_mb(txt, wc, wd, &spaceflag);
+#ifdef USE_WORD_MB
+		  if (MB_CUR_MAX > 1)	/* If multibyte locale,... */
+		     word_mb(txt, wc, wd, &spaceflag);
 		  else
 #endif
 		    {
-		      word(txt, wc, wd);
-		      spaceflag = 1;
+		       word(txt, wc, wd);
+		       spaceflag = 1;
 		    }
-		  if (!wd[0])	eol = 1;
+		  if (!wd[0])
+		     eol = 1;
 
 		  wc++;
 		  eot++;
 		  strcpy(ss, s);
-		  if ( (eot != 1) && spaceflag)
+		  if ((eot != 1) && spaceflag)
 		     strcat(s, " ");
 
 		  if (wd[0] == '_')
-		     {
-			link_txt[0] = '\0';
-			link_link[0] = '\0';
-			link = 1;
-			oldwc = wc;
-		 	TextSize(&ts, s, &lx, &th, 17);
-		     }
-
-		  if ( link == 1 )
 		    {
-		      if ( eol || ( (wd[0] != '_') && spaceflag ) )	/* if NO link tag, ... */
-			{
-			  link_txt[0] = '\0';
-			  link_link[0] = '\0';
-			  link = 0;
-			  wc = oldwc;
-#ifdef HAVE_WCTYPE_H
-			  if ( MB_CUR_MAX > 1 )
-			    word_mb(txt, wc - 1, wd, &spaceflag);
-			  else
+		       link_txt[0] = '\0';
+		       link_link[0] = '\0';
+		       islink = 1;
+		       oldwc = wc;
+		       TextSize(&ts, s, &lx, &th, 17);
+		    }
+
+		  if (islink == 1)
+		    {
+		       if (eol || ((wd[0] != '_') && spaceflag))	/* if NO link tag, ... */
+			 {
+			    link_txt[0] = '\0';
+			    link_link[0] = '\0';
+			    islink = 0;
+			    wc = oldwc;
+#ifdef USE_WORD_MB
+			    if (MB_CUR_MAX > 1)
+			       word_mb(txt, wc - 1, wd, &spaceflag);
+			    else
 #endif
-			    {
-			      word(txt, wc - 1, wd);
-			      spaceflag = 1;
-			    }
-			}
-		      else
-		        {
-			  int	k, linkflg;
+			      {
+				 word(txt, wc - 1, wd);
+				 spaceflag = 1;
+			      }
+			 }
+		       else
+			 {
+			    int                 k, linkflg;
 
-			  j = 0;
-			  linkflg = 0;
-			  if ( wd[0] == '_' )	{ j++; linkflg++; }
+			    j = 0;
+			    linkflg = 0;
+			    if (wd[0] == '_')
+			      {
+				 j++;
+				 linkflg++;
+			      }
 
-			  k = strlen( link_txt );
-			  for ( ; wd[j] != '(' && wd[j] != '\0'; j++, k++)
-			    {
-			      if (wd[j] == '_')	link_txt[k] = ' ';
-			      else		link_txt[k] = wd[j];
-			      if ( linkflg )	wd[ j - 1 ] = link_txt[k];
-			      else		wd[j]       = link_txt[k];
-			    }
-			  link_txt[k] = '\0';
-			  if ( linkflg )	wd[ j - 1 ] = '\0';
+			    k = strlen(link_txt);
+			    for (; wd[j] != '(' && wd[j] != '\0'; j++, k++)
+			      {
+				 if (wd[j] == '_')
+				    link_txt[k] = ' ';
+				 else
+				    link_txt[k] = wd[j];
+				 if (linkflg)
+				    wd[j - 1] = link_txt[k];
+				 else
+				    wd[j] = link_txt[k];
+			      }
+			    link_txt[k] = '\0';
+			    if (linkflg)
+			       wd[j - 1] = '\0';
 
-			  if ( wd[j] == '(' )
-			    {
-			      wd[j++] = '\0';
-			      strcpy( link_link, wd + j);
-			      link_link[ strlen(link_link) - 1 ] = '\0';
-			      strcpy( wd, link_txt );
-			      link = 2;
-			    }
-			  else
-			    continue;
-			}
+			    if (wd[j] == '(')
+			      {
+				 wd[j++] = '\0';
+				 strcpy(link_link, wd + j);
+				 link_link[strlen(link_link) - 1] = '\0';
+				 strcpy(wd, link_txt);
+				 islink = 2;
+			      }
+			    else
+			       continue;
+			 }
 		    }
 
 		  strcat(s, wd);
@@ -771,7 +757,8 @@ RenderPage(Window win, int page_num, int w, int h)
 					{
 					   if ((iix >= sx) && (iix <= ssx))
 					     {
-						if (((ix + iix) / 2) > ((sx + ssx) / 2))
+						if (((ix + iix) / 2) >
+						    ((sx + ssx) / 2))
 						   ssx = ix - 1;
 						else
 						   sx = iix + 1;
@@ -799,28 +786,30 @@ RenderPage(Window win, int page_num, int w, int h)
 		     txt_disp = s;
 		  if (((tw > xspace) || (eol)) && (strlen(txt_disp) > 0))
 		    {
-		       if ( txt_disp[strlen(txt_disp) - 1] == ' ' )
-		         txt_disp[strlen(txt_disp) - 1] = 0;
+		       if (txt_disp[strlen(txt_disp) - 1] == ' ')
+			  txt_disp[strlen(txt_disp) - 1] = 0;
 
 		       if ((eot == 1) && (tw > xspace))
 			 {
 			    char                p1[4096];
-			    int                 point = 0, cnt = 0, i, len;
+			    int                 point = 0, cnt = 0, ii, len;
 
 			    while (txt_disp[(point + cnt)])
 			      {
-				len = mblen( txt_disp + point + cnt, MB_CUR_MAX);
-				if ( len < 0 )
-				  {
-				     cnt++;
-				     continue;
-				  }
-				else
-				  for ( i = 0; i < len; i++, cnt++ )
-					p1[cnt] = txt_disp[point + cnt];
+				 len =
+				    mblen(txt_disp + point + cnt, MB_CUR_MAX);
+				 if (len < 0)
+				   {
+				      cnt++;
+				      continue;
+				   }
+				 else
+				    for (ii = 0; ii < len; ii++, cnt++)
+				       p1[cnt] = txt_disp[point + cnt];
 				 p1[cnt] = 0;
 				 TextSize(&ts, p1, &tw, &th, 17);
-				 if ((tw > xspace) || (!txt_disp[(point + cnt)]))
+				 if ((tw > xspace)
+				     || (!txt_disp[(point + cnt)]))
 				   {
 				      if (txt_disp[(point + cnt)])
 					{
@@ -836,9 +825,13 @@ RenderPage(Window win, int page_num, int w, int h)
 					}
 				      wastext = 1;
 				      TextDraw(&ts, win, p1, x + off, y,
-					    xspace, 99999, 17, justification);
+					       xspace, 99999, 17,
+					       justification);
 				      y += ts.height;
-				      if (y >= (h - (pg->padding + ts.height - (ts.height - ts.xfontset_ascent))))
+				      if (y >=
+					  (h -
+					   (pg->padding + ts.height -
+					    (ts.height - ts.xfontset_ascent))))
 					{
 					   y = pg->padding;
 					   x += col_w + pg->padding;
@@ -854,35 +847,50 @@ RenderPage(Window win, int page_num, int w, int h)
 					   if (pg->obj[j].type == IMG)
 					     {
 						img = pg->obj[j].object;
-						if ((img->w > 0) && (img->h > 0))
+						if ((img->w > 0)
+						    && (img->h > 0))
 						  {
 						     int                 ix,
-						                         iy,
-						                         iix,
-						                         iiy;
+							iy, iix, iiy;
 
 						     ix = img->x;
 						     iy = img->y;
 						     iix = img->x + img->w - 1;
 						     iiy = img->y + img->h - 1;
 
-						     if ((iy <= ssy) && (iiy >= sy))
+						     if ((iy <= ssy)
+							 && (iiy >= sy))
 						       {
-							  if ((ix >= sx) && (ix <= ssx))
+							  if ((ix >= sx)
+							      && (ix <= ssx))
 							    {
-							       if ((iix >= sx) && (iix <= ssx))
+							       if ((iix >= sx)
+								   && (iix <=
+								       ssx))
 								 {
-								    if (((ix + iix) / 2) > ((sx + ssx) / 2))
-								       ssx = ix - 1;
+								    if (((ix +
+									  iix) /
+									 2) >
+									((sx +
+									  ssx) /
+									 2))
+								       ssx =
+									  ix -
+									  1;
 								    else
-								       sx = iix + 1;
+								       sx =
+									  iix +
+									  1;
 								 }
 							       else
 								 {
-								    ssx = ix - 1;
+								    ssx =
+								       ix - 1;
 								 }
 							    }
-							  else if ((iix >= sx) && (iix <= ssx))
+							  else if ((iix >= sx)
+								   && (iix <=
+								       ssx))
 							    {
 							       sx = iix + 1;
 							    }
@@ -904,45 +912,40 @@ RenderPage(Window win, int page_num, int w, int h)
 			    wastext = 1;
 			    TextDraw(&ts, win, txt_disp, x + off, y,
 				     xspace, 99999, 17, justification);
-			    if ( link > 1 && !strcmp( wd, link_txt) )
+			    if (islink > 1 && !strcmp(wd, link_txt))
 			      {
-				 link = 0;
+				 islink = 0;
 				 link_link[0] = '\0';
 				 link_txt[0] = '\0';
 				 wc = oldwc - 1;
 			      }
 
-			    if (link > 1)
+			    if (islink > 1)
 			      {
 				 int                 rr, gg, bb;
-				 int                 r, g, b;
 				 int                 extra;
 				 GC                  gc;
 				 XGCValues           gcv;
 
 				 gc = XCreateGC(disp, win, 0, &gcv);
-				 rr = ts.fg_col.r;
-				 gg = ts.fg_col.g;
-				 bb = ts.fg_col.b;
-				 r = ts.fg_col.r = pg->linkr;
-				 g = ts.fg_col.g = pg->linkg;
-				 b = ts.fg_col.b = pg->linkb;
-				 XSetForeground(disp, gc,
-				      Imlib_best_color_match(pImlibData, &r, &g, &b));
+				 EGetColor(&ts.fg_col, &rr, &gg, &bb);
+				 ESetColor(&ts.fg_col, pg->linkr, pg->linkg,
+					   pg->linkb);
+				 EAllocColor(&ts.fg_col);
+				 XSetForeground(disp, gc, ts.fg_col.pixel);
 				 TextSize(&ts, txt_disp, &tw, &th, 17);
 				 extra = ((xspace - tw) * justification) >> 10;
-				 TextDraw(&ts, win, link_txt, x + off + lx + extra, y,
-					  99999, 99999, 17, 0);
+				 TextDraw(&ts, win, link_txt,
+					  x + off + lx + extra, y, 99999, 99999,
+					  17, 0);
 				 TextSize(&ts, link_txt, &lw, &th, 17);
 				 XDrawLine(disp, win, gc,
 					   x + off + lx + extra,
 					   y + ts.xfontset_ascent,
 					   x + off + lx + lw + extra,
 					   y + ts.xfontset_ascent);
-				 ts.fg_col.r = rr;
-				 ts.fg_col.g = gg;
-				 ts.fg_col.b = bb;
-				 link = 0;
+				 ESetColor(&ts.fg_col, rr, gg, bb);
+				 islink = 0;
 				 XFreeGC(disp, gc);
 				 {
 				    Link               *l;
@@ -960,7 +963,10 @@ RenderPage(Window win, int page_num, int w, int h)
 				 link_txt[0] = '\0';
 			      }
 			    y += ts.height;
-			    if (y >= (h - (pg->padding + ts.height - (ts.height - ts.xfontset_ascent))))
+			    if (y >=
+				(h -
+				 (pg->padding + ts.height -
+				  (ts.height - ts.xfontset_ascent))))
 			      {
 				 y = pg->padding;
 				 x += col_w + pg->padding;
@@ -977,17 +983,14 @@ RenderPage(Window win, int page_num, int w, int h)
 	  default:
 	     break;
 	  }
-	if (y >= (h - (pg->padding + ts.height - (ts.height - ts.xfontset_ascent))))
+	if (y >=
+	    (h - (pg->padding + ts.height - (ts.height - ts.xfontset_ascent))))
 	  {
 	     y = pg->padding;
 	     x += col_w + pg->padding;
 	  }
      }
 
-#if USE_FNLIB
-   if (ts.font)
-      Fnlib_free_font(pFnlibData, ts.font);
-#endif
    if (ts.efont)
       Efont_free(ts.efont);
    if (ts.xfont)
