@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  
  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
@@ -23,8 +23,8 @@
 
 #include "dox.h"
 
-char              **
-TextGetLines(char *text, int *count)
+static char       **
+TextGetLines(const char *text, int *count)
 {
    int                 i, j, k;
    char              **list = NULL;
@@ -57,54 +57,42 @@ TextGetLines(char *text, int *count)
 void
 TextStateLoadFont(TextState * ts)
 {
-#if USE_FNLIB
-   if ((ts->font) || (ts->efont) || (ts->xfont) || (ts->xfontset))
-#else
    if ((ts->efont) || (ts->xfont) || (ts->xfontset))
-#endif
       return;
+
    if (!ts->fontname)
       return;
-#if USE_FNLIB
-   if ((!ts->font) && (!ts->efont))
-      ts->font = Fnlib_load_font(pFnlibData, ts->fontname);
-#endif
-#if USE_FNLIB
-   if ((!ts->font) && (!ts->efont))
-#else
-   if (!ts->efont)
-#endif
-     {
-	char                s[4096], w[4046], *dup, *ss;
 
-	dup = NULL;
-	dup = strdup(ts->fontname);
-	ss = strchr(dup, '/');
+   if (!ts->efont)
+     {
+	char                s[4096], w[4046], *s2, *ss;
+
+	s2 = NULL;
+	s2 = strdup(ts->fontname);
+	ss = strchr(s2, '/');
 	if (ss)
 	  {
 	     *ss = ' ';
-	     word(dup, 1, w);
+	     word(s2, 1, w);
 	     sprintf(s, "%s/%s.ttf", docdir, w);
 	     findLocalizedFile(s);
-	     word(dup, 2, w);
+	     word(s2, 2, w);
 	     ts->efont = Efont_load(s, atoi(w));
 	     if (ts->efont)
 	       {
 		  int                 as, ds;
 
-		  Efont_extents(ts->efont, " ", &as, &ds, NULL, NULL, NULL, NULL, NULL);
+		  Efont_extents(ts->efont, " ", &as, &ds, NULL, NULL, NULL,
+				NULL, NULL);
 		  ts->xfontset_ascent = as;
 		  ts->height = as + ds;
 	       }
 	  }
-	if (dup)
-	   free(dup);
+	if (s2)
+	   free(s2);
      }
-#if USE_FNLIB
-   if ((!ts->font) && (!ts->efont))
-#else
+
    if (!ts->efont)
-#endif
      {
 	if ((!ts->xfont) && (strchr(ts->fontname, ',') == NULL))
 	  {
@@ -145,38 +133,23 @@ TextStateLoadFont(TextState * ts)
 }
 
 void
-TextSize(TextState * ts, char *text,
-	 int *width, int *height, int fsize)
+TextSize(TextState * ts, const char *text, int *width, int *height, int fsize)
 {
    char              **lines;
    int                 i, num_lines;
 
    *width = 0;
    *height = 0;
+
    lines = TextGetLines(text, &num_lines);
    if (!lines)
       return;
+
    if (!ts)
       return;
-   TextStateLoadFont(ts);
-#if USE_FNLIB
-   if (ts->font)
-     {
-	for (i = 0; i < num_lines; i++)
-	  {
-	     int                 high, wid, dummy;
 
-	     Fnlib_measure(pFnlibData, ts->font, 0, 0, 999999, 999999,
-			   0, 0, fsize, &ts->style, (unsigned char *)lines[i],
-			   0, 0, &dummy, &dummy, &wid, &high, &dummy,
-			   &dummy, &dummy, &dummy);
-	     *height += high;
-	     if (wid > *width)
-		*width = wid;
-	  }
-     }
-   else
-#endif
+   TextStateLoadFont(ts);
+
    if (ts->efont)
      {
 	for (i = 0; i < num_lines; i++)
@@ -196,7 +169,8 @@ TextSize(TextState * ts, char *text,
 	  {
 	     XRectangle          ret1, ret2;
 
-	     XmbTextExtents(ts->xfontset, lines[i], strlen(lines[i]), &ret1, &ret2);
+	     XmbTextExtents(ts->xfontset, lines[i], strlen(lines[i]), &ret1,
+			    &ret2);
 	     *height += ret2.height;
 	     if (ret2.width > *width)
 		*width = ret2.width;
@@ -230,52 +204,33 @@ TextSize(TextState * ts, char *text,
      }
    freestrlist(lines, num_lines);
    return;
+   fsize = 0;
 }
 
 void
 TextDraw(TextState * ts, Window win, char *text,
-	 int x, int y, int w, int h, int fsize,
-	 int justification)
+	 int x, int y, int w, int h, int fsize, int justification)
 {
    char              **lines;
    int                 i, num_lines;
    int                 xx, yy;
    XGCValues           gcv;
    static GC           gc = 0;
-   int                 r, g, b;
 
    lines = TextGetLines(text, &num_lines);
    if (!lines)
       return;
+
    if (!ts)
       return;
+
    TextStateLoadFont(ts);
    xx = x;
    yy = y;
+
    if (!gc)
       gc = XCreateGC(disp, win, 0, &gcv);
-#if USE_FNLIB
-   if (ts->font)
-     {
-	for (i = 0; i < num_lines; i++)
-	  {
-	     int                 high, wid, dummy;
 
-	     Fnlib_measure(pFnlibData, ts->font, 0, 0, 999999, 999999,
-			   0, 0, fsize, &ts->style, (unsigned char *)lines[i],
-			   0, 0, &dummy, &dummy, &wid, &high, &dummy,
-			   &dummy, &dummy, &dummy);
-	     if ((ts->style.orientation == FONT_TO_UP) ||
-		 (ts->style.orientation == FONT_TO_DOWN))
-		fsize = w;
-	     xx = x + (((w - wid) * justification) >> 10);
-	     Fnlib_draw(pFnlibData, ts->font, win, 0, xx, yy, w, h,
-			0, 0, fsize, &ts->style, (unsigned char *)lines[i]);
-	     yy += high;
-	  }
-     }
-   else
-#endif
    if (ts->efont)
      {
 	for (i = 0; i < num_lines; i++)
@@ -289,40 +244,28 @@ TextDraw(TextState * ts, Window win, char *text,
 	     xx = x + (((w - wid) * justification) >> 10);
 	     if (ts->effect == 1)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  EFont_draw_string(disp, win, gc, xx + 1, yy + 1,
-				    lines[i], ts->efont, Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 	       }
 	     else if (ts->effect == 2)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  EFont_draw_string(disp, win, gc, xx - 1, yy,
-				    lines[i], ts->efont, Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 		  EFont_draw_string(disp, win, gc, xx + 1, yy,
-				    lines[i], ts->efont, Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 		  EFont_draw_string(disp, win, gc, xx, yy - 1,
-				    lines[i], ts->efont, Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 		  EFont_draw_string(disp, win, gc, xx, yy + 1,
-				    lines[i], ts->efont, Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 	       }
-	     r = ts->fg_col.r;
-	     g = ts->fg_col.g;
-	     b = ts->fg_col.b;
-	     XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+	     EAllocColor(&ts->fg_col);
+	     XSetForeground(disp, gc, ts->fg_col.pixel);
 	     EFont_draw_string(disp, win, gc, xx, yy,
-			       lines[i], ts->efont, Imlib_get_visual(pImlibData),
-			       Imlib_get_colormap(pImlibData));
+			       lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 	     yy += ascent + descent;
 	  }
      }
@@ -332,25 +275,22 @@ TextDraw(TextState * ts, Window win, char *text,
 	  {
 	     XRectangle          ret1, ret2;
 
-	     XmbTextExtents(ts->xfontset, lines[i], strlen(lines[i]), &ret1, &ret2);
+	     XmbTextExtents(ts->xfontset, lines[i], strlen(lines[i]), &ret1,
+			    &ret2);
 	     if (i == 0)
 		yy += ts->xfontset_ascent;
 	     xx = x + (((w - ret2.width) * justification) >> 10);
 	     if (ts->effect == 1)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XmbDrawString(disp, win, ts->xfontset, gc, xx + 1, yy + 1,
 				lines[i], strlen(lines[i]));
 	       }
 	     else if (ts->effect == 2)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XmbDrawString(disp, win, ts->xfontset, gc, xx - 1, yy,
 				lines[i], strlen(lines[i]));
 		  XmbDrawString(disp, win, ts->xfontset, gc, xx + 1, yy,
@@ -360,10 +300,8 @@ TextDraw(TextState * ts, Window win, char *text,
 		  XmbDrawString(disp, win, ts->xfontset, gc, xx, yy + 1,
 				lines[i], strlen(lines[i]));
 	       }
-	     r = ts->fg_col.r;
-	     g = ts->fg_col.g;
-	     b = ts->fg_col.b;
-	     XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+	     EAllocColor(&ts->fg_col);
+	     XSetForeground(disp, gc, ts->fg_col.pixel);
 	     XmbDrawString(disp, win, ts->xfontset, gc, xx, yy,
 			   lines[i], strlen(lines[i]));
 	     yy += ret2.height;
@@ -383,19 +321,15 @@ TextDraw(TextState * ts, Window win, char *text,
 	     xx = x + (((w - wid) * justification) >> 10);
 	     if (ts->effect == 1)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XDrawString(disp, win, gc, xx + 1, yy + 1,
 			      lines[i], strlen(lines[i]));
 	       }
 	     else if (ts->effect == 2)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XDrawString(disp, win, gc, xx - 1, yy,
 			      lines[i], strlen(lines[i]));
 		  XDrawString(disp, win, gc, xx + 1, yy,
@@ -405,12 +339,9 @@ TextDraw(TextState * ts, Window win, char *text,
 		  XDrawString(disp, win, gc, xx, yy + 1,
 			      lines[i], strlen(lines[i]));
 	       }
-	     r = ts->fg_col.r;
-	     g = ts->fg_col.g;
-	     b = ts->fg_col.b;
-	     XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
-	     XDrawString(disp, win, gc, xx, yy,
-			 lines[i], strlen(lines[i]));
+	     EAllocColor(&ts->fg_col);
+	     XSetForeground(disp, gc, ts->fg_col.pixel);
+	     XDrawString(disp, win, gc, xx, yy, lines[i], strlen(lines[i]));
 	     yy += ts->xfont->ascent + ts->xfont->descent;
 	  }
      }
@@ -428,19 +359,15 @@ TextDraw(TextState * ts, Window win, char *text,
 	     xx = x + (((w - wid) * justification) >> 10);
 	     if (ts->effect == 1)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XDrawString16(disp, win, gc, xx + 1, yy + 1,
 				(XChar2b *) lines[i], strlen(lines[i]) / 2);
 	       }
 	     else if (ts->effect == 2)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XDrawString16(disp, win, gc, xx - 1, yy,
 				(XChar2b *) lines[i], strlen(lines[i]) / 2);
 		  XDrawString16(disp, win, gc, xx + 1, yy,
@@ -450,10 +377,8 @@ TextDraw(TextState * ts, Window win, char *text,
 		  XDrawString16(disp, win, gc, xx, yy + 1,
 				(XChar2b *) lines[i], strlen(lines[i]) / 2);
 	       }
-	     r = ts->fg_col.r;
-	     g = ts->fg_col.g;
-	     b = ts->fg_col.b;
-	     XSetForeground(disp, gc, Imlib_best_color_match(pImlibData, &r, &g, &b));
+	     EAllocColor(&ts->fg_col);
+	     XSetForeground(disp, gc, ts->fg_col.pixel);
 	     XDrawString16(disp, win, gc, xx, yy,
 			   (XChar2b *) lines[i], strlen(lines[i]) / 2);
 	     yy += ts->xfont->ascent + ts->xfont->descent;
@@ -461,4 +386,5 @@ TextDraw(TextState * ts, Window win, char *text,
      }
    freestrlist(lines, num_lines);
    return;
+   h = fsize = 0;
 }
