@@ -735,6 +735,7 @@ typedef struct _border
    char                changes_shape;
    char                shadedir;
    unsigned int        ref_count;
+   ActionClass        *aclass;
 }
 Border;
 
@@ -748,6 +749,7 @@ typedef struct _ewinbit
    char                no_expose;
    char                left;
    ImageState         *is;
+   TextState          *ts;
 }
 EWinBit;
 
@@ -826,6 +828,7 @@ struct _ewin
    int                 desktop;
    Group             **groups;
    int                 num_groups;
+   char                visibility;
    char                docked;
    char                sticky;
    char                shown;
@@ -838,12 +841,6 @@ struct _ewin
    char                floating;
    char                shapedone;
    char                fixedpos;
-#if 0				/* ENABLE_GNOME - Not actually used */
-   int                 expanded_x;
-   int                 expanded_y;
-   int                 expanded_width;
-   int                 expanded_height;
-#endif
    char                ignorearrange;
    char                skiptask;
    char                skip_ext_pager;
@@ -873,6 +870,7 @@ struct _ewin
    } strut;
    struct
    {
+      unsigned            donthide:1;	/* Don't hide on show desktop */
       unsigned            vroot:1;	/* Virtual root window */
       unsigned int        opacity;
    } props;
@@ -881,6 +879,7 @@ struct _ewin
       unsigned            maximized_horz:1;
       unsigned            maximized_vert:1;
       unsigned            fullscreen:1;
+      unsigned            showingdesk:1;	/* Iconified by show desktop */
    } st;
    struct
    {
@@ -1103,6 +1102,10 @@ typedef struct
       char                set_xroot_info_on_root_window;
    } hints;
    struct
+   {
+      KeySym              left, right, up, down, escape, ret;
+   } menukeys;
+   struct
    {
       char                enable;
       char                zoom;
@@ -1119,6 +1122,7 @@ typedef struct
       char                manual;
       char                manual_mouse_pointer;
       char                ignore_struts;
+      char                raise_fullscreen;
    } place;
    struct
    {
@@ -1714,12 +1718,16 @@ const char         *EwinGetIconName(const EWin * ewin);
 int                 EwinIsOnScreen(EWin * ewin);
 int                 BorderWinpartIndex(EWin * ewin, Window win);
 
+void                EwinRememberPositionSet(EWin * ewin);
+void                EwinRememberPositionGet(EWin * ewin, int *px, int *py);
+void                EwinPropagateShapes(EWin * ewin);
 void                EwinChange(EWin * ewin, unsigned int flag);
 void                EwinChangesStart(EWin * ewin);
 void                EwinChangesProcess(EWin * ewin);
 
 void                EwinsEventsConfigure(int mode);
 void                EwinsSetFree(void);
+void                EwinsShowDesktop(int on);
 
 int                 BordersEventExpose(XEvent * ev);
 int                 BordersEventMouseDown(XEvent * ev);
@@ -1742,7 +1750,7 @@ void                ButtonHide(Button * b);
 void                ButtonToggle(Button * b);
 void                ButtonDraw(Button * b);
 void                ButtonDrawWithState(Button * b, int state);
-void                ButtonMoveToDesktop(Button * b, int num);
+void                ButtonMoveToDesktop(Button * b, int desk);
 void                ButtonMoveToCoord(Button * b, int x, int y);
 void                ButtonMoveRelative(Button * b, int dx, int dy);
 void                ButtonIncRefcount(Button * b);
@@ -1757,8 +1765,6 @@ ActionClass        *ButtonGetAClass(Button * b);
 int                 ButtonIsFixed(Button * b);
 int                 ButtonEmbedWindow(Button * ButtonToUse,
 				      Window WindowToEmbed);
-void                ButtonFindEmptySpotFor(Button * bt, const char *listname,
-					   char dirtomove);
 int                 ButtonsEventExpose(XEvent * ev);
 int                 ButtonsEventMouseDown(XEvent * ev);
 int                 ButtonsEventMouseUp(XEvent * ev);
@@ -1993,6 +1999,7 @@ void                EdgeHandleMotion(XEvent * ev);
 #define EDBUG_TYPE_SESSION   	134
 #define EDBUG_TYPE_SNAPS     	135
 #define EDBUG_TYPE_DESKS     	136
+#define EDBUG_TYPE_GRABS     	137
 int                 EventDebug(unsigned int type);
 #else
 #define             EventDebug(type) 0
@@ -2080,7 +2087,6 @@ char               *homedir(int uid);
 char               *usershell(int uid);
 const char         *atword(const char *s, int num);
 const char         *atchar(const char *s, char c);
-char               *getword(char *s, int num);
 void                word(const char *s, int num, char *wd);
 int                 canread(const char *s);
 int                 canwrite(const char *s);
@@ -2389,6 +2395,8 @@ void                EDirUserCacheSet(const char *d);
 char               *EDirUserCache(void);
 void                Quicksort(void **a, int l, int r,
 			      int (*CompareFunc) (void *d1, void *d2));
+void                ETimedLoopInit(int k1, int k2, int speed);
+int                 ETimedLoopNext(void);
 void __PRINTF__     Eprintf(const char *fmt, ...);
 
 /* moveresize.c */
