@@ -447,6 +447,11 @@ EventAclass(XEvent * ev, EWin * ewin, ActionClass * a)
 	mouse = 1;
 	break;
      case LeaveNotify:
+	/* If frame window, quit if pointer is still inside */
+	if (ewin && ev->xcrossing.window == ewin->win &&
+	    (ev->xcrossing.x >= 0 && ev->xcrossing.x < ewin->w &&
+	     ev->xcrossing.y >= 0 && ev->xcrossing.y < ewin->h))
+	   EDBUG_RETURN(0);
 	type = EVENT_MOUSE_LEAVE;
 	button = -1;
 	modifiers = ev->xcrossing.state & mask;
@@ -2380,24 +2385,31 @@ DoShade(EWin * ewin, const void *params, int nogroup)
 {
    EWin              **gwins = NULL;
    Group              *curr_group = NULL;
-   int                 i, num;
-   char                shaded;
+   int                 i, num, shade;
 
    EDBUG(6, "doShade");
 
    gwins = ListWinGroupMembersForEwin(ewin, ACTION_SHADE, nogroup, &num);
-   shaded = ewin->shaded;
+
+   shade = 0;
+   if (!params)
+      shade = !ewin->shaded;
+   else if (!strcmp(params, "on"))
+      shade = 1;
+   else if (!strcmp(params, "off"))
+      shade = 0;
+
    for (i = 0; i < num; i++)
      {
 	curr_group = EwinsInGroup(ewin, gwins[i]);
 	if (gwins[i]->shaded
-	    && ((curr_group && !curr_group->cfg.mirror) || shaded))
+	    && ((curr_group && !curr_group->cfg.mirror) || !shade))
 	  {
 	     SoundPlay("SOUND_UNSHADE");
 	     EwinUnShade(gwins[i]);
 	  }
 	else if (!gwins[i]->shaded
-		 && ((curr_group && !curr_group->cfg.mirror) || !shaded))
+		 && ((curr_group && !curr_group->cfg.mirror) || shade))
 	  {
 	     SoundPlay("SOUND_SHADE");
 	     EwinShade(gwins[i]);
@@ -2406,7 +2418,6 @@ DoShade(EWin * ewin, const void *params, int nogroup)
      }
    Efree(gwins);
    EDBUG_RETURN(0);
-   params = NULL;
 }
 
 static int
