@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -22,10 +22,7 @@
  */
 #include "E.h"
 
-int                 ArrangeAddToList(int **array, int current_size, int value);
-void                ArrangeSwapList(RectBox * list, int a, int b);
-
-int
+static int
 ArrangeAddToList(int **array, int current_size, int value)
 {
    int                 i, j;
@@ -47,7 +44,7 @@ ArrangeAddToList(int **array, int current_size, int value)
    EDBUG_RETURN(current_size + 1);
 }
 
-void
+static void
 ArrangeSwapList(RectBox * list, int a, int b)
 {
    RectBox             bb;
@@ -96,7 +93,7 @@ ArrangeRects(RectBox * fixed, int fixed_count, RectBox * floating,
      {
 	int                 xx1, yy1, xx2, yy2;
 
-	GetPointerScreenGeometry(&xx1, &yy1, &xx2, &yy2);
+	GetPointerScreenAvailableArea(&xx1, &yy1, &xx2, &yy2);
 	xx2 += xx1;
 	yy2 += yy1;
 	if (startx < xx1)
@@ -162,7 +159,7 @@ ArrangeRects(RectBox * fixed, int fixed_count, RectBox * floating,
       leftover = Emalloc(floating_count * sizeof(int));
 
    if (!xarray || !yarray || !filled || !spaces)
-      goto exit;
+      goto done;
 
 /* copy "fixed" rects into the sorted list */
    for (i = 0; i < fixed_count; i++)
@@ -500,9 +497,6 @@ ArrangeRects(RectBox * fixed, int fixed_count, RectBox * floating,
 		       spaces[num_spaces].y = yarray[y];
 		       spaces[num_spaces].w = xarray[x1] - xarray[x];
 		       spaces[num_spaces].h = yarray[y1] - yarray[y];
-		       if (spaces[num_spaces].w < floating[leftover[i]].w ||
-			   spaces[num_spaces].h < floating[leftover[i]].h)
-			  fitswin = 0;
 		       spaces[num_spaces].p = fitswin;
 		       num_spaces++;
 		    }
@@ -559,16 +553,16 @@ ArrangeRects(RectBox * fixed, int fixed_count, RectBox * floating,
    for (i = 0; i < num_sorted; i++)
      {
 	if ((sorted[i].x + sorted[i].w) > width)
-	   sorted[i].x = root.w - sorted[i].w;
+	   sorted[i].x = VRoot.w - sorted[i].w;
 	if ((sorted[i].y + sorted[i].h) > height)
-	   sorted[i].y = root.h - sorted[i].h;
+	   sorted[i].y = VRoot.h - sorted[i].h;
 	if (sorted[i].x < startx)
 	   sorted[i].x = startx;
 	if (sorted[i].y < starty)
 	   sorted[i].y = starty;
      }
 
- exit:
+ done:
    /* free up memory */
    if (xarray)
       Efree(xarray);
@@ -596,21 +590,22 @@ SnapEwin(EWin * ewin, int dx, int dy, int *new_dx, int *new_dy)
    if (!ewin)
       EDBUG_RETURN_;
 
-   if (!mode.snap)
+   if (!Conf.snap.enable)
      {
 	*new_dx = dx;
 	*new_dy = dy;
 	EDBUG_RETURN_;
      }
 
-   ScreenGetGeometry(ewin->x, ewin->y, &left_bound, &top_bound, &w, &h);
+   ScreenGetAvailableArea(ewin->shape_x, ewin->shape_y,
+			  &left_bound, &top_bound, &w, &h);
    right_bound = left_bound + w;
    bottom_bound = top_bound + h;
-   screen_snap_dist = mode.constrained ? (w + h) : mode.screen_snap_dist;
+   screen_snap_dist = Mode.constrained ? (w + h) : Conf.snap.screen_snap_dist;
 
    lst = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
-   gwins = ListWinGroupMembersForEwin(ewin, ACTION_MOVE, mode.nogroup
-				      || mode.swapmovemode, &gnum);
+   gwins = ListWinGroupMembersForEwin(ewin, ACTION_MOVE, Mode.nogroup
+				      || Mode.move.swap, &gnum);
    if (gwins)
      {
 	for (i = 0; i < gnum; i++)
@@ -623,155 +618,155 @@ SnapEwin(EWin * ewin, int dx, int dy, int *new_dx, int *new_dy)
 	  }
 	Efree(gwins);
      }
+
    odx = dx;
    ody = dy;
    if (dx < 0)
      {
-	if (IN_BELOW(ewin->x + dx, left_bound, screen_snap_dist)
-	    && (ewin->x >= left_bound))
+	if (IN_BELOW(ewin->shape_x + dx, left_bound, screen_snap_dist)
+	    && (ewin->shape_x >= left_bound))
 	  {
-	     dx = left_bound - ewin->x;
+	     dx = left_bound - ewin->shape_x;
 	  }
 	else if (lst)
 	  {
 	     for (i = 0; i < num; i++)
 	       {
-		  if (lst[i])
+		  if (lst[i] == NULL)
+		     continue;
+
+		  if (((ewin->desktop == lst[i]->desktop) ||
+		       (lst[i]->sticky)) && (!(lst[i]->floating)) &&
+		      (!(lst[i]->iconified)) && (!(lst[i]->ignorearrange)))
 		    {
-		       if (((ewin->desktop == lst[i]->desktop)
-			    || (lst[i]->sticky)) && (!(lst[i]->floating))
-			   && (!(lst[i]->iconified))
-			   && (!(lst[i]->ignorearrange)))
+		       if (IN_BELOW
+			   (ewin->shape_x + dx, lst[i]->x + lst[i]->w - 1,
+			    Conf.snap.edge_snap_dist)
+			   && SPANS_COMMON(ewin->shape_y, ewin->h, lst[i]->y,
+					   lst[i]->h)
+			   && (ewin->shape_x >= (lst[i]->x + lst[i]->w)))
 			 {
-			    if (IN_BELOW
-				(ewin->x + dx, lst[i]->x + lst[i]->w - 1,
-				 mode.edge_snap_dist)
-				&& SPANS_COMMON(ewin->y, ewin->h, lst[i]->y,
-						lst[i]->h)
-				&& (ewin->x >= (lst[i]->x + lst[i]->w)))
-			      {
-				 dx = (lst[i]->x + lst[i]->w) - ewin->x;
-				 break;
-			      }
+			    dx = (lst[i]->x + lst[i]->w) - ewin->shape_x;
+			    break;
 			 }
 		    }
 	       }
 	  }
-	if ((ewin->reqx - ewin->x) > 0)
+	if ((ewin->req_x - ewin->shape_x) > 0)
 	   dx = 0;
      }
    else if (dx > 0)
      {
-	if (IN_ABOVE(ewin->x + ewin->w + dx, right_bound, screen_snap_dist)
-	    && ((ewin->x + ewin->w) <= right_bound))
+	if (IN_ABOVE
+	    (ewin->shape_x + ewin->w + dx, right_bound, screen_snap_dist)
+	    && ((ewin->shape_x + ewin->w) <= right_bound))
 	  {
-	     dx = right_bound - (ewin->x + ewin->w);
+	     dx = right_bound - (ewin->shape_x + ewin->w);
 	  }
 	else if (lst)
 	  {
 	     for (i = 0; i < num; i++)
 	       {
-		  if (lst[i])
+		  if (lst[i] == NULL)
+		     continue;
+
+		  if (((ewin->desktop == lst[i]->desktop) ||
+		       (lst[i]->sticky)) && (!(lst[i]->floating)) &&
+		      (!(lst[i]->iconified)) && (!(lst[i]->ignorearrange)))
 		    {
-		       if (((ewin->desktop == lst[i]->desktop)
-			    || (lst[i]->sticky)) && (!(lst[i]->floating))
-			   && (!(lst[i]->iconified))
-			   && (!(lst[i]->ignorearrange)))
+		       if (IN_ABOVE(ewin->shape_x + ewin->w + dx - 1, lst[i]->x,
+				    Conf.snap.edge_snap_dist) &&
+			   SPANS_COMMON(ewin->shape_y, ewin->h, lst[i]->y,
+					lst[i]->h) &&
+			   ((ewin->shape_x + ewin->w) <= lst[i]->x))
 			 {
-			    if (IN_ABOVE
-				(ewin->x + ewin->w + dx - 1, lst[i]->x,
-				 mode.edge_snap_dist)
-				&& SPANS_COMMON(ewin->y, ewin->h, lst[i]->y,
-						lst[i]->h)
-				&& ((ewin->x + ewin->w) <= lst[i]->x))
-			      {
-				 dx = lst[i]->x - (ewin->x + ewin->w);
-				 break;
-			      }
+			    dx = lst[i]->x - (ewin->shape_x + ewin->w);
+			    break;
 			 }
 		    }
 	       }
 	  }
-	if ((ewin->reqx - ewin->x) < 0)
+	if ((ewin->req_x - ewin->shape_x) < 0)
 	   dx = 0;
      }
+
    if (dy < 0)
      {
-	if (IN_BELOW(ewin->y + dy, top_bound, screen_snap_dist)
-	    && (ewin->y >= top_bound))
+	if (IN_BELOW(ewin->shape_y + dy, top_bound, screen_snap_dist)
+	    && (ewin->shape_y >= top_bound))
 	  {
-	     dy = top_bound - ewin->y;
+	     dy = top_bound - ewin->shape_y;
 	  }
 	else if (lst)
 	  {
 	     for (i = 0; i < num; i++)
 	       {
-		  if (lst[i])
+		  if (lst[i] == NULL)
+		     continue;
+
+		  if (((ewin->desktop == lst[i]->desktop) ||
+		       (lst[i]->sticky)) && (!(lst[i]->floating)) &&
+		      (!(lst[i]->iconified)) && (!(lst[i]->ignorearrange)))
 		    {
-		       if (((ewin->desktop == lst[i]->desktop)
-			    || (lst[i]->sticky)) && (!(lst[i]->floating))
-			   && (!(lst[i]->iconified))
-			   && (!(lst[i]->ignorearrange)))
+		       if (IN_BELOW
+			   (ewin->shape_y + dy, lst[i]->y + lst[i]->h - 1,
+			    Conf.snap.edge_snap_dist)
+			   && SPANS_COMMON(ewin->shape_x, ewin->w, lst[i]->x,
+					   lst[i]->w)
+			   && (ewin->shape_y >= (lst[i]->y + lst[i]->h)))
 			 {
-			    if (IN_BELOW
-				(ewin->y + dy, lst[i]->y + lst[i]->h - 1,
-				 mode.edge_snap_dist)
-				&& SPANS_COMMON(ewin->x, ewin->w, lst[i]->x,
-						lst[i]->w)
-				&& (ewin->y >= (lst[i]->y + lst[i]->h)))
-			      {
-				 dy = (lst[i]->y + lst[i]->h) - ewin->y;
-				 break;
-			      }
+			    dy = (lst[i]->y + lst[i]->h) - ewin->shape_y;
+			    break;
 			 }
 		    }
 	       }
 	  }
-	if ((ewin->reqy - ewin->y) > 0)
+	if ((ewin->req_y - ewin->shape_y) > 0)
 	   dy = 0;
      }
    else if (dy > 0)
      {
-	if (IN_ABOVE(ewin->y + ewin->h + dy, bottom_bound, screen_snap_dist)
-	    && ((ewin->y + ewin->h) <= bottom_bound))
+	if (IN_ABOVE
+	    (ewin->shape_y + ewin->h + dy, bottom_bound, screen_snap_dist)
+	    && ((ewin->shape_y + ewin->h) <= bottom_bound))
 	  {
-	     dy = bottom_bound - (ewin->y + ewin->h);
+	     dy = bottom_bound - (ewin->shape_y + ewin->h);
 	  }
 	else if (lst)
 	  {
 	     for (i = 0; i < num; i++)
 	       {
-		  if (lst[i])
+		  if (lst[i] == NULL)
+		     continue;
+
+		  if (((ewin->desktop == lst[i]->desktop) ||
+		       (lst[i]->sticky)) && (!(lst[i]->floating)) &&
+		      (!(lst[i]->iconified)) && (!(lst[i]->ignorearrange)))
 		    {
-		       if (((ewin->desktop == lst[i]->desktop)
-			    || (lst[i]->sticky)) && (!(lst[i]->floating))
-			   && (!(lst[i]->iconified))
-			   && (!(lst[i]->ignorearrange)))
+		       if (IN_ABOVE(ewin->shape_y + ewin->h + dy - 1, lst[i]->y,
+				    Conf.snap.edge_snap_dist) &&
+			   SPANS_COMMON(ewin->shape_x, ewin->w, lst[i]->x,
+					lst[i]->w) &&
+			   ((ewin->shape_y + ewin->h) <= lst[i]->y))
 			 {
-			    if (IN_ABOVE
-				(ewin->y + ewin->h + dy - 1, lst[i]->y,
-				 mode.edge_snap_dist)
-				&& SPANS_COMMON(ewin->x, ewin->w, lst[i]->x,
-						lst[i]->w)
-				&& ((ewin->y + ewin->h) <= lst[i]->y))
-			      {
-				 dy = lst[i]->y - (ewin->y + ewin->h);
-				 break;
-			      }
+			    dy = lst[i]->y - (ewin->shape_y + ewin->h);
+			    break;
 			 }
 		    }
 	       }
 	  }
-	if ((ewin->reqy - ewin->y) < 0)
+	if ((ewin->req_y - ewin->shape_y) < 0)
 	   dy = 0;
      }
+
    if (lst)
       Efree(lst);
+
    if ((odx != dx) || (ody != dy))
      {
 	if (!last_res)
 	  {
-	     /* AUDIO_PLAY("SOUND_MOVE_RESIST"); */
+	     /* SoundPlay("SOUND_MOVE_RESIST"); */
 	     last_res = 1;
 	  }
      }
@@ -805,7 +800,7 @@ ArrangeEwinCentered(EWin * ewin, int focus)
    ArrangeEwinCenteredXY(ewin, &ewin->x, &ewin->y);
    MoveEwin(ewin, ewin->x, ewin->y);
    if (focus)
-      FocusToEWin(ewin);
+      FocusToEWin(ewin, FOCUS_SET);
 
    EDBUG_RETURN_;
 }
@@ -827,7 +822,7 @@ EWinIsOnViewport(EWin * ewin, int desktop)
 void
 ArrangeEwinXY(EWin * ewin, int *px, int *py)
 {
-   EWin              **lst;
+   EWin               *const *lst;
    Button            **blst;
    int                 i, j, num;
    RectBox            *fixed, *ret, newrect;
@@ -835,7 +830,7 @@ ArrangeEwinXY(EWin * ewin, int *px, int *py)
    fixed = NULL;
    ret = NULL;
 
-   lst = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
+   lst = EwinListGetAll(&num);
    if ((lst) && (num > 0))
      {
 	fixed = Emalloc(sizeof(RectBox) * num);
@@ -859,15 +854,15 @@ ArrangeEwinXY(EWin * ewin, int *px, int *py)
 		  fixed[j].w += fixed[j].x;
 		  fixed[j].x = 0;
 	       }
-	     if ((fixed[j].x + fixed[j].w) > root.w)
-		fixed[j].w = root.w - fixed[j].x;
+	     if ((fixed[j].x + fixed[j].w) > VRoot.w)
+		fixed[j].w = VRoot.w - fixed[j].x;
 	     if (fixed[j].y < 0)
 	       {
 		  fixed[j].h += fixed[j].y;
 		  fixed[j].y = 0;
 	       }
-	     if ((fixed[j].y + fixed[j].h) > root.h)
-		fixed[j].h = root.h - fixed[j].y;
+	     if ((fixed[j].y + fixed[j].h) > VRoot.h)
+		fixed[j].h = VRoot.h - fixed[j].y;
 	     if ((fixed[j].w <= 0) || (fixed[j].h <= 0))
 		continue;
 
@@ -883,36 +878,27 @@ ArrangeEwinXY(EWin * ewin, int *px, int *py)
 	     fixed = Erealloc(fixed, sizeof(RectBox) * (num + j));
 	     for (i = 0; i < num; i++)
 	       {
-		  Button             *b = blst[i];
-
-		  if (!b->visible || b->internal)
-		     continue;
-		  if (!b->sticky && (b->desktop != ewin->desktop))
+		  if (ButtonGetInfo(blst[i], &fixed[j], ewin->desktop))
 		     continue;
 
-		  fixed[j].data = NULL;
-		  fixed[j].x = b->x;
-		  fixed[j].y = b->y;
-		  fixed[j].w = b->w;
-		  fixed[j].h = b->h;
 		  if (fixed[j].x < 0)
 		    {
 		       fixed[j].w += fixed[j].x;
 		       fixed[j].x = 0;
 		    }
-		  if ((fixed[j].x + fixed[j].w) > root.w)
-		     fixed[j].w = root.w - fixed[j].x;
+		  if ((fixed[j].x + fixed[j].w) > VRoot.w)
+		     fixed[j].w = VRoot.w - fixed[j].x;
 		  if (fixed[j].y < 0)
 		    {
 		       fixed[j].h += fixed[j].y;
 		       fixed[j].y = 0;
 		    }
-		  if ((fixed[j].y + fixed[j].h) > root.h)
-		     fixed[j].h = root.h - fixed[j].y;
+		  if ((fixed[j].y + fixed[j].h) > VRoot.h)
+		     fixed[j].h = VRoot.h - fixed[j].y;
 		  if ((fixed[j].w <= 0) || (fixed[j].h <= 0))
 		     continue;
 
-		  if (b->sticky)
+		  if (fixed[j].p)	/* Sticky */
 		     fixed[j].p = 1;
 		  else
 		     fixed[j].p = 0;
@@ -927,20 +913,8 @@ ArrangeEwinXY(EWin * ewin, int *px, int *py)
 	newrect.w = ewin->w;
 	newrect.h = ewin->h;
 	newrect.p = ewin->layer;
-#if ENABLE_KDE
-	if (mode.kde_support)
-	  {
-	     ArrangeRects(fixed, j, &newrect, 1, ret,
-			  mode.kde_x1, mode.kde_y1, mode.kde_x2, mode.kde_y2,
-			  ARRANGE_BY_SIZE, 1);
-
-	  }
-	else
-#endif
-	  {
-	     ArrangeRects(fixed, j, &newrect, 1, ret,
-			  0, 0, root.w, root.h, ARRANGE_BY_SIZE, 1);
-	  }
+	ArrangeRects(fixed, j, &newrect, 1, ret,
+		     0, 0, VRoot.w, VRoot.h, ARRANGE_BY_SIZE, 1);
 
 	for (i = 0; i < j + 1; i++)
 	  {
@@ -951,7 +925,6 @@ ArrangeEwinXY(EWin * ewin, int *px, int *py)
 		  break;
 	       }
 	  }
-	Efree(lst);
 	if (ret)
 	   Efree(ret);
 	if (fixed)
@@ -968,7 +941,7 @@ ArrangeEwinCenteredXY(EWin * ewin, int *px, int *py)
 {
    int                 x, y, w, h;
 
-   GetPointerScreenGeometry(&x, &y, &w, &h);
+   GetPointerScreenAvailableArea(&x, &y, &w, &h);
    *px = (w - ewin->w) / 2 + x;
    *py = (h - ewin->h) / 2 + y;
 }
