@@ -40,7 +40,7 @@
 #define EWIN_CLIENT_EVENT_MASK \
   (EnterWindowMask | LeaveWindowMask | FocusChangeMask | \
    StructureNotifyMask | ResizeRedirectMask | \
-   PropertyChangeMask | ColormapChangeMask)
+   PropertyChangeMask | ColormapChangeMask | VisibilityChangeMask)
 
 static void         EwinBorderSetTo(EWin * ewin, Border * b);
 static EWin        *EwinCreate(Window win);
@@ -184,15 +184,11 @@ GetContextEwin(void)
 void
 SlideEwinTo(EWin * ewin, int fx, int fy, int tx, int ty, int speed)
 {
-   int                 k, spd, x, y, min, tmpx, tmpy, tmpw, tmph;
-   struct timeval      timev1, timev2;
-   int                 dsec, dusec;
-   double              tm;
+   int                 k, x, y, tmpx, tmpy;
    char                firstlast;
 
    EDBUG(3, "SlideEwinTo");
-   spd = 16;
-   min = 2;
+
    firstlast = 0;
    Mode.doingslide = 1;
    SoundPlay("SOUND_WINDOW_SLIDE");
@@ -200,42 +196,33 @@ SlideEwinTo(EWin * ewin, int fx, int fy, int tx, int ty, int speed)
    if (Conf.slidemode > 0)
       GrabX();
 
-   for (k = 0; k <= 1024; k += spd)
+   ETimedLoopInit(0, 1024, speed);
+   for (k = 0; k <= 1024;)
      {
-	gettimeofday(&timev1, NULL);
 	x = ((fx * (1024 - k)) + (tx * k)) >> 10;
 	y = ((fy * (1024 - k)) + (ty * k)) >> 10;
 	tmpx = x;
 	tmpy = y;
-	tmpw = ewin->client.w;
-	tmph = ewin->client.h;
 	if (Conf.slidemode == 0)
 	   EMoveWindow(disp, ewin->win, tmpx, tmpy);
 	else
-	   DrawEwinShape(ewin, Conf.slidemode, tmpx, tmpy, tmpw, tmph,
-			 firstlast);
+	   DrawEwinShape(ewin, Conf.slidemode, tmpx, tmpy,
+			 ewin->client.w, ewin->client.h, firstlast);
 	if (firstlast == 0)
 	   firstlast = 1;
 	XSync(disp, False);
-	gettimeofday(&timev2, NULL);
-	dsec = timev2.tv_sec - timev1.tv_sec;
-	dusec = timev2.tv_usec - timev1.tv_usec;
-	if (dusec < 0)
-	  {
-	     dsec--;
-	     dusec += 1000000;
-	  }
-	tm = (double)dsec + (((double)dusec) / 1000000);
-	spd = (int)((double)speed * tm);
-	if (spd < min)
-	   spd = min;
+
+	k = ETimedLoopNext();
      }
    DrawEwinShape(ewin, Conf.slidemode, x, y, ewin->client.w, ewin->client.h, 2);
    MoveEwin(ewin, tx, ty);
    Mode.doingslide = 0;
+
    if (Conf.slidemode > 0)
       UngrabX();
+
    SoundPlay("SOUND_WINDOW_SLIDE_END");
+
    EDBUG_RETURN_;
 }
 
@@ -243,11 +230,7 @@ void
 SlideEwinsTo(EWin ** ewin, int *fx, int *fy, int *tx, int *ty, int num_wins,
 	     int speed)
 {
-   int                 k, spd, *x = NULL, *y =
-      NULL, min, tmpx, tmpy, tmpw, tmph, i;
-   struct timeval      timev1, timev2;
-   int                 dsec, dusec;
-   double              tm;
+   int                 k, *x = NULL, *y = NULL, tmpx, tmpy, tmpw, tmph, i;
    char                firstlast;
 
    EDBUG(3, "SlideEwinsTo");
@@ -257,48 +240,38 @@ SlideEwinsTo(EWin ** ewin, int *fx, int *fy, int *tx, int *ty, int num_wins,
 	x = Emalloc(sizeof(int) * num_wins);
 	y = Emalloc(sizeof(int) * num_wins);
      }
-   spd = 16;
-   min = 2;
+
    firstlast = 0;
    Mode.doingslide = 1;
    SoundPlay("SOUND_WINDOW_SLIDE");
+
    if (Conf.slidemode > 0)
       GrabX();
-   for (k = 0; k <= 1024; k += spd)
+
+   ETimedLoopInit(0, 1024, speed);
+   for (k = 0; k <= 1024;)
      {
 	for (i = 0; i < num_wins; i++)
 	  {
-	     if (ewin[i])
-	       {
-		  gettimeofday(&timev1, NULL);
-		  x[i] = ((fx[i] * (1024 - k)) + (tx[i] * k)) >> 10;
-		  y[i] = ((fy[i] * (1024 - k)) + (ty[i] * k)) >> 10;
-		  tmpx = x[i];
-		  tmpy = y[i];
-		  tmpw = ewin[i]->client.w;
-		  tmph = ewin[i]->client.h;
-		  if (ewin[i]->menu)
-		     EMoveWindow(disp, ewin[i]->win, tmpx, tmpy);
-		  else
-		     DrawEwinShape(ewin[i], 0, tmpx, tmpy, tmpw, tmph,
-				   firstlast);
-		  if (firstlast == 0)
-		     firstlast = 1;
-		  XSync(disp, False);
-		  gettimeofday(&timev2, NULL);
-		  dsec = timev2.tv_sec - timev1.tv_sec;
-		  dusec = timev2.tv_usec - timev1.tv_usec;
-		  if (dusec < 0)
-		    {
-		       dsec--;
-		       dusec += 1000000;
-		    }
-		  tm = (double)dsec + (((double)dusec) / 1000000);
-		  spd = (int)((double)speed * tm);
-		  if (spd < min)
-		     spd = min;
-	       }
+	     if (!ewin[i])
+		continue;
+
+	     x[i] = ((fx[i] * (1024 - k)) + (tx[i] * k)) >> 10;
+	     y[i] = ((fy[i] * (1024 - k)) + (ty[i] * k)) >> 10;
+	     tmpx = x[i];
+	     tmpy = y[i];
+	     tmpw = ewin[i]->client.w;
+	     tmph = ewin[i]->client.h;
+	     if (ewin[i]->menu)
+		EMoveWindow(disp, ewin[i]->win, tmpx, tmpy);
+	     else
+		DrawEwinShape(ewin[i], 0, tmpx, tmpy, tmpw, tmph, firstlast);
+	     if (firstlast == 0)
+		firstlast = 1;
+	     XSync(disp, False);
 	  }
+	/* We may loop faster here than originally intended */
+	k = ETimedLoopNext();
      }
 
    for (i = 0; i < num_wins; i++)
@@ -314,11 +287,14 @@ SlideEwinsTo(EWin ** ewin, int *fx, int *fy, int *tx, int *ty, int num_wins,
    Mode.doingslide = 0;
    if (Conf.slidemode > 0)
       UngrabX();
+
    SoundPlay("SOUND_WINDOW_SLIDE_END");
+
    if (x)
       Efree(x);
    if (y)
       Efree(y);
+
    EDBUG_RETURN_;
 }
 
@@ -353,7 +329,6 @@ AddToFamily(Window win)
    EWin              **lst;
    int                 i, k, num, speed, fx, fy, x, y;
    char                doslide, manplace;
-   char                cangrab = 0;
 
    EDBUG(3, "AddToFamily");
 
@@ -391,7 +366,7 @@ AddToFamily(Window win)
    if (Conf.dockapp_support && ewin->docked)
      {
 	DockIt(ewin);
-	EDBUG_RETURN_;
+	ewin->props.donthide = 1;
      }
 
    /* if set for borderless then dont slide it in */
@@ -488,23 +463,26 @@ AddToFamily(Window win)
 	ewin->desktop = DESKTOPS_WRAP_NUM(ewin->desktop);
      }
 
+   if (ewin->st.fullscreen)
+     {
+	ewin->st.fullscreen = 0;
+	EwinSetFullscreen(ewin, 1);
+	ewin->client.already_placed = 1;
+	ShowEwin(ewin);
+	UngrabX();
+	EDBUG_RETURN_;
+     }
+
    ResizeEwin(ewin, ewin->client.w, ewin->client.h);
 
    if ((!ewin->client.transient) && (Conf.place.manual)
        && (!ewin->client.already_placed) && (!Mode.wm.startup) && (!Mode.place))
      {
+	char                cangrab;
+
 	cangrab = GrabPointerSet(VRoot.win, ECSR_GRAB, 0);
-	if ((cangrab == GrabNotViewable) || (cangrab == AlreadyGrabbed)
-	    || (cangrab == GrabFrozen))
-	  {
-	     XUngrabPointer(disp, CurrentTime);
-	     cangrab = 0;
-	  }
-	else
-	  {
-	     manplace = 1;
-	     cangrab = 1;
-	  }
+	if (cangrab == GrabSuccess)
+	   manplace = 1;
      }
 
    /* if it hasn't been placed yet.... find a spot for it */
@@ -526,10 +504,8 @@ AddToFamily(Window win)
 	     if (ewin->desktop >= 0)
 		GotoDesktop(ewin->desktop);
 
-	     GrabPointerSet(VRoot.win, ECSR_GRAB, 0);
 	     XQueryPointer(disp, VRoot.win, &root_return, &junk, &rx, &ry, &wx,
 			   &wy, &mask);
-	     XUngrabPointer(disp, CurrentTime);
 	     Mode.x = rx;
 	     Mode.y = ry;
 	     ewin->client.already_placed = 1;
@@ -565,6 +541,7 @@ AddToFamily(Window win)
 	EwinBorderDraw(ewin, 1, 0);
 	MoveEwinToDesktopAt(ewin, ewin->desktop, x, y);
 	UngrabX();
+	ewin->state = EWIN_STATE_MAPPED;
 	IconifyEwin(ewin);
 	ewin->state = EWIN_STATE_ICONIC;
 	EDBUG_RETURN_;
@@ -736,6 +713,7 @@ BorderWinpartITclassApply(EWin * ewin, int i, int force)
 {
    EWinBit            *ewb = &ewin->bits[i];
    ImageState         *is;
+   TextState          *ts;
    const char         *title;
 
    if (ewb->win == None)
@@ -743,9 +721,14 @@ BorderWinpartITclassApply(EWin * ewin, int i, int force)
 
    is = IclassGetImageState(ewin->border->part[i].iclass, ewin->bits[i].state,
 			    ewin->active, ewin->sticky);
-   if (!force && ewin->bits[i].is == is)
+   ts = NULL;
+   if (ewin->border->part[i].tclass)
+      ts = TextGetState(ewin->border->part[i].tclass, ewin->active,
+			ewin->sticky, ewin->bits[i].state);
+   if (!force && ewin->bits[i].is == is && ewin->bits[i].ts == ts)
       return;
    ewin->bits[i].is = is;
+   ewin->bits[i].ts = ts;
 
    IclassApply(ewin->border->part[i].iclass, ewb->win,
 	       ewb->w, ewb->h, ewin->active,
@@ -809,7 +792,7 @@ BorderWinpartChange(EWin * ewin, int i, int force)
    BorderWinpartITclassApply(ewin, i, force);
 
    if (!ewin->shapedone || ewin->border->changes_shape)
-      PropagateShapes(ewin->win);
+      EwinPropagateShapes(ewin);
    ewin->shapedone = 1;
 
    EDBUG_RETURN_;
@@ -833,7 +816,7 @@ EwinBorderDraw(EWin * ewin, int do_shape, int queue_off)
       BorderWinpartITclassApply(ewin, i, do_shape);
 
    if (do_shape || !ewin->shapedone || ewin->border->changes_shape)
-      PropagateShapes(ewin->win);
+      EwinPropagateShapes(ewin);
    ewin->shapedone = 1;
 
    if (queue_off)
@@ -1068,11 +1051,11 @@ CalcEwinSizes(EWin * ewin)
 
 	     pq = Mode.queue_up;
 	     Mode.queue_up = 0;
-	     PropagateShapes(ewin->win);
+	     EwinPropagateShapes(ewin);
 	     Mode.queue_up = pq;
 	  }
 	else
-	   PropagateShapes(ewin->win);
+	   EwinPropagateShapes(ewin);
 	ewin->shapedone = 1;
      }
 
@@ -1105,12 +1088,12 @@ HonorIclass(char *s, int id)
 		  ewin->shapedone = 0;
 		  if (!ewin->shapedone)
 		    {
-		       PropagateShapes(ewin->win);
+		       EwinPropagateShapes(ewin);
 		    }
 		  else
 		    {
 		       if (ewin->border->changes_shape)
-			  PropagateShapes(ewin->win);
+			  EwinPropagateShapes(ewin);
 		    }
 		  ewin->shapedone = 1;
 	       }
@@ -1599,9 +1582,6 @@ EwinEventUnmap(EWin * ewin)
    if (ewin->pager)
       PagerEventUnmap(ewin->pager);
 
-   if (Conf.dockapp_support && ewin->docked)
-      DockDestroy(ewin);
-
    if (ewin == GetContextEwin())
       SlideoutsHide();
 
@@ -1641,8 +1621,9 @@ EwinEventUnmap(EWin * ewin)
 }
 
 void
-EwinEventVisibility(EWin * ewin __UNUSED__, int state __UNUSED__)
+EwinEventVisibility(EWin * ewin, int state)
 {
+   ewin->visibility = state;
 }
 
 static void
@@ -1769,7 +1750,7 @@ EwinBorderSetTo(EWin * ewin, Border * b)
       EMoveWindow(disp, ewin->win_container, b->border.left, b->border.top);
 
    CalcEwinSizes(ewin);
-   PropagateShapes(ewin->win);
+   EwinPropagateShapes(ewin);
 
    EDBUG_RETURN_;
 }
@@ -2297,6 +2278,8 @@ FreeBorder(Border * b)
       Efree(b->name);
    if (b->group_border_name)
       Efree(b->group_border_name);
+   if (b->aclass)
+      b->aclass->ref_count--;
 
    EDBUG_RETURN_;
 }
@@ -2308,21 +2291,12 @@ CreateBorder(const char *name)
 
    EDBUG(5, "CreateBorder");
 
-   b = Emalloc(sizeof(Border));
+   b = Ecalloc(1, sizeof(Border));
    if (!b)
       EDBUG_RETURN(NULL);
 
    b->name = Estrdup(name);
-   b->group_border_name = NULL;
-   b->border.left = 0;
-   b->border.right = 0;
-   b->border.top = 0;
-   b->border.bottom = 0;
-   b->num_winparts = 0;
-   b->part = NULL;
-   b->changes_shape = 0;
    b->shadedir = 2;
-   b->ref_count = 0;
 
    EDBUG_RETURN(b);
 }
@@ -2574,7 +2548,8 @@ EwinInstantShade(EWin * ewin, int force)
 	d = ewin->x + ewin->w - b;
 	ewin->shaded = 2;
 	ewin->w = b;
-	ewin->x = d;
+	if (!Mode.wm.startup)
+	   ewin->x = d;
 	EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w, ewin->h);
 	EMoveResizeWindow(disp, ewin->win_container, -30, -30, 1, 1);
 	CalcEwinSizes(ewin);
@@ -2600,7 +2575,8 @@ EwinInstantShade(EWin * ewin, int force)
 	d = ewin->y + ewin->h - b;
 	ewin->shaded = 2;
 	ewin->h = b;
-	ewin->y = d;
+	if (!Mode.wm.startup)
+	   ewin->y = d;
 	EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w, ewin->h);
 	EMoveResizeWindow(disp, ewin->win_container, -30, -30, 1, 1);
 	CalcEwinSizes(ewin);
@@ -2609,7 +2585,7 @@ EwinInstantShade(EWin * ewin, int force)
      default:
 	break;
      }
-   PropagateShapes(ewin->win);
+   EwinPropagateShapes(ewin);
    Mode.queue_up = pq;
    HintsSetWindowState(ewin);
    if (Mode.mode == MODE_NONE)
@@ -2685,7 +2661,7 @@ EwinInstantUnShade(EWin * ewin)
      default:
 	break;
      }
-   PropagateShapes(ewin->win);
+   EwinPropagateShapes(ewin);
    Mode.queue_up = pq;
    HintsSetWindowState(ewin);
    if (Mode.mode == MODE_NONE)
@@ -2700,11 +2676,7 @@ void
 EwinShade(EWin * ewin)
 {
    XSetWindowAttributes att;
-   int                 i, j, speed, a, b, c, d, ww, hh;
-   int                 k, spd, min;
-   struct timeval      timev1, timev2;
-   int                 dsec, dusec;
-   double              tm;
+   int                 i, j, k, speed, a, b, c, d, ww, hh;
    char                pq;
 
    EDBUG(4, "EwinShade");
@@ -2723,8 +2695,6 @@ EwinShade(EWin * ewin)
    Mode.queue_up = 0;
 
    speed = Conf.shadespeed;
-   spd = 32;
-   min = 2;
 
    GrabX();
 
@@ -2736,43 +2706,35 @@ EwinShade(EWin * ewin)
 	MinShadeSize(ewin, &b, &d);
 	a = ewin->w;
 	if ((Conf.animate_shading) || (ewin->menu))
-	   for (k = 0; k <= 1024; k += spd)
-	     {
-		gettimeofday(&timev1, NULL);
-		i = ((a * (1024 - k)) + (b * k)) >> 10;
-		ewin->w = i;
-		if (ewin->w < 1)
-		   ewin->w = 1;
-		ww = ewin->w - ewin->border->border.left -
-		   ewin->border->border.right;
-		if (ww < 1)
-		   ww = 1;
-		hh = ewin->client.h;
-		EMoveResizeWindow(disp, ewin->win_container,
-				  ewin->border->border.left,
-				  ewin->border->border.top, ww, hh);
-		EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
-				  ewin->h);
-		CalcEwinSizes(ewin);
-		if (ewin->client.shaped)
-		   EShapeCombineShape(disp, ewin->win_container,
-				      ShapeBounding, -(ewin->client.w - ww),
-				      0, ewin->client.win, ShapeBounding,
-				      ShapeSet);
-		PropagateShapes(ewin->win);
-		gettimeofday(&timev2, NULL);
-		dsec = timev2.tv_sec - timev1.tv_sec;
-		dusec = timev2.tv_usec - timev1.tv_usec;
-		if (dusec < 0)
-		  {
-		     dsec--;
-		     dusec += 1000000;
-		  }
-		tm = (double)dsec + (((double)dusec) / 1000000);
-		spd = (int)((double)speed * tm);
-		if (spd < min)
-		   spd = min;
-	     }
+	  {
+	     ETimedLoopInit(0, 1024, speed);
+	     for (k = 0; k <= 1024;)
+	       {
+		  i = ((a * (1024 - k)) + (b * k)) >> 10;
+		  ewin->w = i;
+		  if (ewin->w < 1)
+		     ewin->w = 1;
+		  ww = ewin->w - ewin->border->border.left -
+		     ewin->border->border.right;
+		  if (ww < 1)
+		     ww = 1;
+		  hh = ewin->client.h;
+		  EMoveResizeWindow(disp, ewin->win_container,
+				    ewin->border->border.left,
+				    ewin->border->border.top, ww, hh);
+		  EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
+				    ewin->h);
+		  CalcEwinSizes(ewin);
+		  if (ewin->client.shaped)
+		     EShapeCombineShape(disp, ewin->win_container,
+					ShapeBounding, -(ewin->client.w - ww),
+					0, ewin->client.win, ShapeBounding,
+					ShapeSet);
+		  EwinPropagateShapes(ewin);
+
+		  k = ETimedLoopNext();
+	       }
+	  }
 	ewin->shaded = 2;
 	ewin->w = b;
 	EMoveResizeWindow(disp, ewin->win_container, -30, -30, 1, 1);
@@ -2788,44 +2750,36 @@ EwinShade(EWin * ewin)
 	c = ewin->x;
 	d = ewin->x + ewin->w - b;
 	if ((Conf.animate_shading) || (ewin->menu))
-	   for (k = 0; k <= 1024; k += spd)
-	     {
-		gettimeofday(&timev1, NULL);
-		i = ((a * (1024 - k)) + (b * k)) >> 10;
-		j = ((c * (1024 - k)) + (d * k)) >> 10;
-		ewin->w = i;
-		ewin->x = j;
-		if (ewin->w < 1)
-		   ewin->w = 1;
-		ww = ewin->w - ewin->border->border.left -
-		   ewin->border->border.right;
-		if (ww < 1)
-		   ww = 1;
-		hh = ewin->client.h;
-		EMoveResizeWindow(disp, ewin->win_container,
-				  ewin->border->border.left,
-				  ewin->border->border.top, ww, hh);
-		EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
-				  ewin->h);
-		CalcEwinSizes(ewin);
-		if (ewin->client.shaped)
-		   EShapeCombineShape(disp, ewin->win_container,
-				      ShapeBounding, 0, 0, ewin->client.win,
-				      ShapeBounding, ShapeSet);
-		PropagateShapes(ewin->win);
-		gettimeofday(&timev2, NULL);
-		dsec = timev2.tv_sec - timev1.tv_sec;
-		dusec = timev2.tv_usec - timev1.tv_usec;
-		if (dusec < 0)
-		  {
-		     dsec--;
-		     dusec += 1000000;
-		  }
-		tm = (double)dsec + (((double)dusec) / 1000000);
-		spd = (int)((double)speed * tm);
-		if (spd < min)
-		   spd = min;
-	     }
+	  {
+	     ETimedLoopInit(0, 1024, speed);
+	     for (k = 0; k <= 1024;)
+	       {
+		  i = ((a * (1024 - k)) + (b * k)) >> 10;
+		  j = ((c * (1024 - k)) + (d * k)) >> 10;
+		  ewin->w = i;
+		  ewin->x = j;
+		  if (ewin->w < 1)
+		     ewin->w = 1;
+		  ww = ewin->w - ewin->border->border.left -
+		     ewin->border->border.right;
+		  if (ww < 1)
+		     ww = 1;
+		  hh = ewin->client.h;
+		  EMoveResizeWindow(disp, ewin->win_container,
+				    ewin->border->border.left,
+				    ewin->border->border.top, ww, hh);
+		  EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
+				    ewin->h);
+		  CalcEwinSizes(ewin);
+		  if (ewin->client.shaped)
+		     EShapeCombineShape(disp, ewin->win_container,
+					ShapeBounding, 0, 0, ewin->client.win,
+					ShapeBounding, ShapeSet);
+		  EwinPropagateShapes(ewin);
+
+		  k = ETimedLoopNext();
+	       }
+	  }
 	ewin->shaded = 2;
 	ewin->w = b;
 	ewin->x = d;
@@ -2841,44 +2795,36 @@ EwinShade(EWin * ewin)
 	MinShadeSize(ewin, &b, &d);
 	b = d;
 	if ((Conf.animate_shading) || (ewin->menu))
-	   for (k = 0; k <= 1024; k += spd)
-	     {
-		gettimeofday(&timev1, NULL);
-		i = ((a * (1024 - k)) + (b * k)) >> 10;
-		ewin->h = i;
-		if (ewin->h < 1)
-		   ewin->h = 1;
-		hh = ewin->h - ewin->border->border.top -
-		   ewin->border->border.bottom;
-		if (hh < 1)
-		   hh = 1;
-		ww = ewin->client.w;
-		EMoveResizeWindow(disp, ewin->win_container,
-				  ewin->border->border.left,
-				  ewin->border->border.top, ww, hh);
-		EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
-				  ewin->h);
-		CalcEwinSizes(ewin);
-		if (ewin->client.shaped)
-		   EShapeCombineShape(disp, ewin->win_container,
-				      ShapeBounding, 0,
-				      -(ewin->client.h - hh),
-				      ewin->client.win, ShapeBounding,
-				      ShapeSet);
-		PropagateShapes(ewin->win);
-		gettimeofday(&timev2, NULL);
-		dsec = timev2.tv_sec - timev1.tv_sec;
-		dusec = timev2.tv_usec - timev1.tv_usec;
-		if (dusec < 0)
-		  {
-		     dsec--;
-		     dusec += 1000000;
-		  }
-		tm = (double)dsec + (((double)dusec) / 1000000);
-		spd = (int)((double)speed * tm);
-		if (spd < min)
-		   spd = min;
-	     }
+	  {
+	     ETimedLoopInit(0, 1024, speed);
+	     for (k = 0; k <= 1024;)
+	       {
+		  i = ((a * (1024 - k)) + (b * k)) >> 10;
+		  ewin->h = i;
+		  if (ewin->h < 1)
+		     ewin->h = 1;
+		  hh = ewin->h - ewin->border->border.top -
+		     ewin->border->border.bottom;
+		  if (hh < 1)
+		     hh = 1;
+		  ww = ewin->client.w;
+		  EMoveResizeWindow(disp, ewin->win_container,
+				    ewin->border->border.left,
+				    ewin->border->border.top, ww, hh);
+		  EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
+				    ewin->h);
+		  CalcEwinSizes(ewin);
+		  if (ewin->client.shaped)
+		     EShapeCombineShape(disp, ewin->win_container,
+					ShapeBounding, 0,
+					-(ewin->client.h - hh),
+					ewin->client.win, ShapeBounding,
+					ShapeSet);
+		  EwinPropagateShapes(ewin);
+
+		  k = ETimedLoopNext();
+	       }
+	  }
 	ewin->shaded = 2;
 	ewin->h = b;
 	EMoveResizeWindow(disp, ewin->win_container, -30, -30, 1, 1);
@@ -2895,44 +2841,35 @@ EwinShade(EWin * ewin)
 	c = ewin->y;
 	d = ewin->y + ewin->h - b;
 	if ((Conf.animate_shading) || (ewin->menu))
-	   for (k = 0; k <= 1024; k += spd)
-	     {
-		gettimeofday(&timev1, NULL);
-		i = ((a * (1024 - k)) + (b * k)) >> 10;
-		j = ((c * (1024 - k)) + (d * k)) >> 10;
-		ewin->h = i;
-		ewin->y = j;
-		if (ewin->h < 1)
-		   ewin->h = 1;
-		hh = ewin->h - ewin->border->border.top -
-		   ewin->border->border.bottom;
-		if (hh < 1)
-		   hh = 1;
-		ww = ewin->client.w;
-		EMoveResizeWindow(disp, ewin->win_container,
-				  ewin->border->border.left,
-				  ewin->border->border.top, ww, hh);
-		EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
-				  ewin->h);
-		CalcEwinSizes(ewin);
-		if (ewin->client.shaped)
-		   EShapeCombineShape(disp, ewin->win_container,
-				      ShapeBounding, 0, 0, ewin->client.win,
-				      ShapeBounding, ShapeSet);
-		PropagateShapes(ewin->win);
-		gettimeofday(&timev2, NULL);
-		dsec = timev2.tv_sec - timev1.tv_sec;
-		dusec = timev2.tv_usec - timev1.tv_usec;
-		if (dusec < 0)
-		  {
-		     dsec--;
-		     dusec += 1000000;
-		  }
-		tm = (double)dsec + (((double)dusec) / 1000000);
-		spd = (int)((double)speed * tm);
-		if (spd < min)
-		   spd = min;
-	     }
+	  {
+	     ETimedLoopInit(0, 1024, speed);
+	     for (k = 0; k <= 1024;)
+	       {
+		  i = ((a * (1024 - k)) + (b * k)) >> 10;
+		  j = ((c * (1024 - k)) + (d * k)) >> 10;
+		  ewin->h = i;
+		  ewin->y = j;
+		  if (ewin->h < 1)
+		     ewin->h = 1;
+		  hh = ewin->h - ewin->border->border.top -
+		     ewin->border->border.bottom;
+		  if (hh < 1)
+		     hh = 1;
+		  ww = ewin->client.w;
+		  EMoveResizeWindow(disp, ewin->win_container,
+				    ewin->border->border.left,
+				    ewin->border->border.top, ww, hh);
+		  EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
+				    ewin->h);
+		  CalcEwinSizes(ewin);
+		  if (ewin->client.shaped)
+		     EShapeCombineShape(disp, ewin->win_container,
+					ShapeBounding, 0, 0, ewin->client.win,
+					ShapeBounding, ShapeSet);
+		  EwinPropagateShapes(ewin);
+		  k = ETimedLoopNext();
+	       }
+	  }
 	ewin->shaded = 2;
 	ewin->h = b;
 	ewin->y = d;
@@ -2950,7 +2887,7 @@ EwinShade(EWin * ewin)
    if (ewin->client.shaped)
       EShapeCombineShape(disp, ewin->win_container, ShapeBounding, 0, 0,
 			 ewin->client.win, ShapeBounding, ShapeSet);
-   PropagateShapes(ewin->win);
+   EwinPropagateShapes(ewin);
 
    Mode.queue_up = pq;
 
@@ -2969,14 +2906,11 @@ void
 EwinUnShade(EWin * ewin)
 {
    XSetWindowAttributes att;
-   int                 i, j, speed, a, b, c, d;
-   int                 k, spd, min;
-   struct timeval      timev1, timev2;
-   int                 dsec, dusec;
-   double              tm;
+   int                 i, j, k, speed, a, b, c, d;
    char                pq;
 
    EDBUG(4, "EwinUnShade");
+
    if (GetZoomEWin() == ewin)
       EDBUG_RETURN_;
    if (!ewin->shaded || ewin->iconified)
@@ -2986,8 +2920,6 @@ EwinUnShade(EWin * ewin)
    Mode.queue_up = 0;
 
    speed = Conf.shadespeed;
-   spd = 32;
-   min = 2;
 
    GrabX();
 
@@ -2996,9 +2928,9 @@ EwinUnShade(EWin * ewin)
      case 0:
 	att.win_gravity = EastGravity;
 	XChangeWindowAttributes(disp, ewin->client.win, CWWinGravity, &att);
-	a = ewin->border->border.left;
-	b = ewin->client.w + ewin->border->border.left +
-	   ewin->border->border.right;
+	a = ewin->border->border.left + ewin->border->border.right;
+	b = ewin->client.w + a;
+	a++;
 	ewin->shaded = 0;
 	EMoveResizeWindow(disp, ewin->win_container,
 			  ewin->border->border.left, ewin->border->border.top,
@@ -3008,41 +2940,33 @@ EwinUnShade(EWin * ewin)
 	EMapWindow(disp, ewin->client.win);
 	EMapWindow(disp, ewin->win_container);
 	if ((Conf.animate_shading) || (ewin->menu))
-	   for (k = 0; k <= 1024; k += spd)
-	     {
-		gettimeofday(&timev1, NULL);
-		i = ((a * (1024 - k)) + (b * k)) >> 10;
-		ewin->w = i;
-		EMoveResizeWindow(disp, ewin->win_container,
-				  ewin->border->border.left,
-				  ewin->border->border.top,
-				  ewin->w - ewin->border->border.left -
-				  ewin->border->border.right, ewin->client.h);
-		EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
-				  ewin->h);
-		CalcEwinSizes(ewin);
-		if (ewin->client.shaped)
-		   EShapeCombineShape(disp, ewin->win_container,
-				      ShapeBounding,
-				      -(ewin->client.w -
-					(ewin->w - ewin->border->border.left -
-					 ewin->border->border.right)), 0,
-				      ewin->client.win, ShapeBounding,
-				      ShapeSet);
-		PropagateShapes(ewin->win);
-		gettimeofday(&timev2, NULL);
-		dsec = timev2.tv_sec - timev1.tv_sec;
-		dusec = timev2.tv_usec - timev1.tv_usec;
-		if (dusec < 0)
-		  {
-		     dsec--;
-		     dusec += 1000000;
-		  }
-		tm = (double)dsec + (((double)dusec) / 1000000);
-		spd = (int)((double)speed * tm);
-		if (spd < min)
-		   spd = min;
-	     }
+	  {
+	     ETimedLoopInit(0, 1024, speed);
+	     for (k = 0; k <= 1024;)
+	       {
+		  i = ((a * (1024 - k)) + (b * k)) >> 10;
+		  ewin->w = i;
+		  EMoveResizeWindow(disp, ewin->win_container,
+				    ewin->border->border.left,
+				    ewin->border->border.top,
+				    ewin->w - ewin->border->border.left -
+				    ewin->border->border.right, ewin->client.h);
+		  EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
+				    ewin->h);
+		  CalcEwinSizes(ewin);
+		  if (ewin->client.shaped)
+		     EShapeCombineShape(disp, ewin->win_container,
+					ShapeBounding,
+					-(ewin->client.w -
+					  (ewin->w - ewin->border->border.left -
+					   ewin->border->border.right)), 0,
+					ewin->client.win, ShapeBounding,
+					ShapeSet);
+		  EwinPropagateShapes(ewin);
+
+		  k = ETimedLoopNext();
+	       }
+	  }
 	ewin->w = b;
 	MoveResizeEwin(ewin, ewin->x, ewin->y, ewin->client.w, ewin->client.h);
 	XSync(disp, False);
@@ -3050,12 +2974,11 @@ EwinUnShade(EWin * ewin)
      case 1:
 	att.win_gravity = WestGravity;
 	XChangeWindowAttributes(disp, ewin->client.win, CWWinGravity, &att);
-	a = ewin->border->border.right;
-	b = ewin->client.w + ewin->border->border.left +
-	   ewin->border->border.right;
+	a = ewin->border->border.left + ewin->border->border.right;
+	b = ewin->client.w + a;
 	c = ewin->x;
-	d = ewin->x + ewin->w - (ewin->border->border.right + ewin->client.w +
-				 ewin->border->border.left);
+	d = ewin->x + ewin->w - (ewin->client.w + a);
+	a++;
 	ewin->shaded = 0;
 	EMoveResizeWindow(disp, ewin->win_container,
 			  ewin->border->border.left, ewin->border->border.top,
@@ -3065,39 +2988,31 @@ EwinUnShade(EWin * ewin)
 	EMapWindow(disp, ewin->client.win);
 	EMapWindow(disp, ewin->win_container);
 	if ((Conf.animate_shading) || (ewin->menu))
-	   for (k = 0; k <= 1024; k += spd)
-	     {
-		gettimeofday(&timev1, NULL);
-		i = ((a * (1024 - k)) + (b * k)) >> 10;
-		j = ((c * (1024 - k)) + (d * k)) >> 10;
-		ewin->w = i;
-		ewin->x = j;
-		EMoveResizeWindow(disp, ewin->win_container,
-				  ewin->border->border.left,
-				  ewin->border->border.top,
-				  ewin->w - ewin->border->border.left -
-				  ewin->border->border.right, ewin->client.h);
-		EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
-				  ewin->h);
-		CalcEwinSizes(ewin);
-		if (ewin->client.shaped)
-		   EShapeCombineShape(disp, ewin->win_container,
-				      ShapeBounding, 0, 0, ewin->client.win,
-				      ShapeBounding, ShapeSet);
-		PropagateShapes(ewin->win);
-		gettimeofday(&timev2, NULL);
-		dsec = timev2.tv_sec - timev1.tv_sec;
-		dusec = timev2.tv_usec - timev1.tv_usec;
-		if (dusec < 0)
-		  {
-		     dsec--;
-		     dusec += 1000000;
-		  }
-		tm = (double)dsec + (((double)dusec) / 1000000);
-		spd = (int)((double)speed * tm);
-		if (spd < min)
-		   spd = min;
-	     }
+	  {
+	     ETimedLoopInit(0, 1024, speed);
+	     for (k = 0; k <= 1024;)
+	       {
+		  i = ((a * (1024 - k)) + (b * k)) >> 10;
+		  j = ((c * (1024 - k)) + (d * k)) >> 10;
+		  ewin->w = i;
+		  ewin->x = j;
+		  EMoveResizeWindow(disp, ewin->win_container,
+				    ewin->border->border.left,
+				    ewin->border->border.top,
+				    ewin->w - ewin->border->border.left -
+				    ewin->border->border.right, ewin->client.h);
+		  EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
+				    ewin->h);
+		  CalcEwinSizes(ewin);
+		  if (ewin->client.shaped)
+		     EShapeCombineShape(disp, ewin->win_container,
+					ShapeBounding, 0, 0, ewin->client.win,
+					ShapeBounding, ShapeSet);
+		  EwinPropagateShapes(ewin);
+
+		  k = ETimedLoopNext();
+	       }
+	  }
 	ewin->w = b;
 	ewin->x = d;
 	MoveResizeEwin(ewin, ewin->x, ewin->y, ewin->client.w, ewin->client.h);
@@ -3106,9 +3021,9 @@ EwinUnShade(EWin * ewin)
      case 2:
 	att.win_gravity = SouthGravity;
 	XChangeWindowAttributes(disp, ewin->client.win, CWWinGravity, &att);
-	a = ewin->border->border.top;
-	b = ewin->client.h + ewin->border->border.top +
-	   ewin->border->border.bottom;
+	a = ewin->border->border.top + ewin->border->border.bottom;
+	b = ewin->client.h + a;
+	a++;
 	ewin->shaded = 0;
 	EMoveResizeWindow(disp, ewin->win_container,
 			  ewin->border->border.left, ewin->border->border.top,
@@ -3118,41 +3033,33 @@ EwinUnShade(EWin * ewin)
 	EMapWindow(disp, ewin->client.win);
 	EMapWindow(disp, ewin->win_container);
 	if ((Conf.animate_shading) || (ewin->menu))
-	   for (k = 0; k <= 1024; k += spd)
-	     {
-		gettimeofday(&timev1, NULL);
-		i = ((a * (1024 - k)) + (b * k)) >> 10;
-		ewin->h = i;
-		EMoveResizeWindow(disp, ewin->win_container,
-				  ewin->border->border.left,
-				  ewin->border->border.top, ewin->client.w,
-				  ewin->h - ewin->border->border.top -
-				  ewin->border->border.bottom);
-		EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
-				  ewin->h);
-		CalcEwinSizes(ewin);
-		if (ewin->client.shaped)
-		   EShapeCombineShape(disp, ewin->win_container,
-				      ShapeBounding, 0,
-				      -(ewin->client.h -
-					(ewin->h - ewin->border->border.top -
-					 ewin->border->border.bottom)),
-				      ewin->client.win, ShapeBounding,
-				      ShapeSet);
-		PropagateShapes(ewin->win);
-		gettimeofday(&timev2, NULL);
-		dsec = timev2.tv_sec - timev1.tv_sec;
-		dusec = timev2.tv_usec - timev1.tv_usec;
-		if (dusec < 0)
-		  {
-		     dsec--;
-		     dusec += 1000000;
-		  }
-		tm = (double)dsec + (((double)dusec) / 1000000);
-		spd = (int)((double)speed * tm);
-		if (spd < min)
-		   spd = min;
-	     }
+	  {
+	     ETimedLoopInit(0, 1024, speed);
+	     for (k = 0; k <= 1024;)
+	       {
+		  i = ((a * (1024 - k)) + (b * k)) >> 10;
+		  ewin->h = i;
+		  EMoveResizeWindow(disp, ewin->win_container,
+				    ewin->border->border.left,
+				    ewin->border->border.top, ewin->client.w,
+				    ewin->h - ewin->border->border.top -
+				    ewin->border->border.bottom);
+		  EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
+				    ewin->h);
+		  CalcEwinSizes(ewin);
+		  if (ewin->client.shaped)
+		     EShapeCombineShape(disp, ewin->win_container,
+					ShapeBounding, 0,
+					-(ewin->client.h -
+					  (ewin->h - ewin->border->border.top -
+					   ewin->border->border.bottom)),
+					ewin->client.win, ShapeBounding,
+					ShapeSet);
+		  EwinPropagateShapes(ewin);
+
+		  k = ETimedLoopNext();
+	       }
+	  }
 	ewin->h = b;
 	MoveResizeEwin(ewin, ewin->x, ewin->y, ewin->client.w, ewin->client.h);
 	XSync(disp, False);
@@ -3160,12 +3067,11 @@ EwinUnShade(EWin * ewin)
      case 3:
 	att.win_gravity = SouthGravity;
 	XChangeWindowAttributes(disp, ewin->client.win, CWWinGravity, &att);
-	a = ewin->border->border.bottom;
-	b = ewin->client.h + ewin->border->border.top +
-	   ewin->border->border.bottom;
+	a = ewin->border->border.top + ewin->border->border.bottom;
+	b = ewin->client.h + a;
 	c = ewin->y;
-	d = ewin->y + ewin->h - (ewin->border->border.bottom +
-				 ewin->client.h + ewin->border->border.top);
+	d = ewin->y + ewin->h - (ewin->client.h + a);
+	a++;
 	ewin->shaded = 0;
 	EMoveResizeWindow(disp, ewin->win_container,
 			  ewin->border->border.left, ewin->border->border.top,
@@ -3175,39 +3081,31 @@ EwinUnShade(EWin * ewin)
 	EMapWindow(disp, ewin->client.win);
 	EMapWindow(disp, ewin->win_container);
 	if ((Conf.animate_shading) || (ewin->menu))
-	   for (k = 0; k <= 1024; k += spd)
-	     {
-		gettimeofday(&timev1, NULL);
-		i = ((a * (1024 - k)) + (b * k)) >> 10;
-		j = ((c * (1024 - k)) + (d * k)) >> 10;
-		ewin->h = i;
-		ewin->y = j;
-		EMoveResizeWindow(disp, ewin->win_container,
-				  ewin->border->border.left,
-				  ewin->border->border.top, ewin->client.w,
-				  ewin->h - ewin->border->border.top -
-				  ewin->border->border.bottom);
-		EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
-				  ewin->h);
-		CalcEwinSizes(ewin);
-		if (ewin->client.shaped)
-		   EShapeCombineShape(disp, ewin->win_container,
-				      ShapeBounding, 0, 0, ewin->client.win,
-				      ShapeBounding, ShapeSet);
-		PropagateShapes(ewin->win);
-		gettimeofday(&timev2, NULL);
-		dsec = timev2.tv_sec - timev1.tv_sec;
-		dusec = timev2.tv_usec - timev1.tv_usec;
-		if (dusec < 0)
-		  {
-		     dsec--;
-		     dusec += 1000000;
-		  }
-		tm = (double)dsec + (((double)dusec) / 1000000);
-		spd = (int)((double)speed * tm);
-		if (spd < min)
-		   spd = min;
-	     }
+	  {
+	     ETimedLoopInit(0, 1024, speed);
+	     for (k = 0; k <= 1024;)
+	       {
+		  i = ((a * (1024 - k)) + (b * k)) >> 10;
+		  j = ((c * (1024 - k)) + (d * k)) >> 10;
+		  ewin->h = i;
+		  ewin->y = j;
+		  EMoveResizeWindow(disp, ewin->win_container,
+				    ewin->border->border.left,
+				    ewin->border->border.top, ewin->client.w,
+				    ewin->h - ewin->border->border.top -
+				    ewin->border->border.bottom);
+		  EMoveResizeWindow(disp, ewin->win, ewin->x, ewin->y, ewin->w,
+				    ewin->h);
+		  CalcEwinSizes(ewin);
+		  if (ewin->client.shaped)
+		     EShapeCombineShape(disp, ewin->win_container,
+					ShapeBounding, 0, 0, ewin->client.win,
+					ShapeBounding, ShapeSet);
+		  EwinPropagateShapes(ewin);
+
+		  k = ETimedLoopNext();
+	       }
+	  }
 	ewin->h = b;
 	ewin->y = d;
 	MoveResizeEwin(ewin, ewin->x, ewin->y, ewin->client.w, ewin->client.h);
@@ -3222,7 +3120,7 @@ EwinUnShade(EWin * ewin)
    if (ewin->client.shaped)
       EShapeCombineShape(disp, ewin->win_container, ShapeBounding, 0, 0,
 			 ewin->client.win, ShapeBounding, ShapeSet);
-   PropagateShapes(ewin->win);
+   EwinPropagateShapes(ewin);
 
    Mode.queue_up = pq;
 
@@ -3242,6 +3140,8 @@ EwinSetFullscreen(EWin * ewin, int on)
 {
    int                 x, y, w, h;
    Border             *b;
+   EWin              **lst;
+   int                 i, num;
 
    if (ewin->st.fullscreen == on)
       return;
@@ -3254,9 +3154,20 @@ EwinSetFullscreen(EWin * ewin, int on)
 	ewin->lh = ewin->client.h;
 	ewin->ll = ewin->layer;
 	ScreenGetAvailableArea(ewin->x, ewin->y, &x, &y, &w, &h);
-#if 0
-	ewin->layer = 10;
-#endif
+
+	if (Conf.place.raise_fullscreen)
+	  {
+	     ewin->layer = 8;
+	     lst = EwinListTransients(ewin, &num, 0);
+	     for (i = 0; i < num; i++)
+	       {
+		  lst[i]->ll = lst[i]->layer;
+		  lst[i]->layer += ewin->layer - ewin->ll;
+	       }
+	     if (lst)
+		Efree(lst);
+	  }
+
 	ewin->fixedpos = 1;
 	b = (Border *) FindItem("BORDERLESS", 0, LIST_FINDBY_NAME,
 				LIST_TYPE_BORDER);
@@ -3267,9 +3178,18 @@ EwinSetFullscreen(EWin * ewin, int on)
 	y = ewin->ly;
 	w = ewin->lw;
 	h = ewin->lh;
-	ewin->layer = ewin->ll;
 	ewin->fixedpos = 0;	/* Yeah - well */
 	b = ewin->normal_border;
+
+	if (Conf.place.raise_fullscreen)
+	  {
+	     lst = EwinListTransients(ewin, &num, 0);
+	     for (i = 0; i < num; i++)
+		lst[i]->layer = lst[i]->ll;
+	     if (lst)
+		Efree(lst);
+	  }
+	ewin->layer = ewin->ll;
      }
    ewin->st.fullscreen = on;
    RaiseEwin(ewin);
@@ -3367,6 +3287,54 @@ BorderWinpartIndex(EWin * ewin, Window win)
    return -1;			/* Not found */
 }
 
+/*
+ * Save current position in absolute viewport coordinates
+ */
+void
+EwinRememberPositionSet(EWin * ewin)
+{
+   int                 ax, ay;
+
+   ewin->req_x = ewin->x;
+   ewin->req_y = ewin->y;
+   if (!ewin->sticky)
+     {
+	ax = desks.desk[ewin->desktop].current_area_x;
+	ay = desks.desk[ewin->desktop].current_area_y;
+	ewin->req_x += ax * VRoot.w;
+	ewin->req_y += ay * VRoot.h;
+     }
+}
+
+/*
+ * Get saved position in relative viewport coordinates
+ */
+void
+EwinRememberPositionGet(EWin * ewin, int *px, int *py)
+{
+   int                 x, y, ax, ay;
+
+   x = ewin->req_x;
+   y = ewin->req_y;
+   if (!ewin->sticky)
+     {
+	ax = desks.desk[ewin->desktop].current_area_x;
+	ay = desks.desk[ewin->desktop].current_area_y;
+	x -= ax * VRoot.w;
+	y -= ay * VRoot.h;
+     }
+
+   *px = x;
+   *py = y;
+}
+
+void
+EwinPropagateShapes(EWin * ewin)
+{
+   if (!ewin->docked)
+      PropagateShapes(ewin->win);
+}
+
 /*
  * Change requests
  */
@@ -3501,11 +3469,44 @@ EwinsSetFree(void)
 	   continue;
 
 	/* This makes E determine the client window stacking at exit */
+	EwinInstantUnShade(ewin);
 	EReparentWindow(disp, ewin->client.win, VRoot.win,
 			ewin->client.x, ewin->client.y);
      }
 }
 
+void
+EwinsShowDesktop(int on)
+{
+   EWin               *const *lst, *ewin;
+   int                 i, num;
+
+   lst = EwinListGetForDesktop(desks.current, &num);
+
+   for (i = 0; i < num; i++)
+     {
+	ewin = lst[i];
+
+	if (on)
+	  {
+	     if (ewin->internal || ewin->iconified || ewin->props.donthide ||
+		 ewin->client.transient)
+		continue;
+
+	     ewin->st.showingdesk = 1;
+	     IconifyEwin(ewin);
+	  }
+	else
+	  {
+	     if (!ewin->st.showingdesk)
+		continue;
+
+	     DeIconifyEwin(ewin);
+	  }
+     }
+   EWMH_SetShowingDesktop(on);
+}
+
 /*
  * Border event handlers
  */
@@ -3517,7 +3518,7 @@ BorderWinpartEventExpose(XEvent * ev, EWin * ewin, int j)
    ewin->bits[j].no_expose = 0;
    ewin->bits[j].expose = 1;
    if (BorderWinpartDraw(ewin, j) && IsPropagateEwinOnQueue(ewin))
-      PropagateShapes(ewin->win);
+      EwinPropagateShapes(ewin);
    return;
    ev = NULL;
 }
@@ -3541,11 +3542,13 @@ BorderWinpartEventMouseUp(XEvent * ev, EWin * ewin, int j)
       ewin->bits[j].state = STATE_HILITED;
    else
       ewin->bits[j].state = STATE_NORMAL;
-   ewin->bits[j].left = 0;
    BorderWinpartChange(ewin, j, 0);
 
-   if (ewin->bits[j].win == Mode.context_win && ewin->border->part[j].aclass)
+   if (ewin->bits[j].win == Mode.context_win && (!ewin->bits[j].left) &&
+       ewin->border->part[j].aclass)
       EventAclass(ev, ewin, ewin->border->part[j].aclass);
+
+   ewin->bits[j].left = 0;
 }
 
 static void
@@ -3590,18 +3593,25 @@ static int
 BordersEvent(XEvent * ev, border_event_func_t * func)
 {
    Window              win = ev->xany.window;
-   EWin               *const *ewins;
+   EWin               *const *ewins, *ewin;
    int                 i, j, num, used = 0;
 
    ewins = EwinListGetAll(&num);
    for (i = 0; i < num; i++)
      {
-	for (j = 0; j < ewins[i]->border->num_winparts; j++)
+	ewin = ewins[i];
+	if (win == ewin->win)
 	  {
-	     if (win == ewins[i]->bits[j].win)
+	     if (ewin->border->aclass)
+		EventAclass(ev, ewin, ewin->border->aclass);
+	     used = 0;
+	     goto done;
+	  }
+	for (j = 0; j < ewin->border->num_winparts; j++)
+	  {
+	     if (win == ewin->bits[j].win)
 	       {
-		  func(ev, ewins[i], j);
-
+		  func(ev, ewin, j);
 		  used = 1;
 		  goto done;
 	       }
