@@ -103,6 +103,7 @@
 #define CONTROL_MANUAL_PLACEMENT 343
 #define CONTROL_MANUAL_PLACEMENT_MOUSE_POINTER 3360
 #define CONTROL_PLACE_IGNORE_STRUTS 3361
+#define CONTROL_RAISE_FULLSCREEN 3362
 #define CONTROL_RAISE_ON_NEXT_FOCUS 344
 #define CONTROL_WARP_ON_NEXT_FOCUS 345
 #define CONTROL_EDGE_FLIP_RESISTANCE 346
@@ -155,6 +156,9 @@
 #define CONTROL_ST_PAGER 1383
 #define CONTROL_ST_WARPLIST 1384
 
+#define CONTROL_MENU_NAVIGATION_KEYS 1390
+#define CONTROL_HINTS_XROOT_HACK 1391
+
 #define ICLASS_NAME 350
 #define ICLASS_NORMAL 351
 #define ICLASS_CLICKED 352
