@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,19 +21,32 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
+#include "X11/cursorfont.h"
+
+struct _ecursor
+{
+   char               *name;
+#if 0				/* Not used */
+   Imlib_Color         fg, bg;
+#endif
+   char               *file;
+   Cursor              cursor;
+   unsigned int        ref_count;
+#if 0				/* Not used */
+   char                inroot;
+#endif
+};
 
 ECursor            *
-CreateECursor(char *name, char *image, int native_id,
-	      ImlibColor * fg, ImlibColor * bg)
+ECursorCreate(const char *name, const char *image, int native_id, XColor * fg,
+	      XColor * bg)
 {
    Cursor              curs;
-   XColor              xfg, xbg;
    Pixmap              pmap, mask;
    int                 xh, yh;
    unsigned int        w, h, ww, hh;
    char               *img, msk[FILEPATH_LEN_MAX];
    ECursor            *ec;
-   int                 r, g, b;
 
    if ((!name) || (!image && native_id == -1))
       return NULL;
@@ -49,9 +62,9 @@ CreateECursor(char *name, char *image, int native_id,
 	mask = 0;
 	xh = 0;
 	yh = 0;
-	XReadBitmapFile(disp, root.win, msk, &w, &h, &mask, &xh, &yh);
-	XReadBitmapFile(disp, root.win, img, &w, &h, &pmap, &xh, &yh);
-	XQueryBestCursor(disp, root.win, w, h, &ww, &hh);
+	XReadBitmapFile(disp, VRoot.win, msk, &w, &h, &mask, &xh, &yh);
+	XReadBitmapFile(disp, VRoot.win, img, &w, &h, &pmap, &xh, &yh);
+	XQueryBestCursor(disp, VRoot.win, w, h, &ww, &hh);
 	if ((w > ww) || (h > hh))
 	  {
 	     EFreePixmap(disp, pmap);
@@ -59,68 +72,75 @@ CreateECursor(char *name, char *image, int native_id,
 	     Efree(img);
 	     return NULL;
 	  }
-	r = fg->r;
-	g = fg->g;
-	b = fg->b;
-	xfg.red = (fg->r << 8) | (fg->r);
-	xfg.green = (fg->g << 8) | (fg->g);
-	xfg.blue = (fg->b << 8) | (fg->b);
-	xfg.pixel = Imlib_best_color_match(pImlibData, &r, &g, &b);
-	r = bg->r;
-	g = bg->g;
-	b = bg->b;
-	xbg.red = (bg->r << 8) | (bg->r);
-	xbg.green = (bg->g << 8) | (bg->g);
-	xbg.blue = (bg->b << 8) | (bg->b);
-	xbg.pixel = Imlib_best_color_match(pImlibData, &r, &g, &b);
+
+	EAllocColor(fg);
+	EAllocColor(bg);
+
 	curs = 0;
-	curs = XCreatePixmapCursor(disp, pmap, mask, &xfg, &xbg, xh, yh);
+	curs = XCreatePixmapCursor(disp, pmap, mask, fg, bg, xh, yh);
 	EFreePixmap(disp, pmap);
 	EFreePixmap(disp, mask);
 	Efree(img);
      }
    else
      {
-	curs = XCreateFontCursor(disp, native_id);
+	curs = (native_id == 999) ? None : XCreateFontCursor(disp, native_id);
      }
 
    ec = Emalloc(sizeof(ECursor));
-   ec->name = duplicate(name);
-   ec->file = duplicate(image);
+   ec->name = Estrdup(name);
+   ec->file = Estrdup(image);
+#if 0				/* Not used */
    ec->fg = *fg;
    ec->bg = *bg;
+#endif
    ec->cursor = curs;
    ec->ref_count = 0;
+#if 0				/* Not used */
    ec->inroot = 0;
+#endif
+
+   AddItem(ec, ec->name, 0, LIST_TYPE_ECURSOR);
 
    return ec;
 }
 
 void
-ApplyECursor(Window win, ECursor * ec)
+ECursorApply(ECursor * ec, Window win)
 {
    if (!ec)
       return;
    XDefineCursor(disp, win, ec->cursor);
+#if 0				/* Not used */
+   if (win == VRoot.win)
+      ec->inroot = 1;
+#endif
+}
 
-   return;
+static              Cursor
+ECursorGetByName(const char *name, unsigned int fallback)
+{
+   ECursor            *ec;
+
+   ec = FindItem(name, 0, LIST_FINDBY_NAME, LIST_TYPE_ECURSOR);
+   if (!ec)
+      return XCreateFontCursor(disp, fallback);
+
+   ECursorIncRefcount(ec);
+
+   return ec->cursor;
 }
 
 void
-FreeECursor(ECursor * ec)
+ECursorDestroy(ECursor * ec)
 {
-
    if (!ec)
       return;
 
    if (ec->ref_count > 0)
      {
-	char                stuff[255];
-
-	Esnprintf(stuff, sizeof(stuff), _("%u references remain\n"),
-		  ec->ref_count);
-	DIALOG_OK(_("ECursor Error!"), stuff);
-
+	DialogOK(_("ECursor Error!"), _("%u references remain\n"),
+		 ec->ref_count);
 	return;
      }
 
@@ -131,6 +151,58 @@ FreeECursor(ECursor * ec)
    if (ec->file)
       Efree(ec->file);
    Efree(ec);
+}
 
-   return;
+void
+ECursorIncRefcount(ECursor * ec)
+{
+   if (ec)
+      ec->ref_count++;
+}
+
+void
+ECursorDecRefcount(ECursor * ec)
+{
+   if (ec)
+      ec->ref_count--;
+}
+
+const char         *
+ECursorGetName(ECursor * ec)
+{
+   return (ec) ? ec->name : 0;
+}
+
+int
+ECursorGetRefcount(ECursor * ec)
+{
+   return (ec) ? ec->ref_count : 0;
+}
+
+static Cursor       ECsrs[ECSR_COUNT];
+
+Cursor
+ECsrGet(int which)
+{
+   return (which >= 0 && which < ECSR_COUNT) ? ECsrs[which] : None;
+}
+
+void
+ECsrApply(int which, Window win)
+{
+   XDefineCursor(disp, win, ECsrGet(which));
+}
+
+/*
+ * Set up some basic cursors
+ */
+void
+ECursorsInit(void)
+{
+   ECsrs[ECSR_NONE] = None;
+   ECsrs[ECSR_ROOT] = ECursorGetByName("DEFAULT", XC_left_ptr);
+   ECsrs[ECSR_GRAB] = ECursorGetByName("GRAB", XC_crosshair);
+   ECsrs[ECSR_PGRAB] = ECursorGetByName("PGRAB", XC_X_cursor);
+   ECsrs[ECSR_ACT_MOVE] = ECursorGetByName("GRAB_MOVE", XC_fleur);
+   ECsrs[ECSR_ACT_RESIZE] = ECursorGetByName("GRAB_RESIZE", XC_sizing);
 }
