@@ -23,7 +23,6 @@
 #define DECLARE_STRUCT_BUTTON
 #include "E.h"
 #include <time.h>
-#include <sys/time.h>
 
 #define EDESK_EVENT_MASK \
   (KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask | \
@@ -188,37 +187,25 @@ MoveToDeskBottom(int num)
 void
 SlideWindowTo(Window win, int fx, int fy, int tx, int ty, int speed)
 {
-   int                 k, spd, x, y, min;
-   struct timeval      timev1, timev2;
-   int                 dsec, dusec;
-   double              tm;
+   int                 k, x, y;
 
    EDBUG(5, "SlideWindowTo");
-   spd = 16;
-   min = 2;
+
    GrabX();
-   for (k = 0; k <= 1024; k += spd)
+
+   ETimedLoopInit(0, 1024, speed);
+   for (k = 0; k <= 1024;)
      {
-	gettimeofday(&timev1, NULL);
 	x = ((fx * (1024 - k)) + (tx * k)) >> 10;
 	y = ((fy * (1024 - k)) + (ty * k)) >> 10;
 	EMoveWindow(disp, win, x, y);
 	XSync(disp, False);
-	gettimeofday(&timev2, NULL);
-	dsec = timev2.tv_sec - timev1.tv_sec;
-	dusec = timev2.tv_usec - timev1.tv_usec;
-	if (dusec < 0)
-	  {
-	     dsec--;
-	     dusec += 1000000;
-	  }
-	tm = (double)dsec + (((double)dusec) / 1000000);
-	spd = (int)((double)speed * tm);
-	if (spd < min)
-	   spd = min;
+
+	k = ETimedLoopNext();
      }
    EMoveWindow(disp, win, tx, ty);
    UngrabX();
+
    EDBUG_RETURN_;
 }
 
@@ -588,6 +575,9 @@ DesktopSetBg(int desk, Background * bg, int refresh)
    if (refresh)
       BackgroundPixmapFree(desks.desk[desk].bg);
 
+   if (bg && !strcmp(BackgroundGetName(bg), "NONE"))
+      bg = NULL;
+
    if (desks.desk[desk].bg != bg)
      {
 	if (desks.desk[desk].bg)
@@ -682,11 +672,10 @@ DesktopAt(int x, int y)
 static void
 MoveStickyWindowsToCurrentDesk(void)
 {
-   EWin               *const *lst, *ewin, *last_ewin;
+   EWin               *const *lst, *ewin;
    int                 i, num;
 
    lst = EwinListGetStacking(&num);
-   last_ewin = NULL;
    for (i = 0; i < num; i++)
      {
 	ewin = lst[i];
@@ -700,7 +689,23 @@ MoveStickyWindowsToCurrentDesk(void)
 	EMoveWindow(disp, ewin->win, ewin->x, ewin->y);
 	HintsSetWindowArea(ewin);
 	HintsSetWindowDesktop(ewin);
-	last_ewin = ewin;
+     }
+}
+
+static void
+MoveStickyButtonsToCurrentDesk(void)
+{
+   Button            **lst, *btn;
+   int                 i, num;
+
+   lst = (Button **) ListItemType(&num, LIST_TYPE_BUTTON);
+   for (i = 0; i < num; i++)
+     {
+	btn = lst[i];
+	if (!btn->sticky || btn->internal)
+	   continue;
+
+	ButtonMoveToDesktop(btn, desks.current);
      }
 }
 
@@ -950,6 +955,7 @@ RaiseDesktop(int desk)
    StackDesktops();
    desks.current = desk;
    MoveStickyWindowsToCurrentDesk();
+   MoveStickyButtonsToCurrentDesk();
    StackDesktop(desks.current);
    FocusNewDesk();
    FX_DeskChange();
@@ -980,6 +986,7 @@ LowerDesktop(int desk)
    StackDesktops();
    desks.current = desks.order[0];
    MoveStickyWindowsToCurrentDesk();
+   MoveStickyButtonsToCurrentDesk();
    StackDesktop(desks.current);
    FocusNewDesk();
    FX_DeskChange();
@@ -1072,77 +1079,66 @@ StackDesktop(int desk)
     * Build the window stack, top to bottom
     */
 
-   wl2 = ListProgressWindows(&wnum);
-   if (wl2)
+   if (desk == 0)
      {
-	for (i = 0; i < wnum; i++)
-	   _APPEND_TO_WIN_LIST(wl2[i]);
-	Efree(wl2);
-     }
-   if (init_win_ext)
-     {
-	_APPEND_TO_WIN_LIST(init_win_ext);
-     }
-   if (init_win1)
-     {
-	_APPEND_TO_WIN_LIST(init_win1);
-	_APPEND_TO_WIN_LIST(init_win2);
+	wl2 = ListProgressWindows(&wnum);
+	if (wl2)
+	  {
+	     for (i = 0; i < wnum; i++)
+		_APPEND_TO_WIN_LIST(wl2[i]);
+	     Efree(wl2);
+	  }
+	if (init_win_ext)
+	  {
+	     _APPEND_TO_WIN_LIST(init_win_ext);
+	  }
+	if (init_win1)
+	  {
+	     _APPEND_TO_WIN_LIST(init_win1);
+	     _APPEND_TO_WIN_LIST(init_win2);
+	  }
      }
 
    lst = EwinListGetStacking(&wnum);
    blst = (Button **) ListItemType(&bnum, LIST_TYPE_BUTTON);
 
-   /* Sticky buttons */
-   if (blst)
-     {
-	for (i = 0; i < bnum; i++)
-	  {
-	     if (!blst[i]->sticky || blst[i]->internal)
-		continue;
-
-	     _APPEND_TO_WIN_LIST(blst[i]->win);
-	  }
-     }
-
    /* Floating EWins */
-   if (lst)
-     {
-	for (i = 0; i < wnum; i++)
-	  {
-	     if (!lst[i]->floating)
-		continue;
-
-	     _APPEND_TO_WIN_LIST(lst[i]->win);
-	  }
-     }
-
-   /* The virtual desktop windows */
-   for (i = 0; i < ENLIGHTENMENT_CONF_NUM_DESKTOPS; i++)
-     {
-	if (desks.order[i] == 0)
-	   break;
-
-	_APPEND_TO_WIN_LIST(desks.desk[desks.order[i]].win);
-     }
-
-   /* Non-sticky, "above" buttons */
-   if (blst)
-     {
-	for (i = 0; i < bnum; i++)
-	  {
-	     if (blst[i]->desktop != desk || blst[i]->ontop != 1 ||
-		 blst[i]->sticky || blst[i]->internal)
-		continue;
-
-	     _APPEND_TO_WIN_LIST(blst[i]->win);
-	  }
-     }
-
-   /* Normal EWins on this desk */
    for (i = 0; i < wnum; i++)
      {
 	ewin = lst[i];
-	if (EwinGetDesk(ewin) != desk || ewin->floating)
+	if (!ewin->floating || EwinGetDesk(ewin) != desk)
+	   continue;
+
+	_APPEND_TO_WIN_LIST(ewin->win);
+     }
+
+   /* "Above" buttons */
+   for (i = 0; i < bnum; i++)
+     {
+	if (blst[i]->internal || blst[i]->desktop != desk ||
+	    blst[i]->ontop != 1)
+	   continue;
+
+	_APPEND_TO_WIN_LIST(blst[i]->win);
+     }
+
+   if (desk == 0)
+     {
+	/* The virtual desktop windows */
+	for (i = 0; i < ENLIGHTENMENT_CONF_NUM_DESKTOPS; i++)
+	  {
+	     if (desks.order[i] == 0)
+		break;
+
+	     _APPEND_TO_WIN_LIST(desks.desk[desks.order[i]].win);
+	  }
+     }
+
+   /* Normal EWins on this desk (except Desktops) */
+   for (i = 0; i < wnum; i++)
+     {
+	ewin = lst[i];
+	if (ewin->floating || EwinGetDesk(ewin) != desk || ewin->layer == 0)
 	   continue;
 
 	_APPEND_TO_WIN_LIST(ewin->win);
@@ -1150,21 +1146,35 @@ StackDesktop(int desk)
 	   _APPEND_TO_WIN_LIST(Mode.menus.cover_win);
      }
 
-   /* Non-sticky, "below" buttons */
-   if (blst)
+   /* "Normal" buttons */
+   for (i = 0; i < bnum; i++)
      {
-	for (i = 0; i < bnum; i++)
-	  {
-	     if (blst[i]->desktop != desk || blst[i]->ontop != -1 ||
-		 blst[i]->sticky || blst[i]->internal)
-		continue;
+	if (blst[i]->internal || blst[i]->desktop != desk ||
+	    blst[i]->ontop != 0)
+	   continue;
 
-	     _APPEND_TO_WIN_LIST(blst[i]->win);
-	  }
+	_APPEND_TO_WIN_LIST(blst[i]->win);
      }
 
-   /* The current (virtual) root window */
-   _APPEND_TO_WIN_LIST(desks.desk[desk].win);
+   /* "Below" buttons */
+   for (i = 0; i < bnum; i++)
+     {
+	if (blst[i]->internal || blst[i]->desktop != desk ||
+	    blst[i]->ontop != -1)
+	   continue;
+
+	_APPEND_TO_WIN_LIST(blst[i]->win);
+     }
+
+   /* Desktop type EWins on this desk */
+   for (i = 0; i < wnum; i++)
+     {
+	ewin = lst[i];
+	if (ewin->floating || EwinGetDesk(ewin) != desk || ewin->layer != 0)
+	   continue;
+
+	_APPEND_TO_WIN_LIST(ewin->win);
+     }
 
    if (EventDebug(EDBUG_TYPE_STACKING))
      {
