@@ -552,6 +552,14 @@ HandleMouseIn(XEvent * ev)
 
    EDBUG(5, "HandleMouseIn");
 
+   if (ev->xcrossing.mode == NotifyGrab &&
+       ev->xcrossing.detail == NotifyInferior)
+     {
+	Mode.grabs.pointer_grab_window = win;
+	if (!Mode.grabs.pointer_grab_active)
+	   Mode.grabs.pointer_grab_active = 2;
+     }
+
    if (Mode.mode != MODE_NONE)
       EDBUG_RETURN_;
 
@@ -591,6 +599,13 @@ HandleMouseOut(XEvent * ev)
 
    EDBUG(5, "HandleMouseOut");
 
+   if (ev->xcrossing.mode == NotifyGrab &&
+       ev->xcrossing.detail == NotifyInferior)
+     {
+	Mode.grabs.pointer_grab_window = None;
+	Mode.grabs.pointer_grab_active = 0;
+     }
+
    if (Mode.mode != MODE_NONE)
       EDBUG_RETURN_;
 
@@ -891,7 +906,7 @@ HandleConfigureRequest(XEvent * ev)
 	   if (pshaped != ewin->client.shaped)
 	     {
 		SyncBorderToEwin(ewin);
-		PropagateShapes(ewin->win);
+		EwinPropagateShapes(ewin);
 	     }
 	}
 	ReZoom(ewin);
@@ -933,7 +948,7 @@ HandleResizeRequest(XEvent * ev)
 	   if (pshaped != ewin->client.shaped)
 	     {
 		SyncBorderToEwin(ewin);
-		PropagateShapes(ewin->win);
+		EwinPropagateShapes(ewin);
 	     }
 	}
 	ReZoom(ewin);
@@ -1021,7 +1036,7 @@ HandleChildShapeChange(XEvent * ev)
 	b = ewin->border;
 	SyncBorderToEwin(ewin);
 	if (ewin->border == b)
-	   PropagateShapes(ewin->win);
+	   EwinPropagateShapes(ewin);
      }
 
    EDBUG_RETURN_;
