@@ -694,6 +694,7 @@ EWMH_GetWindowType(EWin * ewin)
 	ewin->skipfocus = 1;
 	ewin->fixedpos = 1;
 	EwinSetBorderByName(ewin, "BORDERLESS", 0);
+	ewin->props.donthide = 1;
      }
    else if (atom == _NET_WM_WINDOW_TYPE_DOCK)
      {
@@ -702,6 +703,7 @@ EWMH_GetWindowType(EWin * ewin)
 	ewin->skipfocus = 1;
 	ewin->sticky = 1;
 	ewin->never_use_area = 1;
+	ewin->props.donthide = 1;
      }
    else if (atom == _NET_WM_WINDOW_TYPE_UTILITY)
      {
@@ -710,6 +712,7 @@ EWMH_GetWindowType(EWin * ewin)
 	ewin->skipwinlist = 1;
 	ewin->skipfocus = 1;
 	ewin->never_use_area = 1;
+	ewin->props.donthide = 1;
      }
 #if 0				/* Not used by E (yet?) */
    else if (atom == _NET_WM_WINDOW_TYPE_TOOLBAR)
@@ -864,9 +867,7 @@ EWMH_ProcessClientMessage(XClientMessageEvent * event)
      }
    else if (event->message_type == _NET_SHOWING_DESKTOP)
      {
-	Eprintf("EWMH_ProcessClientMessage: _NET_SHOWING_DESKTOP: %ld\n",
-		event->data.l[0]);
-	EWMH_SetShowingDesktop(event->data.l[0]);
+	EwinsShowDesktop(event->data.l[0]);
 	goto done;
      }
 
@@ -1014,6 +1015,20 @@ EWMH_ProcessClientMessage(XClientMessageEvent * event)
      {
 	switch (event->data.l[2])
 	  {
+	  case _NET_WM_MOVERESIZE_SIZE_TOPLEFT:
+	  case _NET_WM_MOVERESIZE_SIZE_TOP:
+	  case _NET_WM_MOVERESIZE_SIZE_TOPRIGHT:
+	  case _NET_WM_MOVERESIZE_SIZE_RIGHT:
+	  case _NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT:
+	  case _NET_WM_MOVERESIZE_SIZE_BOTTOM:
+	  case _NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT:
+	  case _NET_WM_MOVERESIZE_SIZE_LEFT:
+	     ActionsCall(ACTION_RESIZE, ewin, NULL);
+	     break;
+	  case _NET_WM_MOVERESIZE_MOVE:
+	     ActionsCall(ACTION_MOVE, ewin, NULL);
+	     break;
+
 	  case _NET_WM_MOVERESIZE_SIZE_KEYBOARD:
 	     /* doResize(NULL); */
 	     break;
