@@ -510,76 +510,6 @@ atchar(const char *s, char c)
    EDBUG_RETURN(NULL);
 }
 
-char               *
-getword(char *s, int num)
-{
-
-   /* *********FIXME**************
-    * This function is broken but it isn't in use so I'll fix it later
-    * (DO NOT USE UNTIL FIXED
-    */
-   int                 cnt, i;
-   char               *start, *finish, *ss, *w;
-   char               *wd = NULL;
-
-   EDBUG(9, "getword");
-   if (!s)
-      EDBUG_RETURN(NULL);
-   if (!wd)
-      EDBUG_RETURN(NULL);
-   if (num <= 0)
-     {
-	*wd = 0;
-	EDBUG_RETURN(NULL);
-     }
-   cnt = 0;
-   i = 0;
-   start = NULL;
-   finish = NULL;
-   ss = NULL;
-   w = wd;
-
-   while (s[i])
-     {
-	if ((cnt == num) && ((s[i] == ' ') || (s[i] == '\t')))
-	  {
-	     finish = &s[i];
-	     break;
-	  }
-	if ((s[i] != ' ') && (s[i] != '\t'))
-	  {
-	     if (i == 0)
-	       {
-		  cnt++;
-		  if (cnt == num)
-		     start = &s[i];
-	       }
-	     else if ((s[i - 1] == ' ') || (s[i - 1] == '\t'))
-	       {
-		  cnt++;
-		  if (cnt == num)
-		     start = &s[i];
-	       }
-	  }
-	i++;
-     }
-   if (cnt == num)
-     {
-	if ((start) && (finish))
-	  {
-	     for (ss = start; ss < finish; ss++)
-		*wd++ = *ss;
-	  }
-	else if (start)
-	  {
-	     for (ss = start; *ss != 0; ss++)
-		*wd++ = *ss;
-	  }
-	*wd = 0;
-     }
-   EDBUG_RETURN(wd);
-}
-
 void
 word(const char *s, int num, char *wd)
 {
