@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -25,109 +25,108 @@
 EWin               *
 FindEwinByBase(Window win)
 {
-   EWin              **ewins;
-   EWin               *ewin;
+   EWin               *const *ewins;
    int                 i, num;
 
    EDBUG(6, "FindEwinByBase");
-   ewins = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
+   ewins = EwinListGetAll(&num);
    for (i = 0; i < num; i++)
      {
 	if (win == ewins[i]->win)
-	  {
-	     ewin = ewins[i];
-	     Efree(ewins);
-	     EDBUG_RETURN(ewin);
-	  }
+	   return ewins[i];
      }
-   if (ewins)
-      Efree(ewins);
    EDBUG_RETURN(NULL);
 }
 
 EWin               *
 FindEwinByChildren(Window win)
 {
-   EWin               *ewin;
-   EWin              **ewins;
+   EWin               *const *ewins;
    int                 i, j, num;
 
    EDBUG(6, "FindEwinByChildren");
 
-   ewins = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
+   ewins = EwinListGetAll(&num);
    for (i = 0; i < num; i++)
      {
 	if ((win == ewins[i]->client.win) || (win == ewins[i]->win_container))
 	  {
-	     ewin = ewins[i];
-	     Efree(ewins);
-	     EDBUG_RETURN(ewin);
+	     return ewins[i];
 	  }
 	else
 	  {
 	     for (j = 0; j < ewins[i]->border->num_winparts; j++)
 		if (win == ewins[i]->bits[j].win)
 		  {
-		     ewin = ewins[i];
-		     Efree(ewins);
-		     EDBUG_RETURN(ewin);
+		     return ewins[i];
 		  }
 	  }
      }
-   if (ewins)
-      Efree(ewins);
    EDBUG_RETURN(NULL);
 }
 
+EWin               *
+FindEwinByPartial(const char *match, int type)
+{
+   EWin               *ewin = NULL;
+   EWin               *const *ewins;
+   int                 i, num, len;
+   char                ewinid[FILEPATH_LEN_MAX];
+
+   EDBUG(6, "FindEwinByPartial");
+
+   len = strlen(match);
+   if (len <= 0)
+      goto done;
+
+   ewins = EwinListGetAll(&num);
+   if (ewins == NULL)
+      goto done;
+
+   for (i = 0; i < num; i++)
+     {
+	if (type == '+')
+	  {
+	     /* Match start of window ID */
+	     sprintf(ewinid, "%x", (unsigned)ewins[i]->client.win);
+	     if (strncmp(ewinid, match, len))
+		continue;
+	  }
+	else if (type == '=')
+	  {
+	     /* Match name (substring) */
+	     if (!strstr(ewins[i]->icccm.wm_name, match))
+		continue;
+	  }
+	else
+	   goto done;
+
+	ewin = ewins[i];
+	break;
+     }
+
+ done:
+   EDBUG_RETURN(ewin);
+}
+
 EWin               *
 FindEwinByDecoration(Window win)
 {
-   EWin               *ewin;
-   EWin              **ewins;
+   EWin               *const *ewins;
    int                 i, j, num;
 
    EDBUG(6, "FindEwinByDecoration");
 
-   ewins = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
+   ewins = EwinListGetAll(&num);
    for (i = 0; i < num; i++)
      {
 	for (j = 0; j < ewins[i]->border->num_winparts; j++)
 	  {
 	     if (win == ewins[i]->bits[j].win)
-	       {
-		  ewin = ewins[i];
-		  Efree(ewins);
-		  EDBUG_RETURN(ewin);
-	       }
+		return ewins[i];
 	  }
      }
-   if (ewins)
-      Efree(ewins);
-   EDBUG_RETURN(NULL);
-}
 
-Button             *
-FindButton(Window win)
-{
-   Button             *b;
-   Button            **buttons;
-   int                 i, num;
-
-   EDBUG(6, "FindButton");
-
-   buttons = (Button **) ListItemType(&num, LIST_TYPE_BUTTON);
-   for (i = 0; i < num; i++)
-     {
-	if ((win == buttons[i]->win) || (win == buttons[i]->inside_win)
-	    || (win == buttons[i]->event_win))
-	  {
-	     b = buttons[i];
-	     Efree(buttons);
-	     EDBUG_RETURN(b);
-	  }
-     }
-   if (buttons)
-      Efree(buttons);
    EDBUG_RETURN(NULL);
 }
 
@@ -139,9 +138,11 @@ FindActionClass(Window win)
    int                 i;
 
    EDBUG(6, "FindActionClass");
+
    b = FindButton(win);
    if (b)
-      EDBUG_RETURN(b->aclass);
+      EDBUG_RETURN(ButtonGetAClass(b));
+
    ewin = FindEwinByDecoration(win);
    if (ewin)
      {
@@ -149,7 +150,8 @@ FindActionClass(Window win)
 	   if (win == ewin->bits[i].win)
 	      EDBUG_RETURN(ewin->border->part[i].aclass);
      }
-   for (i = 0; i < mode.numdesktops; i++)
+
+   for (i = 0; i < Conf.desks.num; i++)
      {
 	ActionClass        *ac;
 
@@ -160,81 +162,25 @@ FindActionClass(Window win)
 	     EDBUG_RETURN(ac);
 	  }
      }
-   EDBUG_RETURN(NULL);
-}
 
-Menu               *
-FindMenuItem(Window win, MenuItem ** mi)
-{
-   Menu               *menu;
-   Menu              **menus;
-   int                 i, j, num;
-
-   EDBUG(6, "FindMenuItem");
-
-   menus = (Menu **) ListItemType(&num, LIST_TYPE_MENU);
-   for (i = 0; i < num; i++)
-     {
-	for (j = 0; j < menus[i]->num; j++)
-	  {
-	     if ((win == menus[i]->items[j]->win)
-		 || (win == menus[i]->items[j]->icon_win))
-	       {
-		  *mi = menus[i]->items[j];
-		  menu = menus[i];
-		  Efree(menus);
-		  EDBUG_RETURN(menu);
-	       }
-	  }
-     }
-   if (menus)
-      Efree(menus);
-   EDBUG_RETURN(NULL);
-}
-
-Menu               *
-FindMenu(Window win)
-{
-   Menu               *menu;
-   Menu              **menus;
-   int                 i, num;
-
-   EDBUG(6, "FindMenu");
-   menus = (Menu **) ListItemType(&num, LIST_TYPE_MENU);
-   for (i = 0; i < num; i++)
-     {
-	if (menus[i]->win == win)
-	  {
-	     menu = menus[i];
-	     Efree(menus);
-	     EDBUG_RETURN(menu);
-	  }
-     }
-   if (menus)
-      Efree(menus);
    EDBUG_RETURN(NULL);
 }
 
 EWin               *
 FindEwinByMenu(Menu * m)
 {
-   EWin               *ewin;
-   EWin              **ewins;
+   EWin               *const *ewins;
    int                 i, num;
 
    EDBUG(6, "FindEwinByMenu");
-   ewins = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
+
+   ewins = EwinListGetAll(&num);
    for (i = 0; i < num; i++)
      {
 	if (ewins[i]->menu == m)
-	  {
-	     ewin = ewins[i];
-	     Efree(ewins);
-	     EDBUG_RETURN(ewin);
-	  }
+	   return ewins[i];
      }
-   if (ewins)
-      Efree(ewins);
+
    EDBUG_RETURN(NULL);
 }
 
@@ -392,39 +338,114 @@ ListWinGroupMembersForEwin(EWin * ewin, int action, char nogroup, int *num)
 }
 
 EWin              **
-ListTransientsFor(Window win, int *num)
+EwinListTransients(EWin * ewin, int *num, int group)
 {
-   EWin              **ewins, **lst = NULL;
+   EWin               *const *ewins, **lst, *ew;
    int                 i, j, n;
 
-   EDBUG(6, "ListTransientsFor");
+   EDBUG(6, "EwinListTransients");
 
-   ewins = (EWin **) ListItemType(&n, LIST_TYPE_EWIN);
    j = 0;
+   lst = NULL;
+
+   if (!ewin->has_transients)
+      goto done;
+
+   ewins = EwinListGetAll(&n);
+
+   /* Find regular transients */
    for (i = 0; i < n; i++)
      {
-	if (win == ewins[i]->client.transient_for)
+	ew = ewins[i];
+
+	/* Skip self-reference */
+	if (ew == ewin)
+	   continue;
+
+	if (ew->client.transient_for == ewin->client.win)
 	  {
-	     j++;
-	     lst = Erealloc(lst, sizeof(EWin *) * j);
-	     lst[j - 1] = ewins[i];
+	     lst = Erealloc(lst, (j + 1) * sizeof(EWin *));
+	     lst[j++] = ew;
 	  }
      }
-   if (ewins)
-      Efree(ewins);
+
+   if (!group)
+      goto done;
+
+   /* Group transients (if ewin is not a transient) */
+   if (ewin->client.transient)
+      goto done;
+
+   for (i = 0; i < n; i++)
+     {
+	ew = ewins[i];
+
+	/* Skip self-reference */
+	if (ew == ewin)
+	   continue;
+
+	if (ew->client.transient_for == VRoot.win &&
+	    ew->client.group == ewin->client.group)
+	  {
+	     lst = Erealloc(lst, (j + 1) * sizeof(EWin *));
+	     lst[j++] = ew;
+	  }
+     }
+
+ done:
    *num = j;
    EDBUG_RETURN(lst);
 }
 
+EWin              **
+EwinListTransientFor(EWin * ewin, int *num)
+{
+   EWin               *const *ewins, **lst, *ew;
+   int                 i, j, n;
+
+   EDBUG(6, "EwinListTransientFor");
+
+   j = 0;
+   lst = NULL;
+
+   if (!ewin->client.transient)
+      goto done;
+
+   ewins = EwinListGetAll(&n);
+   for (i = 0; i < n; i++)
+     {
+	ew = ewins[i];
+
+	/* Skip self-reference */
+	if (ew == ewin)
+	   continue;
+
+	/* Regular parent or if root trans, top level group members */
+	if ((ewin->client.transient_for == ew->client.win) ||
+	    (!ew->client.transient &&
+	     ewin->client.transient_for == VRoot.win &&
+	     ew->client.group == ewin->client.group))
+	  {
+	     lst = Erealloc(lst, (j + 1) * sizeof(EWin *));
+	     lst[j++] = ew;
+	  }
+     }
+
+ done:
+   *num = j;
+   EDBUG_RETURN(lst);
+}
+
+#if 0				/* Not used */
 EWin              **
 ListGroupMembers(Window win, int *num)
 {
-   EWin              **ewins, **lst = NULL;
+   EWin               *const *ewins, **lst = NULL;
    int                 i, j, n;
 
    EDBUG(6, "ListGroupMembers");
 
-   ewins = (EWin **) ListItemType(&n, LIST_TYPE_EWIN);
+   ewins = EwinListGetAll(&n);
    j = 0;
    for (i = 0; i < n; i++)
      {
@@ -435,178 +456,43 @@ ListGroupMembers(Window win, int *num)
 	     lst[j - 1] = ewins[i];
 	  }
      }
-   if (ewins)
-      Efree(ewins);
    *num = j;
    EDBUG_RETURN(lst);
 }
+#endif
 
 EWin               *
 FindEwinByDialog(Dialog * d)
 {
-   EWin               *ewin;
-   EWin              **ewins;
+   EWin               *const *ewins;
    int                 i, num;
 
    EDBUG(6, "FindEwinByDialog");
-   ewins = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
+
+   ewins = EwinListGetAll(&num);
    for (i = 0; i < num; i++)
      {
 	if (ewins[i]->dialog == d)
-	  {
-	     ewin = ewins[i];
-	     Efree(ewins);
-	     EDBUG_RETURN(ewin);
-	  }
+	   return ewins[i];
      }
-   if (ewins)
-      Efree(ewins);
+
    EDBUG_RETURN(NULL);
 }
 
-char
+int
 FindADialog(void)
 {
-   EWin              **ewins;
-   int                 i, num;
+   EWin               *const *ewins;
+   int                 i, num, n;
 
    EDBUG(6, "FindADialog");
-   ewins = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
-   for (i = 0; i < num; i++)
+
+   ewins = EwinListGetAll(&num);
+   for (i = n = 0; i < num; i++)
      {
 	if (ewins[i]->dialog)
-	  {
-	     Efree(ewins);
-	     EDBUG_RETURN(1);
-	  }
+	   n++;
      }
-   if (ewins)
-      Efree(ewins);
-   EDBUG_RETURN(0);
-}
 
-Dialog             *
-FindDialogButton(Window win, int *bnum)
-{
-   Dialog             *d;
-   Dialog            **ds;
-   int                 i, j, num;
-
-   EDBUG(6, "FindDialogButton");
-
-   ds = (Dialog **) ListItemType(&num, LIST_TYPE_DIALOG);
-   for (i = 0; i < num; i++)
-     {
-	for (j = 0; j < ds[i]->num_buttons; j++)
-	  {
-	     if (win == ds[i]->button[j]->win)
-	       {
-		  *bnum = j;
-		  d = ds[i];
-		  Efree(ds);
-		  EDBUG_RETURN(d);
-	       }
-	  }
-     }
-   if (ds)
-      Efree(ds);
-   *bnum = 0;
-   EDBUG_RETURN(NULL);
-}
-
-DItem              *
-FindDialogItem(Window win, Dialog ** dret)
-{
-   DItem              *di;
-   Dialog            **ds;
-   int                 i, num;
-
-   EDBUG(6, "FindDialogButton");
-
-   ds = (Dialog **) ListItemType(&num, LIST_TYPE_DIALOG);
-   for (i = 0; i < num; i++)
-     {
-	if (ds[i]->item)
-	  {
-	     if ((di = DialogItemFindWindow(ds[i]->item, win)))
-	       {
-		  *dret = ds[i];
-		  Efree(ds);
-		  EDBUG_RETURN(di);
-	       }
-	  }
-     }
-   *dret = NULL;
-   if (ds)
-      Efree(ds);
-   EDBUG_RETURN(NULL);
-}
-
-Dialog             *
-FindDialog(Window win)
-{
-   Dialog             *d;
-   Dialog            **ds;
-   int                 i, num;
-
-   EDBUG(6, "FindDialog");
-   ds = (Dialog **) ListItemType(&num, LIST_TYPE_DIALOG);
-   for (i = 0; i < num; i++)
-     {
-	if (ds[i]->win == win)
-	  {
-	     d = ds[i];
-	     Efree(ds);
-	     EDBUG_RETURN(d);
-	  }
-     }
-   if (ds)
-      Efree(ds);
-   EDBUG_RETURN(NULL);
-}
-
-EWin               *
-FindEwinSpawningMenu(Menu * m)
-{
-   EWin               *ewin;
-   EWin              **ewins;
-   int                 i, num;
-
-   EDBUG(6, "FindEwinSpawningMenu");
-   ewins = (EWin **) ListItemType(&num, LIST_TYPE_EWIN);
-   for (i = 0; i < num; i++)
-     {
-	if (ewins[i]->shownmenu == m->win)
-	  {
-	     ewin = ewins[i];
-	     Efree(ewins);
-	     EDBUG_RETURN(ewin);
-	  }
-     }
-   if (ewins)
-      Efree(ewins);
-   EDBUG_RETURN(NULL);
-}
-
-Pager              *
-FindPager(Window win)
-{
-   Pager              *p;
-   Pager             **ps;
-   int                 i, num;
-
-   EDBUG(6, "FindDialog");
-   ps = (Pager **) ListItemType(&num, LIST_TYPE_PAGER);
-   for (i = 0; i < num; i++)
-     {
-	if ((ps[i]->win == win) || (ps[i]->hi_win == win))
-	  {
-	     p = ps[i];
-	     Efree(ps);
-	     EDBUG_RETURN(p);
-	  }
-     }
-   if (ps)
-      Efree(ps);
-   EDBUG_RETURN(NULL);
+   EDBUG_RETURN(n);
 }
