@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -327,6 +327,7 @@ GNOME_GetHintLayer(EWin * ewin, Atom atom_change)
    if (retval)
      {
 	ewin->layer = *retval;
+	EwinChange(ewin, EWIN_CHANGE_LAYER);
 	Efree(retval);
      }
    EDBUG_RETURN_;
@@ -405,6 +406,7 @@ GNOME_GetHintDesktop(EWin * ewin, Atom atom_change)
      {
 	desk = (int *)retval;
 	ewin->desktop = *desk;
+	EwinChange(ewin, EWIN_CHANGE_DESKTOP);
 	Efree(retval);
      }
    EDBUG_RETURN_;
@@ -548,7 +550,7 @@ GNOME_SetUsedHints(void)
    list[7] = XInternAtom(disp, XA_WIN_WORKSPACE_COUNT, False);
    list[8] = XInternAtom(disp, XA_WIN_WORKSPACE_NAMES, False);
    list[9] = XInternAtom(disp, XA_WIN_CLIENT_LIST, False);
-   XChangeProperty(disp, root.win, atom_set, XA_ATOM, 32, PropModeReplace,
+   XChangeProperty(disp, VRoot.win, atom_set, XA_ATOM, 32, PropModeReplace,
 		   (unsigned char *)list, 10);
    EDBUG_RETURN_;
 }
@@ -566,7 +568,7 @@ GNOME_SetCurrentArea(void)
    GetCurrentArea(&ax, &ay);
    val[0] = ax;
    val[1] = ay;
-   XChangeProperty(disp, root.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
+   XChangeProperty(disp, VRoot.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
 		   (unsigned char *)val, 2);
    EDBUG_RETURN_;
 }
@@ -581,7 +583,7 @@ GNOME_SetCurrentDesk(void)
    if (!atom_set)
       atom_set = XInternAtom(disp, XA_WIN_WORKSPACE, False);
    val = (CARD32) desks.current;
-   XChangeProperty(disp, root.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
+   XChangeProperty(disp, VRoot.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
 		   (unsigned char *)&val, 1);
    EDBUG_RETURN_;
 }
@@ -597,7 +599,7 @@ GNOME_SetWMCheck(Window win_wm_check)
    if (!atom_set)
       atom_set = XInternAtom(disp, XA_WIN_SUPPORTING_WM_CHECK, False);
    val = win_wm_check;
-   XChangeProperty(disp, root.win, atom_set, XA_CARDINAL,
+   XChangeProperty(disp, VRoot.win, atom_set, XA_CARDINAL,
 		   32, PropModeReplace, (unsigned char *)&val, 1);
    XChangeProperty(disp, win_wm_check, atom_set, XA_CARDINAL,
 		   32, PropModeReplace, (unsigned char *)&val, 1);
@@ -613,8 +615,8 @@ GNOME_SetDeskCount(void)
    EDBUG(6, "GNOME_SetDeskCount");
    if (!atom_set)
       atom_set = XInternAtom(disp, XA_WIN_WORKSPACE_COUNT, False);
-   val = mode.numdesktops;
-   XChangeProperty(disp, root.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
+   val = Conf.desks.num;
+   XChangeProperty(disp, VRoot.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
 		   (unsigned char *)&val, 1);
    EDBUG_RETURN_;
 }
@@ -632,7 +634,7 @@ GNOME_SetAreaCount(void)
    GetAreaSize(&ax, &ay);
    val[0] = ax;
    val[1] = ay;
-   XChangeProperty(disp, root.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
+   XChangeProperty(disp, VRoot.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
 		   (unsigned char *)val, 2);
    EDBUG_RETURN_;
 }
@@ -648,17 +650,17 @@ GNOME_SetDeskNames(void)
    EDBUG(6, "GNOME_SetDeskNames");
    if (!atom_set)
       atom_set = XInternAtom(disp, XA_WIN_WORKSPACE_NAMES, False);
-   for (i = 0; i < mode.numdesktops; i++)
+   for (i = 0; i < Conf.desks.num; i++)
      {
 	Esnprintf(s, sizeof(s), "%i", i);
-	names[i] = duplicate(s);
+	names[i] = Estrdup(s);
      }
-   if (XStringListToTextProperty(names, mode.numdesktops, &text))
+   if (XStringListToTextProperty(names, Conf.desks.num, &text))
      {
-	XSetTextProperty(disp, root.win, &text, atom_set);
+	XSetTextProperty(disp, VRoot.win, &text, atom_set);
 	XFree(text.value);
      }
-   for (i = 0; i < mode.numdesktops; i++)
+   for (i = 0; i < Conf.desks.num; i++)
       if (names[i])
 	 Efree(names[i]);
    EDBUG_RETURN_;
@@ -688,7 +690,7 @@ GNOME_SetClientList(void)
 		wl[j++] = lst[i]->client.win;
 	  }
      }
-   XChangeProperty(disp, root.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
+   XChangeProperty(disp, VRoot.win, atom_set, XA_CARDINAL, 32, PropModeReplace,
 		   (unsigned char *)wl, j);
    if (wl)
       Efree(wl);
@@ -706,11 +708,11 @@ GNOME_SetWMNameVer(void)
    EDBUG(6, "GNOME_SetWMNameVer");
    if (!atom_set)
       atom_set = XInternAtom(disp, XA_WIN_WM_NAME, False);
-   XChangeProperty(disp, root.win, atom_set, XA_STRING, 8, PropModeReplace,
+   XChangeProperty(disp, VRoot.win, atom_set, XA_STRING, 8, PropModeReplace,
 		   (unsigned char *)e_wm_name, strlen(e_wm_name));
    if (!atom_set2)
       atom_set2 = XInternAtom(disp, XA_WIN_WM_VERSION, False);
-   XChangeProperty(disp, root.win, atom_set2, XA_STRING, 8, PropModeReplace,
+   XChangeProperty(disp, VRoot.win, atom_set2, XA_STRING, 8, PropModeReplace,
 		   (unsigned char *)wm_version, strlen(wm_version));
    EDBUG_RETURN_;
 }
@@ -770,11 +772,11 @@ GNOME_SetHints(Window win_wm_check)
       CARD32              val;
 
       atom_set = XInternAtom(disp, "_WIN_DESKTOP_BUTTON_PROXY", False);
-      bpress_win = ECreateWindow(root.win, -80, -80, 24, 24, 0);
-      val = bpress_win;
-      XChangeProperty(disp, root.win, atom_set, XA_CARDINAL, 32,
+      Mode.button_proxy_win = ECreateWindow(VRoot.win, -80, -80, 24, 24, 0);
+      val = Mode.button_proxy_win;
+      XChangeProperty(disp, VRoot.win, atom_set, XA_CARDINAL, 32,
 		      PropModeReplace, (unsigned char *)&val, 1);
-      XChangeProperty(disp, bpress_win, atom_set, XA_CARDINAL, 32,
+      XChangeProperty(disp, Mode.button_proxy_win, atom_set, XA_CARDINAL, 32,
 		      PropModeReplace, (unsigned char *)&val, 1);
    }
 
@@ -844,34 +846,20 @@ GNOME_ProcessClientMessage(XClientMessageEvent * event)
 	     if (event->data.l[1] & WIN_STATE_STICKY)
 	       {
 		  if (!(ewin->sticky))
-		    {
-		       ewin->sticky = 1;
-		       RaiseEwin(ewin);
-		       DrawEwin(ewin);
-		       ApplySclass(FindItem
-				   ("SOUND_WINDOW_STICK", 0, LIST_FINDBY_NAME,
-				    LIST_TYPE_SCLASS));
-		    }
+		     EwinStick(ewin);
 	       }
 	     else
 	       {
 		  if (ewin->sticky)
-		    {
-		       ewin->sticky = 0;
-		       RaiseEwin(ewin);
-		       DrawEwin(ewin);
-		       ApplySclass(FindItem
-				   ("SOUND_WINDOW_UNSTICK", 0, LIST_FINDBY_NAME,
-				    LIST_TYPE_SCLASS));
-		    }
+		     EwinUnStick(ewin);
 	       }
 	  }
 	if (event->data.l[0] & WIN_STATE_SHADED)
 	  {
 	     if (event->data.l[1] & WIN_STATE_SHADED)
-		ShadeEwin(ewin);
+		EwinShade(ewin);
 	     else
-		UnShadeEwin(ewin);
+		EwinUnShade(ewin);
 	  }
 	HintsSetWindowState(ewin);
 	EDBUG_RETURN_;
