@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,9 +21,12 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
+#include <math.h>
 
-Group              *
-CreateGroup()
+#define DISABLE_PAGER_ICONBOX_GROUPING 0
+
+static Group       *
+GroupCreate(void)
 {
    Group              *g;
    double              t;
@@ -36,31 +39,31 @@ CreateGroup()
    g->index = (int)((GetTime() - (floor(t / 1000) * 1000)) * 10000);
    /* g->index = (int)(GetTime() * 100); */
 
-   g->cfg.iconify = mode.group_config.iconify;
-   g->cfg.kill = mode.group_config.kill;
-   g->cfg.move = mode.group_config.move;
-   g->cfg.raise = mode.group_config.raise;
-   g->cfg.set_border = mode.group_config.set_border;
-   g->cfg.stick = mode.group_config.stick;
-   g->cfg.shade = mode.group_config.shade;
-   g->cfg.mirror = mode.group_config.mirror;
+   g->cfg.iconify = Conf.group_config.iconify;
+   g->cfg.kill = Conf.group_config.kill;
+   g->cfg.move = Conf.group_config.move;
+   g->cfg.raise = Conf.group_config.raise;
+   g->cfg.set_border = Conf.group_config.set_border;
+   g->cfg.stick = Conf.group_config.stick;
+   g->cfg.shade = Conf.group_config.shade;
+   g->cfg.mirror = Conf.group_config.mirror;
    g->num_members = 0;
    g->members = NULL;
 
    EDBUG_RETURN(g);
 }
 
-void
-FreeGroup(Group * g)
+static void
+GroupDestroy(Group * g)
 {
-   if (g)
-     {
-	if (g == current_group)
-	   current_group = NULL;
-	if (g->members)
-	   Efree(g->members);
-	Efree(g);
-     }
+   if (!g)
+      return;
+
+   if (g == current_group)
+      current_group = NULL;
+   if (g->members)
+      Efree(g->members);
+   Efree(g);
 }
 
 void
@@ -104,21 +107,23 @@ BuildWindowGroup(EWin ** ewins, int num)
    int                 i;
    Group              *g;
 
-   current_group = g = CreateGroup();
+   current_group = g = GroupCreate();
    AddItem(g, NULL, g->index, LIST_TYPE_GROUP);
 
    for (i = 0; i < num; i++)
      {
+#if DISABLE_PAGER_ICONBOX_GROUPING
 	/* disable iconboxes and pagers to go into groups */
-	if (!((ewins[i]->ibox) || (ewins[i]->pager)))
-	   AddEwinToGroup(ewins[i], g);
-	else
+	if ((ewins[i]->ibox) || (ewins[i]->pager))
 	  {
-	     DIALOG_OK(_("Cannot comply"),
-		       _("Iconboxes and Pagers are disallowed from being\n"
-			 "members of a group. You cannot add these windows\n"
-			 "to a group.\n"));
+	     DialogOK(_("Cannot comply"),
+		      _("Iconboxes and Pagers are disallowed from being\n"
+			"members of a group. You cannot add these windows\n"
+			"to a group.\n"));
+	     return;
 	  }
+#endif
+	AddEwinToGroup(ewins[i], g);
      }
 }
 
@@ -129,28 +134,28 @@ AddEwinToGroup(EWin * ewin, Group * g)
 
    if (ewin && g)
      {
+#if DISABLE_PAGER_ICONBOX_GROUPING
 	/* disable iconboxes and pagers to go into groups */
-	if (!((ewin->ibox) || (ewin->pager)))
+	if ((ewin->ibox) || (ewin->pager))
 	  {
-	     for (i = 0; i < ewin->num_groups; i++)
-		if (ewin->groups[i] == g)
-		   return;
-	     ewin->num_groups++;
-	     ewin->groups =
-		Erealloc(ewin->groups, sizeof(Group *) * ewin->num_groups);
-	     ewin->groups[ewin->num_groups - 1] = g;
-	     g->num_members++;
-	     g->members = Erealloc(g->members, sizeof(EWin *) * g->num_members);
-	     g->members[g->num_members - 1] = ewin;
-	     RememberImportantInfoForEwin(ewin);
-	  }
-	else
-	  {
-	     DIALOG_OK(_("Cannot comply"),
-		       _("Iconboxes and Pagers are disallowed from being\n"
-			 "members of a group. You cannot add these windows\n"
-			 "to a group.\n"));
+	     DialogOK(_("Cannot comply"),
+		      _("Iconboxes and Pagers are disallowed from being\n"
+			"members of a group. You cannot add these windows\n"
+			"to a group.\n"));
+	     return;
 	  }
+#endif
+	for (i = 0; i < ewin->num_groups; i++)
+	   if (ewin->groups[i] == g)
+	      return;
+	ewin->num_groups++;
+	ewin->groups =
+	   Erealloc(ewin->groups, sizeof(Group *) * ewin->num_groups);
+	ewin->groups[ewin->num_groups - 1] = g;
+	g->num_members++;
+	g->members = Erealloc(g->members, sizeof(EWin *) * g->num_members);
+	g->members[g->num_members - 1] = ewin;
+	RememberImportantInfoForEwin(ewin);
      }
 }
 
@@ -189,83 +194,66 @@ EwinsInGroup(EWin * ewin1, EWin * ewin2)
 void
 RemoveEwinFromGroup(EWin * ewin, Group * g)
 {
-   int                 i, j, k, i2, x, y;
+   int                 i, j, k, i2;
 
-   if (ewin && g)
+   if (!ewin || !g)
+      return;
+
+   for (k = 0; k < ewin->num_groups; k++)
      {
-	if (ewin->groups)
+	/* is the window actually part of the given group */
+	if (ewin->groups[k] != g)
+	   continue;
+
+	for (i = 0; i < g->num_members; i++)
 	  {
-	     for (k = 0; k < ewin->num_groups; k++)
+	     if (g->members[i] != ewin)
+		continue;
+
+	     /* remove it from the group */
+	     for (j = i; j < g->num_members - 1; j++)
+		g->members[j] = g->members[j + 1];
+	     g->num_members--;
+	     if (g->num_members > 0)
+		g->members =
+		   Erealloc(g->members, sizeof(EWin *) * g->num_members);
+	     else
 	       {
-		  /* if the window is actually part of the given group */
-		  if (ewin->groups[k] == g)
-		    {
-		       for (i = 0; i < g->num_members; i++)
-			 {
-			    if (g->members[i] == ewin)
-			      {
-				 /* remove it from the group */
-				 for (j = i; j < g->num_members - 1; j++)
-				    g->members[j] = g->members[j + 1];
-				 g->num_members--;
-				 if (g->num_members > 0)
-				    g->members =
-				       Erealloc(g->members,
-						sizeof(EWin *) *
-						g->num_members);
-				 else
-				   {
-				      RemoveItem((char *)g, 0,
-						 LIST_FINDBY_POINTER,
-						 LIST_TYPE_GROUP);
-				      FreeGroup(g);
-				   }
-				 /* and remove the group from the groups that the window is in */
-				 for (i2 = k; i2 < ewin->num_groups - 1; i2++)
-				    ewin->groups[i2] = ewin->groups[i2 + 1];
-				 ewin->num_groups--;
-				 if (ewin->num_groups <= 0)
-				   {
-				      Efree(ewin->groups);
-				      ewin->groups = NULL;
-				      ewin->num_groups = 0;
-				   }
-				 else
-				    ewin->groups =
-				       Erealloc(ewin->groups,
-						sizeof(Group *) *
-						ewin->num_groups);
-				 SaveGroups();
-
-				 x = ewin->x;
-				 y = ewin->y;
-				 if ((ewin->x + ewin->border->border.left + 1) >
-				     root.w)
-				    x = root.w - ewin->border->border.left - 1;
-				 else if ((ewin->x + ewin->w -
-					   ewin->border->border.right - 1) < 0)
-				    x = 0 - ewin->w +
-				       ewin->border->border.right + 1;
-				 if ((ewin->y + ewin->border->border.top + 1) >
-				     root.h)
-				    y = root.h - ewin->border->border.top - 1;
-				 else if ((ewin->y + ewin->h -
-					   ewin->border->border.bottom - 1) < 0)
-				    y = 0 - ewin->h +
-				       ewin->border->border.bottom + 1;
-
-				 MoveEwin(ewin, x, y);
-
-				 RememberImportantInfoForEwin(ewin);
-				 return;
-			      }
-			 }
-		    }
+		  RemoveItem((char *)g, 0,
+			     LIST_FINDBY_POINTER, LIST_TYPE_GROUP);
+		  GroupDestroy(g);
 	       }
+
+	     /* and remove the group from the groups that the window is in */
+	     for (i2 = k; i2 < ewin->num_groups - 1; i2++)
+		ewin->groups[i2] = ewin->groups[i2 + 1];
+	     ewin->num_groups--;
+	     if (ewin->num_groups <= 0)
+	       {
+		  Efree(ewin->groups);
+		  ewin->groups = NULL;
+		  ewin->num_groups = 0;
+	       }
+	     else
+		ewin->groups =
+		   Erealloc(ewin->groups, sizeof(Group *) * ewin->num_groups);
+
+	     SaveGroups();
+	     return;
 	  }
      }
 }
 
+void
+GroupsEwinRemove(EWin * ewin)
+{
+   int                 num, i;
+
+   num = ewin->num_groups;
+   for (i = 0; i < num; i++)
+      RemoveEwinFromGroup(ewin, ewin->groups[0]);
+}
+
 char              **
 GetWinGroupMemberNames(Group ** groups, int num)
 {
@@ -280,7 +268,7 @@ GetWinGroupMemberNames(Group ** groups, int num)
 	for (j = 0; j < groups[i]->num_members; j++)
 	  {
 	     strcat(group_member_strings[i],
-		    groups[i]->members[j]->client.title);
+		    EwinGetTitle(groups[i]->members[j]));
 	     strcat(group_member_strings[i], "\n");
 	  }
      }
@@ -365,11 +353,7 @@ ShowHideWinGroups(EWin * ewin, Group * g, char onoff)
 
 	if (b)
 	  {
-	     gwins[i]->border_new = 1;
-	     SetEwinToBorder(gwins[i], b);
-	     ICCCM_MatchSize(gwins[i]);
-	     MoveResizeEwin(gwins[i], gwins[i]->x, gwins[i]->y,
-			    gwins[i]->client.w, gwins[i]->client.h);
+	     EwinSetBorder(gwins[i], b, 1);
 	     RememberImportantInfoForEwin(gwins[i]);
 	  }
      }
@@ -390,8 +374,8 @@ SaveGroups(void)
 	FILE               *f;
 	char                s[1024];
 
-	Esnprintf(s, sizeof(s), "%s/...e_session-XXXXXX.groups.%i", UserEDir(),
-		  root.scr);
+	Esnprintf(s, sizeof(s), "%s/...e_session-XXXXXX.groups.%i", EDirUser(),
+		  VRoot.scr);
 	f = fopen(s, "w");
 	if (f)
 	  {
@@ -435,8 +419,8 @@ LoadGroups(void)
    FILE               *f;
    char                s[1024];
 
-   Esnprintf(s, sizeof(s), "%s/...e_session-XXXXXX.groups.%i", UserEDir(),
-	     root.scr);
+   Esnprintf(s, sizeof(s), "%s/...e_session-XXXXXX.groups.%i", EDirUser(),
+	     VRoot.scr);
    f = fopen(s, "r");
    if (f)
      {
@@ -451,7 +435,7 @@ LoadGroups(void)
 	     word(s, 1, ss);
 	     if (!strcmp(ss, "NEW:"))
 	       {
-		  g = CreateGroup();
+		  g = GroupCreate();
 		  if (g)
 		    {
 		       word(s, 2, ss);
@@ -567,7 +551,7 @@ ChooseGroup(int val, void *data)
    data = NULL;
 }
 
-void
+static void
 GroupCallback(int val, void *data)
 {
    ShowHideWinGroups(tmp_ewin, tmp_groups[tmp_index], SET_OFF);
@@ -596,26 +580,26 @@ ChooseGroupDialog(EWin * ewin, char *message, char group_select, int action)
    if ((num_groups == 0)
        && (action == ACTION_BREAK_GROUP || action == ACTION_REMOVE_FROM_GROUP))
      {
-	DIALOG_OK(_("Window Group Error"),
-		  _
-		  ("\n  This window currently does not belong to any groups.  \n"
-		   "  You can only destroy groups or remove windows from groups  \n"
-		   "  through a window that actually belongs to at least one group.\n\n"));
+	DialogOK(_("Window Group Error"),
+		 _
+		 ("\n  This window currently does not belong to any groups.  \n"
+		  "  You can only destroy groups or remove windows from groups  \n"
+		  "  through a window that actually belongs to at least one group.\n\n"));
 	return;
      }
    if ((num_groups == 0) && (group_select == GROUP_SELECT_ALL_EXCEPT_EWIN))
      {
-	DIALOG_OK(_("Window Group Error"),
-		  _("\n  Currently, no groups exist or this window  \n"
-		    "  already belongs to all existing groups.  \n"
-		    "  You have to start other groups first.  \n\n"));
+	DialogOK(_("Window Group Error"),
+		 _("\n  Currently, no groups exist or this window  \n"
+		   "  already belongs to all existing groups.  \n"
+		   "  You have to start other groups first.  \n\n"));
 	return;
      }
    if (!tmp_groups)
      {
-	DIALOG_OK(_("Window Group Error"),
-		  _
-		  ("\n  Currently, no groups exist. You have to start a group first.\n\n"));
+	DialogOK(_("Window Group Error"),
+		 _
+		 ("\n  Currently, no groups exist. You have to start a group first.\n\n"));
 	return;
      }
 
@@ -624,18 +608,18 @@ ChooseGroupDialog(EWin * ewin, char *message, char group_select, int action)
 
    if ((d = FindItem("GROUP_SELECTION", 0, LIST_FINDBY_NAME, LIST_TYPE_DIALOG)))
      {
-	AUDIO_PLAY("GROUP_SETTINGS_ACTIVE");
+	SoundPlay("GROUP_SETTINGS_ACTIVE");
 	ShowDialog(d);
      }
-   AUDIO_PLAY("SOUND_SETTINGS_GROUP");
+   SoundPlay("SOUND_SETTINGS_GROUP");
 
-   d = CreateDialog("GROUP_SELECTION");
+   d = DialogCreate("GROUP_SELECTION");
    DialogSetTitle(d, _("Window Group Selection"));
 
    table = DialogInitItem(d);
    DialogItemTableSetOptions(table, 2, 0, 0, 0);
 
-   if (mode.dialog_headers)
+   if (Conf.dialogs.headers)
      {
 	di = DialogAddItem(table, DITEM_IMAGE);
 	DialogItemSetPadding(di, 2, 2, 2, 2);
