@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -38,7 +38,6 @@ SetupFallbackClasses(void)
    ActionClass        *ac;
    Action             *a;
    Background         *bg;
-   ImlibColor          icl;
    TextClass          *tc;
 
    EDBUG(5, "SetupFallbackClasses");
@@ -58,113 +57,53 @@ SetupFallbackClasses(void)
 
    /* create a fallback imageclass in case no imageclass can be found */
    ic = CreateIclass();
-   ic->name = duplicate("__FALLBACK_ICLASS");
+   ic->name = Estrdup("__FALLBACK_ICLASS");
    ic->norm.normal = CreateImageState();
-   ic->norm.normal->hihi.r = 255;
-   ic->norm.normal->hihi.g = 255;
-   ic->norm.normal->hihi.b = 255;
-   ic->norm.normal->hi.r = 255;
-   ic->norm.normal->hi.g = 255;
-   ic->norm.normal->hi.b = 255;
-   ic->norm.normal->bg.r = 160;
-   ic->norm.normal->bg.g = 160;
-   ic->norm.normal->bg.b = 160;
-   ic->norm.normal->lo.r = 0;
-   ic->norm.normal->lo.g = 0;
-   ic->norm.normal->lo.b = 0;
-   ic->norm.normal->lolo.r = 0;
-   ic->norm.normal->lolo.g = 0;
-   ic->norm.normal->lolo.b = 0;
+   ESetColor(&(ic->norm.normal->hihi), 255, 255, 255);
+   ESetColor(&(ic->norm.normal->hi), 255, 255, 255);
+   ESetColor(&(ic->norm.normal->bg), 160, 160, 160);
+   ESetColor(&(ic->norm.normal->lo), 0, 0, 0);
+   ESetColor(&(ic->norm.normal->lolo), 0, 0, 0);
    ic->norm.normal->bevelstyle = BEVEL_AMIGA;
 
    ic->norm.hilited = CreateImageState();
-   ic->norm.hilited->hihi.r = 255;
-   ic->norm.hilited->hihi.g = 255;
-   ic->norm.hilited->hihi.b = 255;
-   ic->norm.hilited->hi.r = 255;
-   ic->norm.hilited->hi.g = 255;
-   ic->norm.hilited->hi.b = 255;
-   ic->norm.hilited->bg.r = 192;
-   ic->norm.hilited->bg.g = 192;
-   ic->norm.hilited->bg.b = 192;
-   ic->norm.hilited->lo.r = 0;
-   ic->norm.hilited->lo.g = 0;
-   ic->norm.hilited->lo.b = 0;
-   ic->norm.hilited->lolo.r = 0;
-   ic->norm.hilited->lolo.g = 0;
-   ic->norm.hilited->lolo.b = 0;
+   ESetColor(&(ic->norm.hilited->hihi), 255, 255, 255);
+   ESetColor(&(ic->norm.hilited->hi), 255, 255, 255);
+   ESetColor(&(ic->norm.hilited->bg), 192, 192, 192);
+   ESetColor(&(ic->norm.hilited->lo), 0, 0, 0);
+   ESetColor(&(ic->norm.hilited->lolo), 0, 0, 0);
    ic->norm.hilited->bevelstyle = BEVEL_AMIGA;
 
    ic->norm.clicked = CreateImageState();
-   ic->norm.clicked->hihi.r = 0;
-   ic->norm.clicked->hihi.g = 0;
-   ic->norm.clicked->hihi.b = 0;
-   ic->norm.clicked->hi.r = 0;
-   ic->norm.clicked->hi.g = 0;
-   ic->norm.clicked->hi.b = 0;
-   ic->norm.clicked->bg.r = 192;
-   ic->norm.clicked->bg.g = 192;
-   ic->norm.clicked->bg.b = 192;
-   ic->norm.clicked->lo.r = 255;
-   ic->norm.clicked->lo.g = 255;
-   ic->norm.clicked->lo.b = 255;
-   ic->norm.clicked->lolo.r = 255;
-   ic->norm.clicked->lolo.g = 255;
-   ic->norm.clicked->lolo.b = 255;
+   ESetColor(&(ic->norm.clicked->hihi), 0, 0, 0);
+   ESetColor(&(ic->norm.clicked->hi), 0, 0, 0);
+   ESetColor(&(ic->norm.clicked->bg), 192, 192, 192);
+   ESetColor(&(ic->norm.clicked->lo), 255, 255, 255);
+   ESetColor(&(ic->norm.clicked->lolo), 255, 255, 255);
    ic->norm.clicked->bevelstyle = BEVEL_AMIGA;
 
    ic->active.normal = CreateImageState();
-   ic->active.normal->hihi.r = 255;
-   ic->active.normal->hihi.g = 255;
-   ic->active.normal->hihi.b = 255;
-   ic->active.normal->hi.r = 255;
-   ic->active.normal->hi.g = 255;
-   ic->active.normal->hi.b = 255;
-   ic->active.normal->bg.r = 180;
-   ic->active.normal->bg.g = 140;
-   ic->active.normal->bg.b = 160;
-   ic->active.normal->lo.r = 0;
-   ic->active.normal->lo.g = 0;
-   ic->active.normal->lo.b = 0;
-   ic->active.normal->lolo.r = 0;
-   ic->active.normal->lolo.g = 0;
-   ic->active.normal->lolo.b = 0;
+   ESetColor(&(ic->active.normal->hihi), 255, 255, 255);
+   ESetColor(&(ic->active.normal->hi), 255, 255, 255);
+   ESetColor(&(ic->active.normal->bg), 180, 140, 160);
+   ESetColor(&(ic->active.normal->lo), 0, 0, 0);
+   ESetColor(&(ic->active.normal->lolo), 0, 0, 0);
    ic->active.normal->bevelstyle = BEVEL_AMIGA;
 
    ic->active.hilited = CreateImageState();
-   ic->active.hilited->hihi.r = 255;
-   ic->active.hilited->hihi.g = 255;
-   ic->active.hilited->hihi.b = 255;
-   ic->active.hilited->hi.r = 255;
-   ic->active.hilited->hi.g = 255;
-   ic->active.hilited->hi.b = 255;
-   ic->active.hilited->bg.r = 230;
-   ic->active.hilited->bg.g = 190;
-   ic->active.hilited->bg.b = 210;
-   ic->active.hilited->lo.r = 0;
-   ic->active.hilited->lo.g = 0;
-   ic->active.hilited->lo.b = 0;
-   ic->active.hilited->lolo.r = 0;
-   ic->active.hilited->lolo.g = 0;
-   ic->active.hilited->lolo.b = 0;
+   ESetColor(&(ic->active.hilited->hihi), 255, 255, 255);
+   ESetColor(&(ic->active.hilited->hi), 255, 255, 255);
+   ESetColor(&(ic->active.hilited->bg), 230, 190, 210);
+   ESetColor(&(ic->active.hilited->lo), 0, 0, 0);
+   ESetColor(&(ic->active.hilited->lolo), 0, 0, 0);
    ic->active.hilited->bevelstyle = BEVEL_AMIGA;
 
    ic->active.clicked = CreateImageState();
-   ic->active.clicked->hihi.r = 0;
-   ic->active.clicked->hihi.g = 0;
-   ic->active.clicked->hihi.b = 0;
-   ic->active.clicked->hi.r = 0;
-   ic->active.clicked->hi.g = 0;
-   ic->active.clicked->hi.b = 0;
-   ic->active.clicked->bg.r = 230;
-   ic->active.clicked->bg.g = 190;
-   ic->active.clicked->bg.b = 210;
-   ic->active.clicked->lo.r = 255;
-   ic->active.clicked->lo.g = 255;
-   ic->active.clicked->lo.b = 255;
-   ic->active.clicked->lolo.r = 255;
-   ic->active.clicked->lolo.g = 255;
-   ic->active.clicked->lolo.b = 255;
+   ESetColor(&(ic->active.clicked->hihi), 0, 0, 0);
+   ESetColor(&(ic->active.clicked->hi), 0, 0, 0);
+   ESetColor(&(ic->active.clicked->bg), 230, 190, 210);
+   ESetColor(&(ic->active.clicked->lo), 255, 255, 255);
+   ESetColor(&(ic->active.clicked->lolo), 255, 255, 255);
    ic->active.clicked->bevelstyle = BEVEL_AMIGA;
 
    IclassPopulate(ic);
@@ -188,28 +127,17 @@ SetupFallbackClasses(void)
    AddItem(b, b->name, 0, LIST_TYPE_BORDER);
 
    /* create a fallback background in case no background is found */
-   bg = CreateDesktopBG("NONE", &icl, NULL, 0, 0, 0, 0, 0, 0, NULL, 0, 0, 0, 0,
-			0);
-   AddItem(bg, bg->name, 0, LIST_TYPE_BACKGROUND);
+   bg = BackgroundCreate("NONE", NULL, NULL, 0, 0, 0, 0, 0, 0, NULL, 0, 0, 0, 0,
+			 0);
 
    /* create a fallback textclass in case no textclass is found */
    tc = CreateTclass();
-   tc->name = "__FALLBACK_TCLASS";
+   tc->name = Estrdup("__FALLBACK_TCLASS");
    tc->norm.normal = CreateTextState();
    tc->norm.normal->fontname =
-      duplicate("-*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*");
-   tc->norm.normal->fg_col.r = 0;
-   tc->norm.normal->fg_col.g = 0;
-   tc->norm.normal->fg_col.b = 0;
+      Estrdup("-*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*");
+   ESetColor(&(tc->norm.normal->fg_col), 0, 0, 0);
    AddItem(tc, tc->name, 0, LIST_TYPE_TCLASS);
 
    EDBUG_RETURN_;
 }
-
-void
-SetupInit(void)
-{
-   EDBUG(5, "SetupInit");
-
-   EDBUG_RETURN_;
-}
