@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -23,7 +23,7 @@
 #include "E.h"
 
 void               *
-FindItem(char *name, int id, int find_by, int type)
+FindItem(const char *name, int id, int find_by, int type)
 {
    List               *ptr;
 
@@ -76,7 +76,7 @@ FindItem(char *name, int id, int find_by, int type)
 }
 
 void
-AddItem(void *item, char *name, int id, int type)
+AddItem(void *item, const char *name, int id, int type)
 {
    List               *ptr;
 
@@ -85,7 +85,7 @@ AddItem(void *item, char *name, int id, int type)
    if (!ptr)
       EDBUG_RETURN_;
    ptr->item = item;
-   ptr->name = duplicate(name);
+   ptr->name = Estrdup(name);
    ptr->id = id;
    ptr->next = lists[type].next;
    lists[type].next = ptr;
@@ -93,7 +93,7 @@ AddItem(void *item, char *name, int id, int type)
 }
 
 void
-AddItemEnd(void *item, char *name, int id, int type)
+AddItemEnd(void *item, const char *name, int id, int type)
 {
    List               *ptr, *p;
 
@@ -102,7 +102,7 @@ AddItemEnd(void *item, char *name, int id, int type)
    if (!ptr)
       EDBUG_RETURN_;
    ptr->item = item;
-   ptr->name = duplicate(name);
+   ptr->name = Estrdup(name);
    ptr->id = id;
    ptr->next = NULL;
    p = lists[type].next;
@@ -197,7 +197,7 @@ MoveItemToListBottom(void *item, int type)
 }
 
 void               *
-RemoveItem(char *name, int id, int find_by, int type)
+RemoveItem(const char *name, int id, int find_by, int type)
 {
    List               *ptr, *pptr;
    void               *p;
@@ -415,7 +415,7 @@ ListItems(int *num, int type)
      {
 	while (ptr)
 	  {
-	     list[i] = duplicate(ptr->name);
+	     list[i] = Estrdup(ptr->name);
 	     i++;
 	     ptr = ptr->next;
 	  }
@@ -424,7 +424,7 @@ ListItems(int *num, int type)
      {
 	while (ptr)
 	  {
-	     list[i] = duplicate(ptr->name);
+	     list[i] = Estrdup(ptr->name);
 	     i++;
 	     ptr = ptr->next;
 	  }
@@ -470,7 +470,7 @@ ListItemTypeID(int *num, int type, int id)
 }
 
 void              **
-ListItemTypeName(int *num, int type, char *name)
+ListItemTypeName(int *num, int type, const char *name)
 {
    List               *ptr;
    int                 i, len;
