@@ -305,10 +305,7 @@ main(int argc, char **argv)
    ECursorsInit();
    ECsrApply(ECSR_ROOT, VRoot.win);
 
-#if 0				/* AAA: Why here? */
-   Mode.wm.startup = 0;
-#endif
-   Mode.wm.save_ok = Mode.wm.master;
+   Mode.wm.save_ok = 1;
    /* ok - paranoia - save current settings to disk */
    if (VRoot.scr == 0)
       autosave();
@@ -363,9 +360,6 @@ main(int argc, char **argv)
    if (Mode.firsttime)
       runDocBrowser();
 
-#if 0				/* Not if AAA */
-   Mode.wm.startup = 1;
-#endif
    SpawnSnappedCmds();
    Mode.wm.startup = 0;
 
