@@ -1257,12 +1257,13 @@ MenuCreateFromFlatFile(const char *name, MenuStyle * ms, const char *file,
 		       Menu * parent)
 {
    Menu               *m;
-   char                s[4096], *ff = NULL;
+   char                s[4096], *ff;
    static int          calls = 0;
 
    EDBUG(5, "MenuCreateFromFlatFile");
-   if (calls > 255)
+   if (calls > 32)
       EDBUG_RETURN(NULL);
+   calls++;
 
    ff = FindFile(file);
    if (!ff)
@@ -1847,7 +1848,6 @@ ShowNamedMenu(const char *name)
    m = FindItem(name, 0, LIST_FINDBY_NAME, LIST_TYPE_MENU);
    if (m)
      {
-	XUngrabPointer(disp, CurrentTime);
 	if (!FindEwinByMenu(m))	/* Don't show if already shown */
 	   MenuShow(m, 0);
 	Mode.menus.list[0] = m;
@@ -1991,8 +1991,6 @@ ShowInternalMenu(Menu ** pm, MenuStyle ** pms, const char *style,
 
    EDBUG(5, "ShowInternalMenu");
 
-   XUngrabPointer(disp, CurrentTime);
-
    if (!ms)
      {
 	ms = FindItem(style, 0, LIST_FINDBY_NAME, LIST_TYPE_MENU_STYLE);
@@ -2196,6 +2194,27 @@ MenuFindContextEwin(Menu * m)
    return FindEwinSpawningMenu(m);
 }
 
+static              KeySym
+MenuKeyPressConversion(KeySym key)
+{
+   if (key == Conf.menukeys.left)
+      return XK_Left;
+   if (key == Conf.menukeys.right)
+      return XK_Right;
+   if (key == Conf.menukeys.up)
+      return XK_Up;
+   if (key == Conf.menukeys.down)
+      return XK_Down;
+   if (key == Conf.menukeys.escape)
+      return XK_Escape;
+   if (key == Conf.menukeys.ret)
+      return XK_Return;
+
+   /* The key does not correspond to any set, use the default behavior 
+    * associated to the key */
+   return key;
+}
+
 int
 MenusEventKeyPress(XEvent * ev)
 {
@@ -2219,7 +2238,7 @@ MenusEventKeyPress(XEvent * ev)
    /* NB! m != NULL */
 
    key = XLookupKeysym(&ev->xkey, 0);
-   switch (key)
+   switch (MenuKeyPressConversion(key))
      {
      case XK_Escape:
 	MenusHide();
@@ -2295,7 +2314,6 @@ MenusEventMouseDown(XEvent * ev)
 	Mode.menus.list[0] = m;
 	Mode.menus.current_depth = 1;
 	MenuShowMasker(m);
-	XUngrabPointer(disp, CurrentTime);
 	ewin = FindEwinByMenu(m);
 	if (ewin)
 	  {
