@@ -21,6 +21,7 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
+#include <sys/time.h>
 
 static char        *userDir = NULL;
 static char        *cacheDir = NULL;
@@ -180,6 +181,59 @@ Quicksort(void **a, int l, int r, int (*CompareFunc) (void *d1, void *d2))
      }
 }
 
+/*
+ * Stuff to make loops for animated effects.
+ */
+struct timeval      etl_tv_start;
+static int          etl_k1, etl_k2;
+static double       etl_k, etl_fac;
+
+/*
+ * Return elapsed time in seconds since t0
+ */
+static double
+ETimeElapsed(struct timeval *t0)
+{
+   struct timeval      tv;
+   int                 sec, usec;
+
+   gettimeofday(&tv, NULL);
+   sec = tv.tv_sec - t0->tv_sec;
+   usec = tv.tv_usec - t0->tv_usec;
+   return (double)sec + (((double)usec) / 1000000);
+}
+
+void
+ETimedLoopInit(int k1, int k2, int speed)
+{
+   etl_k1 = k1;
+   etl_k2 = k2;
+   if (speed < 500)
+      speed = 500;
+   /* When speed is 1000 the loop will take one sec. */
+   etl_fac = (k2 - k1) * speed / 1000.;
+
+   gettimeofday(&etl_tv_start, NULL);
+}
+
+int
+ETimedLoopNext(void)
+{
+   double              tm;
+
+   /* Find elapsed time since loop start */
+   tm = ETimeElapsed(&etl_tv_start);
+#if 0
+   Eprintf("ETimedLoopNext k=%4f tm=%.3f\n", etl_k, tm);
+#endif
+   etl_k = etl_k1 + tm * etl_fac;
+
+   return etl_k;
+}
+
+/*
+ * Debug/error message printing.
+ */
 void
 Eprintf(const char *fmt, ...)
 {
