@@ -23,6 +23,7 @@
 #include "E.h"
 #include <X11/keysym.h>
 #include <sys/time.h>
+#include <signal.h>
 
 void
 MapUnmap(int start)
@@ -179,6 +180,7 @@ SetupX(void)
 	     else
 	       {
 		  /* We are a slave */
+		  close(ConnectionNumber(disp));
 		  Mode.wm.master = 0;
 		  VRoot.scr = i;
 #ifdef SIGSTOP
@@ -456,6 +458,13 @@ SetupX(void)
    Conf.st_trans.warplist = ICLASS_ATTR_BG;
 #endif
 
+   Conf.menukeys.left = XK_Left;
+   Conf.menukeys.right = XK_Right;
+   Conf.menukeys.up = XK_Up;
+   Conf.menukeys.down = XK_Down;
+   Conf.menukeys.escape = XK_Escape;
+   Conf.menukeys.ret = XK_Return;
+
    ScreenInit();
 
    MenusInit();
