@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,8 +21,11 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
+
 #ifdef HAVE_LIBESD
+#include <esd.h>
 #include <audiofile.h>
+
 #ifdef WORDS_BIGENDIAN
 #define SWAP_SHORT( x ) x = ( ( x & 0x00ff ) << 8 ) | ( ( x >> 8 ) & 0x00ff )
 #define SWAP_LONG( x ) x = ( ( ( x & 0x000000ff ) << 24 ) |\
@@ -32,7 +35,27 @@
 #endif
 #endif
 
-Sample             *
+typedef struct
+{
+   char               *file;
+   int                 rate;
+   int                 format;
+   int                 samples;
+   unsigned char      *data;
+   int                 id;
+} Sample;
+
+struct _soundclass
+{
+   char               *name;
+   char               *file;
+   Sample             *sample;
+   unsigned int        ref_count;
+};
+
+static int          sound_fd = -1;
+
+static Sample      *
 LoadWav(char *file)
 {
 #ifdef HAVE_LIBESD
@@ -50,13 +73,11 @@ LoadWav(char *file)
    find = FindFile(file);
    if (!find)
      {
-	DIALOG_PARAM_OK(_("Error finding sound file"));
-	DIALOG_PARAM        _("Warning!  Enlightenment was unable "
-			      "to load the\nfollowing sound file:\n%s\n"
-			      "Enlightenment will continue to operate, but you\n"
-			      "may wish to check your configuration settings.\n"),
-	   file DIALOG_PARAM_END;
-
+	DialogOK(_("Error finding sound file"),
+		 _("Warning!  Enlightenment was unable to load the\n"
+		   "following sound file:\n%s\n"
+		   "Enlightenment will continue to operate, but you\n"
+		   "may wish to check your configuration settings.\n"), file);
 	EDBUG_RETURN(NULL);
      }
    in_file = afOpenFile(find, "r", NULL);
@@ -82,7 +103,7 @@ LoadWav(char *file)
 #else
    afSetVirtualByteOrder(in_file, AF_DEFAULT_TRACK, AF_BYTEORDER_LITTLEENDIAN);
 #endif
-   s->file = duplicate(find);
+   s->file = Estrdup(find);
    s->rate = 44100;
    s->format = ESD_STREAM | ESD_PLAY;
    s->samples = 0;
@@ -112,17 +133,16 @@ LoadWav(char *file)
 #endif
 }
 
-void
-SoundPlay(Sample * s)
+static void
+SamplePlay(Sample * s)
 {
 #ifdef HAVE_LIBESD
    int                 size, confirm = 0;
-
 #endif
 
-   EDBUG(5, "SoundPlay");
+   EDBUG(5, "SamplePlay");
 #ifdef HAVE_LIBESD
-   if ((sound_fd < 0) || (!mode.sound) || (!s))
+   if ((sound_fd < 0) || (!Conf.sound) || (!s))
       EDBUG_RETURN_;
    if (!s->id)
      {
@@ -149,58 +169,13 @@ SoundPlay(Sample * s)
      }
    if (s->id > 0)
       esd_sample_play(sound_fd, s->id);
-   EDBUG_RETURN_;
 #else
    s = NULL;
-   EDBUG_RETURN_;
 #endif
+   EDBUG_RETURN_;
 }
 
-SoundClass         *
-CreateSoundClass(char *name, char *file)
-{
-   SoundClass         *sclass;
-
-   EDBUG(6, "CreateSoundClass");
-   sclass = Emalloc(sizeof(SoundClass));
-   if (!sclass)
-      EDBUG_RETURN(NULL);
-   sclass->name = duplicate(name);
-   sclass->file = duplicate(file);
-   sclass->sample = NULL;
-   EDBUG_RETURN(sclass);
-}
-
-void
-ApplySclass(SoundClass * sclass)
-{
-#ifdef HAVE_LIBESD
-   char               *f;
-
-#endif
-
-   EDBUG(4, "ApplySclass");
-   if (!sclass)
-      EDBUG_RETURN_;
-#ifdef HAVE_LIBESD
-   if ((!sclass->sample) && (mode.sound))
-     {
-	f = FindFile(sclass->file);
-	if (f)
-	  {
-	     sclass->sample = LoadWav(f);
-	     Efree(f);
-	  }
-     }
-   if ((mode.sound) && (sclass->sample))
-      SoundPlay(sclass->sample);
-   EDBUG_RETURN_;
-#else
-   EDBUG_RETURN_;
-#endif
-}
-
-void
+static void
 DestroySample(Sample * s)
 {
    EDBUG(5, "DestroySample");
@@ -223,13 +198,55 @@ DestroySample(Sample * s)
    EDBUG_RETURN_;
 }
 
-void
-DestroySclass(SoundClass * sclass)
+SoundClass         *
+SclassCreate(const char *name, const char *file)
 {
+   SoundClass         *sclass;
+
+   EDBUG(6, "SclassCreate");
+   sclass = Emalloc(sizeof(SoundClass));
+   if (!sclass)
+      EDBUG_RETURN(NULL);
+   sclass->name = Estrdup(name);
+   sclass->file = Estrdup(file);
+   sclass->sample = NULL;
+   AddItem(sclass, sclass->name, 0, LIST_TYPE_SCLASS);
+   EDBUG_RETURN(sclass);
+}
+
+static void
+SclassApply(SoundClass * sclass)
+{
+#ifdef HAVE_LIBESD
+   char               *f;
+#endif
+
+   EDBUG(4, "SclassApply");
    if (!sclass)
       EDBUG_RETURN_;
+#ifdef HAVE_LIBESD
+   if ((!sclass->sample) && (Conf.sound))
+     {
+	f = FindFile(sclass->file);
+	if (f)
+	  {
+	     sclass->sample = LoadWav(f);
+	     Efree(f);
+	  }
+     }
+   if ((Conf.sound) && (sclass->sample))
+      SamplePlay(sclass->sample);
+#endif
+   EDBUG_RETURN_;
+}
 
-   EDBUG(5, "DestroySclass");
+static void
+SclassDestroy(SoundClass * sclass)
+{
+   EDBUG(5, "SclassDestroy");
+   if (!sclass)
+      EDBUG_RETURN_;
+   RemoveItem(sclass->name, 0, LIST_FINDBY_NAME, LIST_TYPE_SCLASS);
    if (sclass->name)
       Efree(sclass->name);
    if (sclass->file)
@@ -240,17 +257,42 @@ DestroySclass(SoundClass * sclass)
    EDBUG_RETURN_;
 }
 
+const char         *
+SclassGetName(SoundClass * sclass)
+{
+   return sclass->name;
+}
+
+int
+SoundPlay(const char *name)
+{
+   SoundClass         *sclass;
+
+   sclass = FindItem(name, 0, LIST_FINDBY_NAME, LIST_TYPE_SCLASS);
+   SclassApply(sclass);
+   return sclass != NULL;
+}
+
+int
+SoundFree(const char *name)
+{
+   SoundClass         *sclass;
+
+   sclass = FindItem(name, 0, LIST_FINDBY_NAME, LIST_TYPE_SCLASS);
+   SclassDestroy(sclass);
+   return sclass != NULL;
+}
+
 void
-SoundInit()
+SoundInit(void)
 {
 #ifdef HAVE_LIBESD
    int                 fd;
-
 #endif
 
    EDBUG(5, "SoundInit");
 #ifdef HAVE_LIBESD
-   if (!mode.sound)
+   if (!Conf.sound)
       EDBUG_RETURN_;
    if (sound_fd != -1)
       EDBUG_RETURN_;
@@ -259,23 +301,20 @@ SoundInit()
       sound_fd = fd;
    else
      {
-	ASSIGN_ALERT(_("Error initialising sound"), _("OK"), " ", " ");
-	Alert(_
-	      ("Audio was enabled for Enlightenment but there was an error\n"
-	       "communicating with the audio server (Esound). Audio will\n"
-	       "now be disabled.\n"));
-	RESET_ALERT;
-	mode.sound = 0;
+	AlertX(_("Error initialising sound"), _("OK"), " ", " ",
+	       _("Audio was enabled for Enlightenment but there was an error\n"
+		 "communicating with the audio server (Esound). Audio will\n"
+		 "now be disabled.\n"));
+	Conf.sound = 0;
      }
-   EDBUG_RETURN_;
 #else
-   mode.sound = 0;
-   EDBUG_RETURN_;
+   Conf.sound = 0;
 #endif
+   EDBUG_RETURN_;
 }
 
 void
-SoundExit()
+SoundExit(void)
 {
    SoundClass        **lst;
    int                 num, i;
