@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -22,10 +22,10 @@
  */
 #include "E.h"
 
-void                TextDrawRotTo(Window win, Drawable * drawable, int x, int y,
+static void         TextDrawRotTo(Window win, Drawable * drawable, int x, int y,
 				  int w, int h, TextState * ts);
 
-void                TextDrawRotBack(Window win, Drawable drawable, int x, int y,
+static void         TextDrawRotBack(Window win, Drawable drawable, int x, int y,
 				    int w, int h, TextState * ts);
 
 TextState          *
@@ -103,8 +103,8 @@ TextGetState(TextClass * tclass, int active, int sticky, int state)
    EDBUG_RETURN(NULL);
 }
 
-char              **
-TextGetLines(char *text, int *count)
+static char       **
+TextGetLines(const char *text, int *count)
 {
    int                 i, j, k;
    char              **list = NULL;
@@ -139,117 +139,118 @@ void
 TextStateLoadFont(TextState * ts)
 {
    EDBUG(5, "TextStateLoadFont");
-#if USE_FNLIB
-   if ((ts->font) || (ts->efont) || (ts->xfont) || (ts->xfontset))
-#else
-   if ((ts->efont) || (ts->xfont) || (ts->xfontset))
-#endif
-      EDBUG_RETURN_;
+
    if (!ts->fontname)
       EDBUG_RETURN_;
-#if USE_FNLIB
-   if ((!ts->font) && (!ts->efont))
-      ts->font = Fnlib_load_font(pFnlibData, ts->fontname);
-#endif
-#if USE_FNLIB
-   if ((!ts->font) && (!ts->efont))
-#else
-   if (!ts->efont)
-#endif
-     {
-	char                s[4096], w[4046], *dup, *ss;
 
-	dup = NULL;
-	dup = duplicate(ts->fontname);
-	ss = strchr(dup, '/');
-	if (ss)
-	  {
-	     *ss = ' ';
-	     word(dup, 1, w);
-	     Esnprintf(s, sizeof(s), "ttfonts/%s.ttf", w);
-	     word(dup, 2, w);
-	     ss = FindFile(s);
-	     if (ss)
-	       {
-		  ts->efont = Efont_load(ss, atoi(w));
-		  Efree(ss);
-	       }
-	  }
-	if (dup)
-	   Efree(dup);
-     }
-#if USE_FNLIB
-   if ((!ts->font) && (!ts->efont))
-#else
-   if (!ts->efont)
-#endif
-     {
-	if ((!ts->xfont) && (strchr(ts->fontname, ',') == NULL))
-	  {
-	     ts->xfont = XLoadQueryFont(disp, ts->fontname);
-	  }
-	else if (!ts->xfontset)
-	  {
-	     int                 i, missing_cnt, font_cnt;
-	     char              **missing_list, *def_str, **fn;
-	     XFontStruct       **fs;
+   /* Quit if already done */
+   if ((ts->efont) || (ts->xfont) || (ts->xfontset))
+      EDBUG_RETURN_;
 
-	     ts->xfontset =
-		XCreateFontSet(disp, ts->fontname, &missing_list, &missing_cnt,
-			       &def_str);
-	     if (missing_cnt)
-	       {
-		  XFreeStringList(missing_list);
-		  /* EDBUG_RETURN_; */
-	       }
-	     if (!ts->xfontset)
-		EDBUG_RETURN_;
+   ts->need_utf8 = Mode.text.utf8_int;
 
-	     ts->xfontset_ascent = 0;
-	     font_cnt = XFontsOfFontSet(ts->xfontset, &fs, &fn);
-	     for (i = 0; i < font_cnt; i++)
-		ts->xfontset_ascent = MAX(fs[i]->ascent, ts->xfontset_ascent);
-	  }
-     }
+   /* Try FreeType */
+   {
+      char                s[4096], w[4046], *s2, *ss;
+
+      s2 = NULL;
+      s2 = Estrdup(ts->fontname);
+      ss = strchr(s2, '/');
+      if (ss)
+	{
+	   *ss = ' ';
+	   word(s2, 1, w);
+	   Esnprintf(s, sizeof(s), "ttfonts/%s.ttf", w);
+	   word(s2, 2, w);
+	   ss = FindFile(s);
+	   if (ss)
+	     {
+		ts->efont = Efont_load(ss, atoi(w));
+		Efree(ss);
+	     }
+	}
+      if (s2)
+	 Efree(s2);
+
+      if (ts->efont)
+	 ts->need_utf8 = 1;
+   }
+   if (ts->efont)
+      goto done;
+
+   /* Try X11 XCreateFontSet */
+   {
+      int                 i, missing_cnt, font_cnt;
+      char              **missing_list, *def_str, **fn;
+      XFontStruct       **fs;
+
+      ts->xfontset = XCreateFontSet(disp, ts->fontname, &missing_list,
+				    &missing_cnt, &def_str);
+      if (missing_cnt)
+	 XFreeStringList(missing_list);
+
+      if (!ts->xfontset)
+	{
+	   ts->xfontset = XCreateFontSet(disp, "fixed", &missing_list,
+					 &missing_cnt, &def_str);
+	   if (missing_cnt)
+	      XFreeStringList(missing_list);
+	}
+
+      if (ts->xfontset)
+	{
+	   ts->xfontset_ascent = 0;
+	   font_cnt = XFontsOfFontSet(ts->xfontset, &fs, &fn);
+	   for (i = 0; i < font_cnt; i++)
+	      ts->xfontset_ascent = MAX(fs[i]->ascent, ts->xfontset_ascent);
+	}
+   }
+   if (ts->xfontset)
+      goto done;
+
+   /* Try X11 XLoadQueryFont */
+   if (strchr(ts->fontname, ',') == NULL)
+      ts->xfont = XLoadQueryFont(disp, ts->fontname);
+   if (ts->xfont)
+      goto done;
+
+   /* This one really should succeed! */
+   ts->xfont = XLoadQueryFont(disp, "fixed");
+
+ done:
    EDBUG_RETURN_;
 }
 
 void
-TextSize(TextClass * tclass, int active, int sticky, int state, char *text,
-	 int *width, int *height, int fsize)
+TextSize(TextClass * tclass, int active, int sticky, int state,
+	 const char *text, int *width, int *height, int fsize)
 {
+   const char         *str;
    char              **lines;
    int                 i, num_lines;
    TextState          *ts;
 
    EDBUG(4, "TextSize");
+
    *width = 0;
    *height = 0;
-   lines = TextGetLines(text, &num_lines);
-   if (!lines)
+
+   if (!text)
       EDBUG_RETURN_;
+
    ts = TextGetState(tclass, active, sticky, state);
    if (!ts)
       EDBUG_RETURN_;
-   TextStateLoadFont(ts);
-#if USE_FNLIB
-   if (ts->font)
-     {
-	for (i = 0; i < num_lines; i++)
-	  {
-	     int                 high, wid, dummy;
 
-	     Fnlib_measure(pFnlibData, ts->font, 0, 0, 999999, 999999, 0, 0,
-			   fsize, &ts->style, (unsigned char *)lines[i], 0, 0,
-			   &dummy, &dummy, &wid, &high, &dummy, &dummy, &dummy,
-			   &dummy);
-	     *height += high;
-	     if (wid > *width)
-		*width = wid;
-	  }
-     }
-   else
-#endif
+   TextStateLoadFont(ts);
+
+   /* Do encoding conversion, if necessary */
+   str = EstrInt2Enc(text, ts->need_utf8);
+   lines = TextGetLines(str, &num_lines);
+   EstrInt2EncFree(str, ts->need_utf8);
+   if (!lines)
+      EDBUG_RETURN_;
+
    if (ts->efont)
      {
 	for (i = 0; i < num_lines; i++)
@@ -269,8 +270,8 @@ TextSize(TextClass * tclass, int active, int sticky, int state, char *text,
 	  {
 	     XRectangle          ret1, ret2;
 
-	     XmbTextExtents(ts->xfontset, lines[i], strlen(lines[i]), &ret1,
-			    &ret2);
+	     ExTextExtents(ts->xfontset, lines[i], strlen(lines[i]), &ret1,
+			   &ret2);
 	     *height += ret2.height;
 	     if (ret2.width > *width)
 		*width = ret2.width;
@@ -295,9 +296,8 @@ TextSize(TextClass * tclass, int active, int sticky, int state, char *text,
 	  {
 	     int                 wid;
 
-	     wid =
-		XTextWidth16(ts->xfont, (XChar2b *) lines[i],
-			     strlen(lines[i]) / 2);
+	     wid = XTextWidth16(ts->xfont, (XChar2b *) lines[i],
+				strlen(lines[i]) / 2);
 	     *height += ts->xfont->ascent + ts->xfont->descent;
 	     if (wid > *width)
 		*width = wid;
@@ -305,64 +305,54 @@ TextSize(TextClass * tclass, int active, int sticky, int state, char *text,
      }
    freestrlist(lines, num_lines);
    EDBUG_RETURN_;
+   fsize = 0;
 }
 
 void
 TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
-	 char *text, int x, int y, int w, int h, int fsize, int justification)
+	 const char *text, int x, int y, int w, int h, int fsize,
+	 int justification)
 {
+   const char         *str;
    char              **lines;
    int                 i, num_lines;
    TextState          *ts;
    int                 xx, yy;
    XGCValues           gcv;
    static GC           gc = 0;
-   int                 r, g, b;
-
    int                 textwidth_limit, offset_x, offset_y;
    Pixmap              drawable;
 
    EDBUG(4, "TextDraw");
-   lines = TextGetLines(text, &num_lines);
-   if (!lines)
+
+   if (!tclass || !text)
       EDBUG_RETURN_;
+
    ts = TextGetState(tclass, active, sticky, state);
    if (!ts)
       EDBUG_RETURN_;
+
    TextStateLoadFont(ts);
-   xx = x;
-   yy = y;
+
+   /* Do encoding conversion, if necessary */
+   str = EstrInt2Enc(text, ts->need_utf8);
+   lines = TextGetLines(str, &num_lines);
+   EstrInt2EncFree(str, ts->need_utf8);
+   if (!lines)
+      EDBUG_RETURN_;
+
    if (!gc)
       gc = XCreateGC(disp, win, 0, &gcv);
 
-   if (ts->style.orientation == FONT_TO_RIGHT
-       || ts->style.orientation == FONT_TO_LEFT)
+   if (ts->style.orientation == FONT_TO_RIGHT ||
+       ts->style.orientation == FONT_TO_LEFT)
       textwidth_limit = w;
    else
       textwidth_limit = h;
 
-#if USE_FNLIB
-   if (ts->font)
-     {
-	for (i = 0; i < num_lines; i++)
-	  {
-	     int                 high, wid, dummy;
+   xx = x;
+   yy = y;
 
-	     Fnlib_measure(pFnlibData, ts->font, 0, 0, 999999, 999999, 0, 0,
-			   fsize, &ts->style, (unsigned char *)lines[i], 0, 0,
-			   &dummy, &dummy, &wid, &high, &dummy, &dummy, &dummy,
-			   &dummy);
-	     if ((ts->style.orientation == FONT_TO_UP)
-		 || (ts->style.orientation == FONT_TO_DOWN))
-		fsize = w;
-	     xx = x + (((w - wid) * justification) >> 10);
-	     Fnlib_draw(pFnlibData, ts->font, win, 0, xx, yy, w, h, 0, 0, fsize,
-			&ts->style, (unsigned char *)lines[i]);
-	     yy += high;
-	  }
-     }
-   else
-#endif
    if (ts->efont)
      {
 	for (i = 0; i < num_lines; i++)
@@ -406,7 +396,7 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 
 	     if (ts->style.orientation != FONT_TO_RIGHT)
 		drawable =
-		   ECreatePixmap(disp, root.win, wid + 2, ascent + descent + 2,
+		   ECreatePixmap(disp, VRoot.win, wid + 2, ascent + descent + 2,
 				 GetWinDepth(win));
 	     else
 		drawable = win;
@@ -425,50 +415,29 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 
 	     if (ts->effect == 1)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc,
-				 Imlib_best_color_match(pImlibData, &r, &g,
-							&b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  EFont_draw_string(disp, drawable, gc, offset_x + 1,
 				    offset_y + 1, lines[i], ts->efont,
-				    Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    VRoot.vis, VRoot.cmap);
 	       }
 	     else if (ts->effect == 2)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc,
-				 Imlib_best_color_match(pImlibData, &r, &g,
-							&b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  EFont_draw_string(disp, drawable, gc, offset_x - 1, offset_y,
-				    lines[i], ts->efont,
-				    Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 		  EFont_draw_string(disp, drawable, gc, offset_x + 1, offset_y,
-				    lines[i], ts->efont,
-				    Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 		  EFont_draw_string(disp, drawable, gc, offset_x, offset_y - 1,
-				    lines[i], ts->efont,
-				    Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 		  EFont_draw_string(disp, drawable, gc, offset_x, offset_y + 1,
-				    lines[i], ts->efont,
-				    Imlib_get_visual(pImlibData),
-				    Imlib_get_colormap(pImlibData));
+				    lines[i], ts->efont, VRoot.vis, VRoot.cmap);
 	       }
-	     r = ts->fg_col.r;
-	     g = ts->fg_col.g;
-	     b = ts->fg_col.b;
-	     XSetForeground(disp, gc,
-			    Imlib_best_color_match(pImlibData, &r, &g, &b));
+	     EAllocColor(&ts->fg_col);
+	     XSetForeground(disp, gc, ts->fg_col.pixel);
 	     EFont_draw_string(disp, drawable, gc, offset_x, offset_y, lines[i],
-			       ts->efont, Imlib_get_visual(pImlibData),
-			       Imlib_get_colormap(pImlibData));
+			       ts->efont, VRoot.vis, VRoot.cmap);
 
 	     TextDrawRotBack(win, drawable, xx - 1, yy - 1 - ascent, wid + 2,
 			     ascent + descent + 2, ts);
@@ -483,8 +452,8 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 	  {
 	     XRectangle          ret1, ret2;
 
-	     XmbTextExtents(ts->xfontset, lines[i], strlen(lines[i]), &ret1,
-			    &ret2);
+	     ExTextExtents(ts->xfontset, lines[i], strlen(lines[i]), &ret1,
+			   &ret2);
 	     if (ret2.width > textwidth_limit)
 	       {
 		  char               *new_line;
@@ -555,8 +524,8 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 				   lines[i] + ((len - nuke_count) / 2) +
 				   nuke_count);
 			 }
-		       XmbTextExtents(ts->xfontset, new_line, strlen(new_line),
-				      &ret1, &ret2);
+		       ExTextExtents(ts->xfontset, new_line, strlen(new_line),
+				     &ret1, &ret2);
 		    }
 		  Efree(lines[i]);
 		  lines[i] = new_line;
@@ -569,7 +538,7 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 
 	     if (ts->style.orientation != FONT_TO_RIGHT)
 		drawable =
-		   ECreatePixmap(disp, root.win, ret2.width + 2,
+		   ECreatePixmap(disp, VRoot.win, ret2.width + 2,
 				 ret2.height + 2, GetWinDepth(win));
 	     else
 		drawable = win;
@@ -589,39 +558,28 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 
 	     if (ts->effect == 1)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc,
-				 Imlib_best_color_match(pImlibData, &r, &g,
-							&b));
-		  XmbDrawString(disp, drawable, ts->xfontset, gc, offset_x + 1,
-				offset_y + 1, lines[i], strlen(lines[i]));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
+		  ExDrawString(disp, drawable, ts->xfontset, gc, offset_x + 1,
+			       offset_y + 1, lines[i], strlen(lines[i]));
 	       }
 	     else if (ts->effect == 2)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc,
-				 Imlib_best_color_match(pImlibData, &r, &g,
-							&b));
-		  XmbDrawString(disp, drawable, ts->xfontset, gc, offset_x - 1,
-				offset_y, lines[i], strlen(lines[i]));
-		  XmbDrawString(disp, drawable, ts->xfontset, gc, offset_x + 1,
-				offset_y, lines[i], strlen(lines[i]));
-		  XmbDrawString(disp, drawable, ts->xfontset, gc, offset_x,
-				offset_y - 1, lines[i], strlen(lines[i]));
-		  XmbDrawString(disp, drawable, ts->xfontset, gc, offset_x,
-				offset_y + 1, lines[i], strlen(lines[i]));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
+		  ExDrawString(disp, drawable, ts->xfontset, gc, offset_x - 1,
+			       offset_y, lines[i], strlen(lines[i]));
+		  ExDrawString(disp, drawable, ts->xfontset, gc, offset_x + 1,
+			       offset_y, lines[i], strlen(lines[i]));
+		  ExDrawString(disp, drawable, ts->xfontset, gc, offset_x,
+			       offset_y - 1, lines[i], strlen(lines[i]));
+		  ExDrawString(disp, drawable, ts->xfontset, gc, offset_x,
+			       offset_y + 1, lines[i], strlen(lines[i]));
 	       }
-	     r = ts->fg_col.r;
-	     g = ts->fg_col.g;
-	     b = ts->fg_col.b;
-	     XSetForeground(disp, gc,
-			    Imlib_best_color_match(pImlibData, &r, &g, &b));
-	     XmbDrawString(disp, drawable, ts->xfontset, gc, offset_x, offset_y,
-			   lines[i], strlen(lines[i]));
+	     EAllocColor(&ts->fg_col);
+	     XSetForeground(disp, gc, ts->fg_col.pixel);
+	     ExDrawString(disp, drawable, ts->xfontset, gc, offset_x, offset_y,
+			  lines[i], strlen(lines[i]));
 
 	     TextDrawRotBack(win, drawable, xx - 1,
 			     yy - (ts->xfontset_ascent) - 1, ret2.width + 2,
@@ -676,7 +634,7 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 
 	     if (ts->style.orientation != FONT_TO_RIGHT)
 		drawable =
-		   ECreatePixmap(disp, root.win, wid + 2, ascent + descent + 2,
+		   ECreatePixmap(disp, VRoot.win, wid + 2, ascent + descent + 2,
 				 GetWinDepth(win));
 	     else
 		drawable = win;
@@ -695,23 +653,15 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 
 	     if (ts->effect == 1)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc,
-				 Imlib_best_color_match(pImlibData, &r, &g,
-							&b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XDrawString(disp, drawable, gc, offset_x + 1, offset_y + 1,
 			      lines[i], strlen(lines[i]));
 	       }
 	     else if (ts->effect == 2)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc,
-				 Imlib_best_color_match(pImlibData, &r, &g,
-							&b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XDrawString(disp, drawable, gc, offset_x - 1, offset_y,
 			      lines[i], strlen(lines[i]));
 		  XDrawString(disp, drawable, gc, offset_x + 1, offset_y,
@@ -721,11 +671,8 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 		  XDrawString(disp, drawable, gc, offset_x, offset_y + 1,
 			      lines[i], strlen(lines[i]));
 	       }
-	     r = ts->fg_col.r;
-	     g = ts->fg_col.g;
-	     b = ts->fg_col.b;
-	     XSetForeground(disp, gc,
-			    Imlib_best_color_match(pImlibData, &r, &g, &b));
+	     EAllocColor(&ts->fg_col);
+	     XSetForeground(disp, gc, ts->fg_col.pixel);
 	     XDrawString(disp, drawable, gc, offset_x, offset_y, lines[i],
 			 strlen(lines[i]));
 
@@ -743,9 +690,8 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 	  {
 	     int                 wid, ascent, descent;
 
-	     wid =
-		XTextWidth16(ts->xfont, (XChar2b *) lines[i],
-			     strlen(lines[i]) / 2);
+	     wid = XTextWidth16(ts->xfont, (XChar2b *) lines[i],
+				strlen(lines[i]) / 2);
 	     ascent = ts->xfont->ascent;
 	     descent = ts->xfont->descent;
 	     if (wid > textwidth_limit)
@@ -784,7 +730,7 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 
 	     if (ts->style.orientation != FONT_TO_RIGHT)
 		drawable =
-		   ECreatePixmap(disp, root.win, wid + 2, ascent + descent + 2,
+		   ECreatePixmap(disp, VRoot.win, wid + 2, ascent + descent + 2,
 				 GetWinDepth(win));
 	     else
 		drawable = win;
@@ -803,23 +749,15 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 
 	     if (ts->effect == 1)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc,
-				 Imlib_best_color_match(pImlibData, &r, &g,
-							&b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XDrawString16(disp, drawable, gc, offset_x + 1, offset_y + 1,
 				(XChar2b *) lines[i], strlen(lines[i]) / 2);
 	       }
 	     else if (ts->effect == 2)
 	       {
-		  r = ts->bg_col.r;
-		  g = ts->bg_col.g;
-		  b = ts->bg_col.b;
-		  XSetForeground(disp, gc,
-				 Imlib_best_color_match(pImlibData, &r, &g,
-							&b));
+		  EAllocColor(&ts->bg_col);
+		  XSetForeground(disp, gc, ts->bg_col.pixel);
 		  XDrawString16(disp, drawable, gc, offset_x - 1, offset_y,
 				(XChar2b *) lines[i], strlen(lines[i]) / 2);
 		  XDrawString16(disp, drawable, gc, offset_y + 1, offset_y,
@@ -829,11 +767,8 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
 		  XDrawString16(disp, drawable, gc, offset_x, offset_y + 1,
 				(XChar2b *) lines[i], strlen(lines[i]) / 2);
 	       }
-	     r = ts->fg_col.r;
-	     g = ts->fg_col.g;
-	     b = ts->fg_col.b;
-	     XSetForeground(disp, gc,
-			    Imlib_best_color_match(pImlibData, &r, &g, &b));
+	     EAllocColor(&ts->fg_col);
+	     XSetForeground(disp, gc, ts->fg_col.pixel);
 	     XDrawString16(disp, drawable, gc, offset_x, offset_y,
 			   (XChar2b *) lines[i], strlen(lines[i]) / 2);
 
@@ -846,86 +781,96 @@ TextDraw(TextClass * tclass, Window win, int active, int sticky, int state,
      }
    freestrlist(lines, num_lines);
    EDBUG_RETURN_;
+   fsize = 0;
 }
 
 void
 TextDrawRotTo(Window win, Drawable * drawable, int x, int y, int w, int h,
 	      TextState * ts)
 {
-   ImlibImage         *ii = NULL;
+   Imlib_Image        *ii = NULL;
    int                 win_x, win_y;
    unsigned int        win_w, win_h, win_b, win_d;
+   Window              rr;
 
    switch (ts->style.orientation)
      {
      case FONT_TO_UP:
-	ii = Imlib_create_image_from_drawable(pImlibData, win, 0, y, x, h, w);
-	Imlib_rotate_image(pImlibData, ii, 1);
-	Imlib_flip_image_horizontal(pImlibData, ii);
-	Imlib_paste_image(pImlibData, ii, *drawable, 0, 0, w, h);
+	XSync(disp, False);	/* Workaround for crash seen with Absolute E (Imlib2?) */
+	imlib_context_set_drawable(win);
+	ii = imlib_create_image_from_drawable(0, y, x, h, w, 0);
+	imlib_context_set_image(ii);
+	imlib_image_orientate(1);
+	imlib_context_set_drawable(*drawable);
+	imlib_render_image_on_drawable_at_size(0, 0, w, h);
 	break;
      case FONT_TO_DOWN:
-	EGetGeometry(disp, win, &(root.win), &win_x, &win_y, &win_w, &win_h,
+	EGetGeometry(disp, win, &rr, &win_x, &win_y, &win_w, &win_h,
 		     &win_b, &win_d);
-	ii =
-	   Imlib_create_image_from_drawable(pImlibData, win, 0, win_w - y - h,
-					    x, h, w);
-	Imlib_rotate_image(pImlibData, ii, -1);
-	Imlib_flip_image_vertical(pImlibData, ii);
-	Imlib_paste_image(pImlibData, ii, *drawable, 0, 0, w, h);
+	XSync(disp, False);	/* Workaround for crash seen with Absolute E (Imlib2?) */
+	imlib_context_set_drawable(win);
+	ii = imlib_create_image_from_drawable(0, win_w - y - h, x, h, w, 0);
+	imlib_context_set_image(ii);
+	imlib_image_orientate(3);
+	imlib_context_set_drawable(*drawable);
+	imlib_render_image_on_drawable_at_size(0, 0, w, h);
 	break;
      case FONT_TO_LEFT:	/* Holy carumba! That's for yoga addicts, maybe .... */
-	ii = Imlib_create_image_from_drawable(pImlibData, win, 0, x, y, w, h);
-	Imlib_flip_image_vertical(pImlibData, ii);
-	Imlib_flip_image_horizontal(pImlibData, ii);
-	Imlib_paste_image(pImlibData, ii, *drawable, 0, 0, w, h);
+	XSync(disp, False);	/* Workaround for crash seen with Absolute E (Imlib2?) */
+	imlib_context_set_drawable(win);
+	ii = imlib_create_image_from_drawable(0, x, y, w, h, 0);
+	imlib_context_set_image(ii);
+	imlib_image_orientate(2);
+	imlib_context_set_drawable(*drawable);
+	imlib_render_image_on_drawable_at_size(0, 0, w, h);
 	break;
      default:
 	break;
      }
    if (ii)
-      Imlib_destroy_image(pImlibData, ii);
+      imlib_free_image();
 }
 
 void
 TextDrawRotBack(Window win, Pixmap drawable, int x, int y, int w, int h,
 		TextState * ts)
 {
-   ImlibImage         *ii = NULL;
+   Imlib_Image        *ii = NULL;
    int                 win_x, win_y;
    unsigned int        win_w, win_h, win_b, win_d;
+   Window              rr;
 
    switch (ts->style.orientation)
      {
      case FONT_TO_UP:
-	ii =
-	   Imlib_create_image_from_drawable(pImlibData, drawable, 0, 0, 0, w,
-					    h);
-	Imlib_rotate_image(pImlibData, ii, -1);
-	Imlib_flip_image_vertical(pImlibData, ii);
-	Imlib_paste_image(pImlibData, ii, win, y, x, h, w);
+	imlib_context_set_drawable(drawable);
+	ii = imlib_create_image_from_drawable(0, 0, 0, w, h, 0);
+	imlib_context_set_image(ii);
+	imlib_image_orientate(3);
+	imlib_context_set_drawable(win);
+	imlib_render_image_on_drawable_at_size(y, x, h, w);
 	break;
      case FONT_TO_DOWN:
-	EGetGeometry(disp, win, &(root.win), &win_x, &win_y, &win_w, &win_h,
+	imlib_context_set_drawable(drawable);
+	EGetGeometry(disp, win, &rr, &win_x, &win_y, &win_w, &win_h,
 		     &win_b, &win_d);
-	ii =
-	   Imlib_create_image_from_drawable(pImlibData, drawable, 0, 0, 0, w,
-					    h);
-	Imlib_rotate_image(pImlibData, ii, 1);
-	Imlib_flip_image_horizontal(pImlibData, ii);
-	Imlib_paste_image(pImlibData, ii, win, win_w - y - h, x, h, w);
+	ii = imlib_create_image_from_drawable(0, 0, 0, w, h, 0);
+	imlib_context_set_image(ii);
+	imlib_image_orientate(1);
+	imlib_context_set_drawable(win);
+	imlib_render_image_on_drawable_at_size(win_w - y - h, x, h, w);
 	break;
      case FONT_TO_LEFT:	/* Holy carumba! That's for yoga addicts, maybe .... */
-	ii =
-	   Imlib_create_image_from_drawable(pImlibData, drawable, 0, 0, 0, w,
-					    h);
-	Imlib_flip_image_vertical(pImlibData, ii);
-	Imlib_flip_image_horizontal(pImlibData, ii);
-	Imlib_paste_image(pImlibData, ii, win, x, y, w, h);
+	imlib_context_set_drawable(drawable);
+	ii = imlib_create_image_from_drawable(0, 0, 0, w, h, 0);
+	imlib_context_set_image(ii);
+	imlib_image_orientate(2);
+	imlib_context_set_drawable(win);
+	imlib_render_image_on_drawable_at_size(x, y, w, h);
 	break;
      default:
 	break;
      }
    if (ii)
-      Imlib_destroy_image(pImlibData, ii);
+      imlib_free_image();
 }
