@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,6 +21,7 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 #include "E.h"
+#include <sys/time.h>
 
 double
 GetTime(void)
@@ -34,7 +35,7 @@ GetTime(void)
 static Qentry      *q_first = NULL;
 
 void
-DoIn(char *name, double in_time, void (*func) (int val, void *data),
+DoIn(const char *name, double in_time, void (*func) (int val, void *data),
      int runtime_val, void *runtime_data)
 {
    Qentry             *qe, *ptr, *pptr;
@@ -45,7 +46,7 @@ DoIn(char *name, double in_time, void (*func) (int val, void *data),
    qe = Emalloc(sizeof(Qentry));
    if (!qe)
       EDBUG_RETURN_;
-   qe->name = duplicate(name);
+   qe->name = Estrdup(name);
    qe->func = func;
    qe->next = NULL;
    qe->in_time = in_time;
@@ -140,7 +141,7 @@ HandleTimerEvent(void)
 }
 
 void
-RemoveTimerEvent(char *name)
+RemoveTimerEvent(const char *name)
 {
    Qentry             *qe, *ptr, *pptr;
 
