@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -38,32 +38,38 @@
 
 #include "E.h"
 
+typedef struct
+{
+   EWin               *ewin;
+   Window              win;
+   char               *txt;
+} WarplistItem;
+
 static void         WarpFocusShowTitle(EWin * ewin);
 static void         WarpFocusHideTitle(void);
 
 static int          warpFocusIndex = 0;
 static char         warpFocusTitleShowing = 0;
 static Window       warpFocusTitleWindow = 0;
-static int          warptitles_num = 0;
-static Window      *warptitles = NULL;
-static EWin       **warptitles_ewin = NULL;
-static int          warpFocusKey = 0;
+static unsigned int warpFocusKey = 0;
+static int          warplist_num = 0;
+static WarplistItem *warplist;
 
 int
 WarpFocusHandleEvent(XEvent * ev)
 {
    EDBUG(5, "WarpFocusHandleEvent");
 
-   if (!mode.display_warp)
+   if (!Conf.warplist.enable)
       EDBUG_RETURN(0);
 
    if (ev->type != KeyPress && ev->type != KeyRelease)
       EDBUG_RETURN(0);
 
-#if 0
-   printf("WarpFocusHandleEvent win=%#x key=%#x(%#x) %d\n",
-	  (unsigned)ev->xkey.window, ev->xkey.keycode,
-	  warpFocusKey, warpFocusTitleShowing);
+#if 0				/* Debug */
+   Eprintf("WarpFocusHandleEvent win=%#x key=%#x(%#x) %d\n",
+	   (unsigned)ev->xkey.window, ev->xkey.keycode,
+	   warpFocusKey, warpFocusTitleShowing);
 #endif
    if (warpFocusTitleShowing)
      {
@@ -81,48 +87,44 @@ WarpFocusHandleEvent(XEvent * ev)
 void
 WarpFocus(int delta)
 {
-   EWin              **lst0, **lst, *ewin;
+   EWin               *const *lst0;
+   EWin              **lst, *ewin;
    int                 i, num0, num;
 
    EDBUG(5, "WarpFocus");
 
-   if (!mode.display_warp)
+   if (!Conf.warplist.enable)
       EDBUG_RETURN_;
 
    lst = (EWin **) ListItemType(&num, LIST_TYPE_WARP_RING);
    if (!lst)
      {
-	lst0 = (EWin **) ListItemType(&num0, LIST_TYPE_EWIN);
+	lst0 = EwinListGetFocus(&num0);
 	num = 0;
 	lst = NULL;
-	if (lst0)
+	for (i = num0 - 1; i >= 0; --i)
 	  {
-	     for (i = num0 - 1; i >= 0; --i)
-	       {
-		  ewin = lst0[i];
-		  if (((ewin->sticky) || (ewin->desktop == desks.current))
-		      && (ewin->x + ewin->w > 0) && (ewin->x < root.w)
-		      && (ewin->y + ewin->h > 0) && (ewin->y < root.h)
-		      && (!ewin->skipfocus) && !(ewin->shaded
-						 && !mode.warpshaded)
-		      && (!ewin->menu) && (!ewin->pager) && !(ewin->sticky
-							      && !mode.
-							      warpsticky)
-		      && (!ewin->ibox) && !(ewin->iconified
-					    && !mode.warpiconified)
-		      /*&& (ewin->client.mwm_decor_title) &&
-		       * (ewin->client.mwm_decor_border) */
-		     )
-		     AddItem(ewin, "", 0, LIST_TYPE_WARP_RING);
-	       }
+	     ewin = lst0[i];
+	     if (		/* Either visible or iconified */
+		   ((EwinIsOnScreen(ewin)) || (ewin->iconified))
+		   /* Exclude windows that explicitely say so */
+		   && (!ewin->skipfocus)
+		   /* Keep shaded windows if conf say so */
+		   && ((!ewin->shaded) || (Conf.warplist.warpshaded))
+		   /* Keep sticky windows if conf say so */
+		   && ((!ewin->sticky) || (Conf.warplist.warpsticky))
+		   /* Keep iconified windows if conf say so */
+		   && ((!ewin->iconified) || (Conf.warplist.warpiconified))
+		   /*&& (ewin->client.mwm_decor_title) &&
+		    * (ewin->client.mwm_decor_border) */
+		)
+		AddItem(ewin, "", 0, LIST_TYPE_WARP_RING);
 	  }
-	MoveItemToListBottom(mode.focuswin, LIST_TYPE_WARP_RING);
+	MoveItemToListBottom(GetFocusEwin(), LIST_TYPE_WARP_RING);
 	lst = (EWin **) ListItemType(&num, LIST_TYPE_WARP_RING);
 	warpFocusIndex = num - 1;
      }
 
-   ewin = NULL;
-
    if (lst)
      {
 	warpFocusIndex = (warpFocusIndex + num + delta) % num;
@@ -131,17 +133,16 @@ WarpFocus(int delta)
 	   ewin = NULL;
 	if (ewin)
 	  {
-	     if (mode.raise_on_next_focus)
+	     if (Conf.focus.raise_on_next)
 		RaiseEwin(ewin);
-	     if (mode.warp_on_next_focus && !ewin->iconified)
-		XWarpPointer(disp, None, ewin->win, 0, 0, 0, 0, ewin->w / 2,
-			     ewin->h / 2);
-	     /* if (mode.focusmode == FOCUS_CLICK) */
-	     /* FocusToEWin(ewin); */
-	     if (mode.warpfocused && !ewin->iconified)
-		FocusToEWin(ewin);
+	     if (Conf.focus.warp_on_next)
+		if (ewin != Mode.mouse_over_ewin && !ewin->iconified)
+		   XWarpPointer(disp, None, ewin->win, 0, 0, 0, 0,
+				ewin->w / 2, ewin->h / 2);
+	     if (Conf.warplist.warpfocused)
+		FocusToEWin(ewin, FOCUS_SET);
+	     WarpFocusShowTitle(ewin);
 	  }
-	WarpFocusShowTitle(ewin);
 	Efree(lst);
      }
 
@@ -166,19 +167,18 @@ WarpFocusFinish(void)
 	   ewin = NULL;
 	if (ewin)
 	  {
-	     if (mode.warpiconified && ewin->iconified)
+	     if (Conf.warplist.warpiconified && ewin->iconified)
 		DeIconifyEwin(ewin);
-	     FocusToEWin(ewin);
-	     if (mode.warp_after_next_focus || mode.warp_on_next_focus)
-	       {
-		  XWarpPointer(disp, None, ewin->win, 0, 0, 0, 0, ewin->w / 2,
-			       ewin->h / 2);
-	       }
-	     if (mode.raise_after_next_focus)
+	     if (Conf.warplist.raise_on_select)
 		RaiseEwin(ewin);
+	     if (Conf.warplist.warp_on_select)
+		if (ewin != Mode.mouse_over_ewin)
+		   XWarpPointer(disp, None, ewin->win, 0, 0, 0, 0,
+				ewin->w / 2, ewin->h / 2);
+	     FocusToEWin(ewin, FOCUS_SET);
 	  }
-	/*printf ("Raise: %s\n", ewin->client.title); */
 	Efree(lst);
+
 	while (RemoveItem("", 0, LIST_FINDBY_NONE, LIST_TYPE_WARP_RING));
      }
 
@@ -193,6 +193,7 @@ WarpFocusShowTitle(EWin * ewin)
    char                pq;
    int                 i, x, y, w, h, num, ww, hh;
    static int          mw, mh;
+   char                s[1024];
 
    tc = FindItem("WARPFOCUS", 0, LIST_FINDBY_NAME, LIST_TYPE_TCLASS);
    if (!tc)
@@ -204,10 +205,10 @@ WarpFocusShowTitle(EWin * ewin)
       return;
 
    if (!warpFocusTitleWindow)
-      warpFocusTitleWindow = ECreateWindow(root.win, 0, 0, 1, 1, 1);
+      warpFocusTitleWindow = ECreateWindow(VRoot.win, 0, 0, 1, 1, 1);
 
-   pq = queue_up;
-   queue_up = 0;
+   pq = Mode.queue_up;
+   Mode.queue_up = 0;
    XRaiseWindow(disp, warpFocusTitleWindow);
 
    if (!warpFocusTitleShowing)
@@ -221,17 +222,18 @@ WarpFocusShowTitle(EWin * ewin)
 	  {
 	     for (i = 0; i < num; i++)
 	       {
-		  warptitles_num++;
-		  warptitles =
-		     Erealloc(warptitles, (sizeof(Window) * warptitles_num));
-		  warptitles_ewin =
-		     Erealloc(warptitles_ewin,
-			      (sizeof(EWin *) * warptitles_num));
-		  warptitles[warptitles_num - 1] =
+		  warplist_num++;
+		  warplist = Erealloc(warplist,
+				      warplist_num * sizeof(WarplistItem));
+		  warplist[warplist_num - 1].win =
 		     ECreateWindow(warpFocusTitleWindow, 0, 0, 1, 1, 0);
-		  EMapWindow(disp, warptitles[warptitles_num - 1]);
-		  warptitles_ewin[warptitles_num - 1] = lst[i];
-		  TextSize(tc, 0, 0, 0, lst[i]->client.title, &ww, &hh, 17);
+		  EMapWindow(disp, warplist[warplist_num - 1].win);
+		  warplist[warplist_num - 1].ewin = lst[i];
+		  Esnprintf(s, sizeof(s), (lst[i]->iconified) ? "[%s]" : "%s",
+			    EwinGetTitle(lst[i]));
+		  warplist[warplist_num - 1].txt = strdup(s);
+		  TextSize(tc, 0, 0, 0, warplist[warplist_num - 1].txt, &ww,
+			   &hh, 17);
 		  if (ww > w)
 		     w = ww;
 		  if (hh > h)
@@ -241,20 +243,22 @@ WarpFocusShowTitle(EWin * ewin)
 	  }
 	w += (ic->padding.left + ic->padding.right);
 	h += (ic->padding.top + ic->padding.bottom);
-	GetPointerScreenGeometry(&x, &y, &ww, &hh);
+	GetPointerScreenAvailableArea(&x, &y, &ww, &hh);
 	x += (ww - w) / 2;
-	y += (hh - h * warptitles_num) / 2;
+	y += (hh - h * warplist_num) / 2;
 	mw = w;
 	mh = h;
 	EMoveResizeWindow(disp, warpFocusTitleWindow, x, y, w,
-			  (h * warptitles_num));
-	for (i = 0; i < warptitles_num; i++)
+			  (h * warplist_num));
+	for (i = 0; i < warplist_num; i++)
 	  {
-	     EMoveResizeWindow(disp, warptitles[i], 0, (h * i), mw, mh);
-	     if (ewin == warptitles_ewin[i])
-		IclassApply(ic, warptitles[i], mw, mh, 0, 0, STATE_CLICKED, 0);
+	     EMoveResizeWindow(disp, warplist[i].win, 0, (h * i), mw, mh);
+	     if (ewin == warplist[i].ewin)
+		IclassApply(ic, warplist[i].win, mw, mh, 0, 0, STATE_CLICKED,
+			    0, ST_WARPLIST);
 	     else
-		IclassApply(ic, warptitles[i], mw, mh, 0, 0, STATE_NORMAL, 0);
+		IclassApply(ic, warplist[i].win, mw, mh, 0, 0, STATE_NORMAL,
+			    0, ST_WARPLIST);
 	  }
 	PropagateShapes(warpFocusTitleWindow);
 	EMapWindow(disp, warpFocusTitleWindow);
@@ -267,32 +271,26 @@ WarpFocusShowTitle(EWin * ewin)
 		      GrabModeAsync, CurrentTime);
      }
 
-   for (i = 0; i < warptitles_num; i++)
+   for (i = 0; i < warplist_num; i++)
      {
-	if (!FindItem
-	    ((char *)warptitles_ewin[i], 0, LIST_FINDBY_POINTER,
-	     LIST_TYPE_EWIN))
-	   warptitles_ewin[i] = NULL;
-	if (warptitles_ewin[i])
+	if (!FindItem((char *)warplist[i].ewin, 0, LIST_FINDBY_POINTER,
+		      LIST_TYPE_EWIN))
+	   warplist[i].ewin = NULL;
+	if (warplist[i].ewin)
 	  {
-	     if (ewin == warptitles_ewin[i])
-	       {
-		  IclassApply(ic, warptitles[i], mw, mh, 0, 0, STATE_CLICKED,
-			      0);
-		  TclassApply(ic, warptitles[i], mw, mh, 0, 0, STATE_CLICKED, 0,
-			      tc, warptitles_ewin[i]->client.title);
-	       }
-	     else
-	       {
-		  IclassApply(ic, warptitles[i], mw, mh, 0, 0, STATE_NORMAL, 0);
-		  TclassApply(ic, warptitles[i], mw, mh, 0, 0, STATE_NORMAL, 0,
-			      tc, warptitles_ewin[i]->client.title);
-	       }
+	     int                 state;
+
+	     state = (ewin == warplist[i].ewin) ? STATE_CLICKED : STATE_NORMAL;
+
+	     IclassApply(ic, warplist[i].win, mw, mh, 0, 0, state, 0,
+			 ST_WARPLIST);
+	     TclassApply(ic, warplist[i].win, mw, mh, 0, 0, state, 0, tc,
+			 warplist[i].txt);
 	  }
      }
 
    PropagateShapes(warpFocusTitleWindow);
-   queue_up = pq;
+   Mode.queue_up = pq;
    XFlush(disp);
    warpFocusTitleShowing = 1;
 }
@@ -305,16 +303,16 @@ WarpFocusHideTitle(void)
    if (warpFocusTitleWindow)
      {
 	EUnmapWindow(disp, warpFocusTitleWindow);
-	for (i = 0; i < warptitles_num; i++)
-	   EDestroyWindow(disp, warptitles[i]);
+	for (i = 0; i < warplist_num; i++)
+	  {
+	     EDestroyWindow(disp, warplist[i].win);
+	     Efree(warplist[i].txt);
+	  }
      }
 
-   if (warptitles)
-      Efree(warptitles);
-   if (warptitles_ewin)
-      Efree(warptitles_ewin);
-   warptitles = NULL;
-   warptitles_ewin = NULL;
-   warptitles_num = 0;
+   if (warplist)
+      Efree(warplist);
+   warplist = NULL;
+   warplist_num = 0;
    warpFocusTitleShowing = 0;
 }
