@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2000-2004 Carsten Haitzler, Geoff Harrison and various contributors
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -20,12 +20,30 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
+#define INCLUDE_NEW_EVENT_DISPATCHER 0
 #include "E.h"
 #include <X11/Xutil.h>
 #include <X11/Xresource.h>
 
+#if INCLUDE_NEW_EVENT_DISPATCHER
+typedef struct
+{
+   EventCallbackFunc  *func;
+   void               *prm;
+} EventCallbackItem;
+
+typedef struct
+{
+   int                 num;
+   EventCallbackItem  *lst;
+} EventCallbackList;
+#endif /* INCLUDE_NEW_EVENT_DISPATCHER */
+
 typedef struct _exid
 {
+#if INCLUDE_NEW_EVENT_DISPATCHER
+   EventCallbackList   cbl;
+#endif				/* INCLUDE_NEW_EVENT_DISPATCHER */
    Window              parent;
    Window              win;
    int                 x, y, w, h;
@@ -36,8 +54,7 @@ typedef struct _exid
    int                 depth;
    Pixmap              bgpmap;
    int                 bgcol;
-}
-EXID;
+} EXID;
 
 static XContext     xid_context = 0;
 
@@ -46,20 +63,7 @@ NewXID(void)
 {
    EXID               *xid;
 
-   xid = Emalloc(sizeof(EXID));
-   xid->parent = 0;
-   xid->win = 0;
-   xid->x = 0;
-   xid->y = 0;
-   xid->w = 0;
-   xid->h = 0;
-   xid->num_rect = 0;
-   xid->ord = 0;
-   xid->rects = NULL;
-   xid->depth = 0;
-   xid->mapped = 0;
-   xid->bgpmap = 0;
-   xid->bgcol = 0;
+   xid = Ecalloc(1, sizeof(EXID));
 
    return xid;
 }
@@ -76,12 +80,15 @@ AddXID(EXID * xid)
 static EXID        *
 FindXID(Window win)
 {
-   EXID               *xid = NULL;
+   EXID               *xid;
+   XPointer            xp;
 
    if (xid_context == 0)
       xid_context = XUniqueContext();
-   if (XFindContext(disp, win, xid_context, (XPointer *) & xid) == XCNOENT)
-      xid = NULL;
+   xp = NULL;
+   if (XFindContext(disp, win, xid_context, &xp) == XCNOENT)
+      xp = NULL;
+   xid = (EXID *) xp;
    return xid;
 }
 
@@ -114,10 +121,79 @@ SetXID(Window win, Window parent, int x, int y, int w, int h, int depth)
    xid->y = y;
    xid->w = w;
    xid->h = h;
-   xid->depth = root.depth;
+   xid->depth = VRoot.depth;
    AddXID(xid);
+   return;
+   depth = 0;
 }
 
+#if INCLUDE_NEW_EVENT_DISPATCHER
+void
+EventCallbackRegister(Window win, int type __UNUSED__, EventCallbackFunc * func,
+		      void *prm)
+{
+   EXID               *xid;
+   EventCallbackItem  *eci;
+
+   xid = FindXID(win);
+   if (xid == NULL)
+      return;
+
+   xid->cbl.num++;
+   xid->cbl.lst =
+      Erealloc(xid->cbl.lst, xid->cbl.num * sizeof(EventCallbackItem));
+   eci = xid->cbl.lst + xid->cbl.num - 1;
+   eci->func = func;
+   eci->prm = prm;
+}
+
+void
+EventCallbackUnregister(Window win, int type __UNUSED__,
+			EventCallbackFunc * func, void *prm)
+{
+   EXID               *xid;
+   EventCallbackList  *ecl;
+   EventCallbackItem  *eci;
+   int                 i;
+
+   xid = FindXID(win);
+   if (xid == NULL)
+      return;
+
+   ecl = &xid->cbl;
+   eci = ecl->lst;
+   for (i = 0; i < ecl->num; i++, eci++)
+      if (eci->func == func && eci->prm == prm)
+	{
+	   /* Well - remove it */
+	   return;
+	}
+}
+
+void
+EventCallbacksProcess(XEvent * ev)
+{
+   EXID               *xid;
+   EventCallbackList  *ecl;
+   EventCallbackItem  *eci;
+   int                 i;
+
+   xid = FindXID(ev->xany.window);
+   if (xid == NULL)
+      return;
+
+   ecl = &xid->cbl;
+   eci = ecl->lst;
+   for (i = 0; i < ecl->num; i++, eci++)
+     {
+	if (EventDebug(200))
+	   Eprintf("EventDispatch: type=%d win=%#lx func=%p prm=%p\n",
+		   ev->type, ev->xany.window, eci->func, eci->prm);
+	eci->func(ev, eci->prm);
+     }
+}
+#endif /* INCLUDE_NEW_EVENT_DISPATCHER */
+
 Pixmap
 ECreatePixmap(Display * display, Drawable d, unsigned int width,
 	      unsigned int height, unsigned depth)
@@ -143,22 +219,22 @@ ECreateWindow(Window parent, int x, int y, int w, int h, int saveunder)
    EDBUG(6, "ECreateWindow");
    attr.backing_store = NotUseful;
    attr.override_redirect = True;
-   attr.colormap = root.cmap;
+   attr.colormap = VRoot.cmap;
    attr.border_pixel = 0;
 /*   attr.background_pixel = 0; */
    attr.background_pixmap = None;
-   if ((saveunder == 1) && (mode.save_under))
+   if ((saveunder == 1) && (Conf.save_under))
       attr.save_under = True;
    else if (saveunder == 2)
       attr.save_under = True;
    else
       attr.save_under = False;
    win =
-      XCreateWindow(disp, parent, x, y, w, h, 0, root.depth, InputOutput,
-		    root.vis,
+      XCreateWindow(disp, parent, x, y, w, h, 0, VRoot.depth, InputOutput,
+		    VRoot.vis,
 		    CWOverrideRedirect | CWSaveUnder | CWBackingStore |
 		    CWColormap | CWBackPixmap | CWBorderPixel, &attr);
-   SetXID(win, parent, x, y, w, h, root.depth);
+   SetXID(win, parent, x, y, w, h, VRoot.depth);
 
    EDBUG_RETURN(win);
 }
@@ -227,8 +303,8 @@ EDestroyWindow(Display * d, Window win)
 {
    EXID               *xid;
 
-   if ((mode.slideout) && (mode.slideout->from_win == win))
-      HideSlideout(mode.slideout, 0);
+   SlideoutsHideIfContextWin(win);
+
    xid = FindXID(win);
    if (xid)
      {
@@ -256,6 +332,8 @@ void
 EForgetWindow(Display * d, Window win)
 {
    DelXID(win);
+   return;
+   d = NULL;
 }
 
 void
@@ -340,6 +418,30 @@ EShapeCombineMask(Display * d, Window win, int dest, int x, int y, Pixmap pmap,
       XShapeCombineMask(d, win, dest, x, y, pmap, op);
 }
 
+void
+EShapeCombineMaskTiled(Display * d, Window win, int dest, int x, int y,
+		       Pixmap pmap, int op, int w, int h)
+{
+   XGCValues           gcv;
+   GC                  gc;
+   Window              tm;
+
+   gcv.fill_style = FillTiled;
+   gcv.tile = pmap;
+   gcv.ts_x_origin = 0;
+   gcv.ts_y_origin = 0;
+   tm = ECreatePixmap(disp, win, w, h, 1);
+   gc = XCreateGC(disp, tm, GCFillStyle | GCTile |
+		  GCTileStipXOrigin | GCTileStipYOrigin, &gcv);
+   XFillRectangle(disp, tm, gc, 0, 0, w, h);
+   XFreeGC(disp, gc);
+   EShapeCombineMask(disp, win, dest, x, y, tm, op);
+   EFreePixmap(disp, tm);
+
+   return;
+   d = NULL;
+}
+
 void
 EShapeCombineRectangles(Display * d, Window win, int dest, int x, int y,
 			XRectangle * rect, int n_rects, int op, int ordering)
@@ -531,16 +633,16 @@ EGetGeometry(Display * d, Window win, Window * root_return, int *x, int *y,
 	if (depth)
 	   *depth = xid->depth;
 	if (root_return)
-	   *root_return = root.win;
+	   *root_return = VRoot.win;
 	ok = 1;
      }
    else
      {
 	ok = XGetGeometry(d, win, root_return, x, y, w, h, bw, depth);
      }
-#if 0
+#if 0				/* Debug */
    if (!ok)
-      printf("EGetGeometry win=%#x, error %d\n", (unsigned)win, ok);
+      Eprintf("EGetGeometry win=%#x, error %d\n", (unsigned)win, ok);
 #endif
    return ok;
 }
@@ -633,10 +735,10 @@ ECreateEventWindow(Window parent, int x, int y, int w, int h)
    EDBUG(6, "ECreateEventWindow");
    attr.override_redirect = False;
    win =
-      XCreateWindow(disp, parent, x, y, w, h, 0, 0, InputOnly, root.vis,
+      XCreateWindow(disp, parent, x, y, w, h, 0, 0, InputOnly, VRoot.vis,
 		    CWOverrideRedirect, &attr);
 #if 0				/* Not yet */
-   SetXID(win, parent, x, y, w, h, root.depth);
+   SetXID(win, parent, x, y, w, h, VRoot.depth);
 #endif
 
    EDBUG_RETURN(win);
@@ -656,7 +758,7 @@ ECreateFocusWindow(Window parent, int x, int y, int w, int h)
 
    attr.backing_store = NotUseful;
    attr.override_redirect = True;
-   attr.colormap = root.cmap;
+   attr.colormap = VRoot.cmap;
    attr.border_pixel = 0;
    attr.background_pixel = 0;
    attr.save_under = False;
@@ -675,13 +777,29 @@ ECreateFocusWindow(Window parent, int x, int y, int w, int h)
    EDBUG_RETURN(win);
 }
 
+void
+ESetColor(XColor * pxc, int r, int g, int b)
+{
+   pxc->red = (r << 8) | r;
+   pxc->green = (g << 8) | g;
+   pxc->blue = (b << 8) | b;
+}
+
+void
+EGetColor(XColor * pxc, int *pr, int *pg, int *pb)
+{
+   *pr = pxc->red >> 8;
+   *pg = pxc->green >> 8;
+   *pb = pxc->blue >> 8;
+}
+
 void
 GrabX()
 {
    EDBUG(6, "GrabX");
-   if (mode.server_grabbed <= 0)
+   if (Mode.server_grabbed <= 0)
       XGrabServer(disp);
-   mode.server_grabbed++;
+   Mode.server_grabbed++;
    EDBUG_RETURN_;
 }
 
@@ -689,47 +807,14 @@ void
 UngrabX()
 {
    EDBUG(6, "UngrabX");
-   if (mode.server_grabbed == 1)
+   if (Mode.server_grabbed == 1)
      {
 	XUngrabServer(disp);
 	XFlush(disp);
      }
-   mode.server_grabbed--;
-   if (mode.server_grabbed < 0)
-      mode.server_grabbed = 0;
-   EDBUG_RETURN_;
-}
-
-void
-SetBG(Window win, Pixmap pmap, int color)
-{
-   static Atom         a = 0, aa = 0, aaa = 0;
-   static Window       alive_win = 0;
-
-   EDBUG(6, "SetBG");
-   if (!a)
-     {
-	a = XInternAtom(disp, "_XROOTPMAP_ID", False);
-	aa = XInternAtom(disp, "_XROOTCOLOR_PIXEL", False);
-	aaa = XInternAtom(disp, "_XROOTWINDOW", False);
-     }
-   if (!alive_win)
-     {
-	alive_win = ECreateWindow(root.win, -100, -100, 1, 1, 0);
-	XChangeProperty(disp, alive_win, aaa, XA_WINDOW, 32, PropModeReplace,
-			(unsigned char *)&alive_win, 1);
-	XChangeProperty(disp, root.win, aaa, XA_WINDOW, 32, PropModeReplace,
-			(unsigned char *)&alive_win, 1);
-     }
-   XChangeProperty(disp, win, a, XA_PIXMAP, 32, PropModeReplace,
-		   (unsigned char *)&pmap, 1);
-   XChangeProperty(disp, win, aa, XA_CARDINAL, 32, PropModeReplace,
-		   (unsigned char *)&color, 1);
-   if (pmap)
-      XSetWindowBackgroundPixmap(disp, win, pmap);
-   else
-      XSetWindowBackground(disp, win, color);
-   XClearWindow(disp, win);
+   Mode.server_grabbed--;
+   if (Mode.server_grabbed < 0)
+      Mode.server_grabbed = 0;
    EDBUG_RETURN_;
 }
 
@@ -744,6 +829,20 @@ GetWinXY(Window win, int *x, int *y)
    EDBUG_RETURN_;
 }
 
+Window
+GetWinParent(Window win)
+{
+   EXID               *xid;
+
+   EDBUG(7, "GetWinParent");
+
+   xid = FindXID(win);
+   if (xid)
+      return xid->parent;
+
+   EDBUG_RETURN(0);
+}
+
 void
 GetWinWH(Window win, unsigned int *w, unsigned int *h)
 {
@@ -840,10 +939,10 @@ WindowAtXY(int x, int y)
 
    EDBUG(7, "WindowAtXY");
    GrabX();
-   if (!XQueryTree(disp, root.win, &root_win, &parent_win, &list, &num))
+   if (!XQueryTree(disp, VRoot.win, &root_win, &parent_win, &list, &num))
      {
 	UngrabX();
-	EDBUG_RETURN(root.win);
+	EDBUG_RETURN(VRoot.win);
      }
    if (list)
      {
@@ -867,7 +966,7 @@ WindowAtXY(int x, int y)
 	XFree(list);
      }
    UngrabX();
-   EDBUG_RETURN(root.win);
+   EDBUG_RETURN(VRoot.win);
 }
 
 void
@@ -877,7 +976,7 @@ PointerAt(int *x, int *y)
    int                 dd;
    unsigned int        mm;
 
-   XQueryPointer(disp, root.win, &dw, &dw, &dd, &dd, x, y, &mm);
+   XQueryPointer(disp, VRoot.win, &dw, &dw, &dd, &dd, x, y, &mm);
 }
 
 void
@@ -898,12 +997,11 @@ PastePixmap(Display * d, Drawable w, Pixmap p, Mask m, int x, int y)
 void
 PasteMask(Display * d, Drawable w, Pixmap p, int x, int y, int wd, int ht)
 {
-   static GC           gc = 0;
+   GC                  gc;
    XGCValues           gcv;
    int                 ww, hh;
 
-   if (!gc)
-      gc = XCreateGC(d, w, 0, &gcv);
+   gc = XCreateGC(d, w, 0, &gcv);
    if (p)
      {
 	GetWinWH(p, (unsigned int *)&ww, (unsigned int *)&hh);
@@ -916,4 +1014,5 @@ PasteMask(Display * d, Drawable w, Pixmap p, int x, int y, int wd, int ht)
 	XSetForeground(disp, gc, 1);
 	XFillRectangle(disp, w, gc, x, y, wd, ht);
      }
+   XFreeGC(disp, gc);
 }
